@@ -1,5 +1,5 @@
 /* ldapcompare.c -- LDAP compare tool */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapcompare.c,v 1.25.2.7 2005/01/20 17:00:55 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapcompare.c,v 1.34.2.3 2005/05/06 16:10:21 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -46,6 +46,7 @@
 #include <ac/string.h>
 #include <ac/unistd.h>
 #include <ac/errno.h>
+#include <ac/time.h>
 #include <sys/stat.h>
 
 #ifdef HAVE_FCNTL_H
@@ -217,8 +218,8 @@ main( int argc, char **argv )
 
 	free( bvalue.bv_val );
 
-	ldap_unbind_ext( ld, NULL, NULL );
-
+	tool_unbind( ld );
+	tool_destroy();
 	return rc;
 }
 
@@ -232,27 +233,88 @@ static int docompare(
 	LDAPControl **sctrls,
 	LDAPControl **cctrls )
 {
-	int			rc;
+	int		rc, msgid, code;
+	LDAPMessage	*res;
+	char		*matcheddn;
+	char		*text;
+	char		**refs;
 
 	if ( not ) {
 		return LDAP_SUCCESS;
 	}
 
-	rc = ldap_compare_ext_s( ld, dn, attr, bvalue,
-		sctrls, cctrls );
+	rc = ldap_compare_ext( ld, dn, attr, bvalue,
+		sctrls, cctrls, &msgid );
+	if ( rc == -1 ) {
+		return( rc );
+	}
 
-	/* if we were told to be quiet, use the return value. */
-	if ( !quiet ) {
-		if ( rc == LDAP_COMPARE_TRUE ) {
-			printf(_("TRUE\n"));
-		} else if ( rc == LDAP_COMPARE_FALSE ) {
-			printf(_("FALSE\n"));
-		} else {
-			printf(_("UNDEFINED\n"));
-			ldap_perror( ld, "ldap_compare" );
+	for ( ; ; ) {
+		struct timeval	tv;
+
+		tv.tv_sec = 0;
+		tv.tv_usec = 100000;
+
+		if ( tool_check_abandon( ld, msgid ) ) {
+			return LDAP_CANCELLED;
+		}
+
+		rc = ldap_result( ld, LDAP_RES_ANY, LDAP_MSG_ALL, &tv, &res );
+		if ( rc < 0 ) {
+			ldap_perror( ld, "ldapcompare: ldap_result" );
+			return rc;
+		}
+
+		if ( rc != 0 ) {
+			break;
 		}
 	}
 
-	return( rc );
+	rc = ldap_parse_result( ld, res, &code, &matcheddn, &text, &refs, NULL, 1 );
+
+	if( rc != LDAP_SUCCESS ) {
+		fprintf( stderr, "%s: ldap_parse_result: %s (%d)\n",
+			prog, ldap_err2string( rc ), rc );
+		return rc;
+	}
+
+	if ( !quiet && ( verbose || ( code != LDAP_SUCCESS && code != LDAP_COMPARE_TRUE && code != LDAP_COMPARE_FALSE )||
+		(matcheddn && *matcheddn) || (text && *text) || (refs && *refs) ) )
+	{
+		printf( _("Compare Result: %s (%d)\n"),
+			ldap_err2string( code ), code );
+
+		if( text && *text ) {
+			printf( _("Additional info: %s\n"), text );
+		}
+
+		if( matcheddn && *matcheddn ) {
+			printf( _("Matched DN: %s\n"), matcheddn );
+		}
+
+		if( refs ) {
+			int i;
+			for( i=0; refs[i]; i++ ) {
+				printf(_("Referral: %s\n"), refs[i] );
+			}
+		}
+	}
+
+	ber_memfree( text );
+	ber_memfree( matcheddn );
+	ber_memvfree( (void **) refs );
+
+	/* if we were told to be quiet, use the return value. */
+	if ( !quiet ) {
+		if ( code == LDAP_COMPARE_TRUE ) {
+			printf(_("TRUE\n"));
+		} else if ( code == LDAP_COMPARE_FALSE ) {
+			printf(_("FALSE\n"));
+		} else {
+			printf(_("UNDEFINED\n"));
+		}
+	}
+
+	return( code );
 }
 
