@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/include/lber.h,v 1.83.2.11 2005/01/20 17:00:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/include/lber.h,v 1.94.2.2 2005/04/29 21:29:02 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -597,6 +597,10 @@ LBER_F( char * )
 ber_strdup LDAP_P((
 	LDAP_CONST char * ));
 
+LBER_F( struct berval * )
+ber_bvreplace LDAP_P((
+	struct berval *dst, LDAP_CONST struct berval *src ));
+
 LBER_F( void )
 ber_bvarray_free LDAP_P(( BerVarray p ));
 
