@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/include/ldap_log.h,v 1.29.2.4 2004/01/01 18:16:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/include/ldap_log.h,v 1.35.2.1 2005/01/20 18:03:48 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  * 
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,36 +31,6 @@
 
 LDAP_BEGIN_DECL
 
-/*
- * While it's not important that the subsystem number are
- * contiguous, it is important that the LDAP_SUBSYS_NUM accurately
- * reflect the number of subsystems and LDAP_SUBSYS_MAX reflect
- * the largest subsystem number.
- */
-#define LDAP_SUBSYS_NUM 20
-#define LDAP_SUBSYS_MAX 19
-
-#define LDAP_SUBSYS_GLOBAL      0
-#define LDAP_SUBSYS_OPERATION   1
-#define LDAP_SUBSYS_TRANSPORT   2
-#define LDAP_SUBSYS_CONNECTION  3
-#define LDAP_SUBSYS_FILTER      4
-#define LDAP_SUBSYS_BER         5
-#define LDAP_SUBSYS_CONFIG      6
-#define LDAP_SUBSYS_ACL         7
-#define LDAP_SUBSYS_CACHE       8
-#define LDAP_SUBSYS_INDEX      	9
-#define LDAP_SUBSYS_LDIF       	10
-#define LDAP_SUBSYS_TOOLS      	11
-#define LDAP_SUBSYS_SLAPD      	12
-#define LDAP_SUBSYS_SLURPD     	13
-#define LDAP_SUBSYS_BACKEND     14
-#define LDAP_SUBSYS_BACK_BDB 	15
-#define LDAP_SUBSYS_BACK_LDBM 	16
-#define LDAP_SUBSYS_BACK_LDAP 	17
-#define LDAP_SUBSYS_BACK_META 	18
-#define LDAP_SUBSYS_BACK_MON 	19
-
 /*
  * debug reporting levels.
  *
@@ -96,13 +66,13 @@ LDAP_BEGIN_DECL
 #define LDAP_DEBUG_PARSE	0x0800
 #define LDAP_DEBUG_CACHE    0x1000
 #define LDAP_DEBUG_INDEX    0x2000
+#define LDAP_DEBUG_SYNC		0x4000
 
 #define LDAP_DEBUG_NONE		0x8000
 #define LDAP_DEBUG_ANY		-1
 
 /* debugging stuff */
 #ifdef LDAP_DEBUG
-
     /*
      * This is a bogus extern declaration for the compiler. No need to ensure
      * a 'proper' dllimport.
@@ -114,6 +84,12 @@ LDAP_BEGIN_DECL
 #   ifdef LDAP_SYSLOG
     extern int	ldap_syslog;
     extern int	ldap_syslog_level;
+
+#	ifdef HAVE_EBCDIC
+#	define syslog	eb_syslog
+	extern void eb_syslog(int pri, const char *fmt, ...);
+#	endif
+
 #   endif /* LDAP_SYSLOG */
 
 /* this doesn't below as part of ldap.h */
@@ -139,42 +115,13 @@ LDAP_BEGIN_DECL
 
 #endif /* LDAP_DEBUG */
 
-#ifdef NEW_LOGGING
-extern int ldap_loglevels[LDAP_SUBSYS_NUM];
-
-#ifdef LDAP_DEBUG
-
-#define LDAP_LOG(a, b, fmt, arg1, arg2, arg3) do {\
-	if (ldap_loglevels[LDAP_SUBSYS_##a] >= LDAP_LEVEL_##b || \
-		ldap_loglevels[LDAP_SUBSYS_GLOBAL] >= LDAP_LEVEL_##b)\
-		   lutil_log (LDAP_SUBSYS_##a, LDAP_LEVEL_##b, fmt, arg1, arg2, arg3); \
-	} while (0)
-
-
-#define LDAP_LOGS_TEST(a, b) \
-	(ldap_loglevels[LDAP_SUBSYS_##a] >= LDAP_LEVEL_##b || \
-	 ldap_loglevels[LDAP_SUBSYS_GLOBAL] >= LDAP_LEVEL_##b)
-
-#endif /* LDAP_DEBUG */
-
-#endif /* NEW_LOGGING */
-
 #ifndef LDAP_LOG
 #define LDAP_LOG(a, b, fmt, arg1, arg2, arg3)
 #define LDAP_LOGS_TEST(a, b) 0
 #endif
 
-LDAP_LUTIL_F(int) lutil_mnem2level LDAP_P(( const char *level ));
-LDAP_LUTIL_F(void) lutil_log_initialize LDAP_P((
-	int argc, char **argv ));
-LDAP_LUTIL_F(int) lutil_set_debug_level LDAP_P((
-	const char *subsys, int level ));
-LDAP_LUTIL_F(void) lutil_log LDAP_P((
-	const int subsys, int level, const char *fmt, ... ));
-
 LDAP_LUTIL_F(int) lutil_debug_file LDAP_P(( FILE *file ));
 
-
 LDAP_LUTIL_F(void) lutil_debug LDAP_P((
 	int debug, int level,
 	const char* fmt, ... )) LDAP_GCCATTR((format(printf, 3, 4)));
