@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/init.c,v 1.82.2.6 2005/01/20 17:01:01 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/init.c,v 1.93.2.2 2005/04/29 21:29:03 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -23,7 +23,9 @@
 #include <ac/ctype.h>
 #include <ac/time.h>
 
+#ifdef HAVE_LIMITS_H
 #include <limits.h>
+#endif
 
 #include "ldap-int.h"
 #include "ldap_defaults.h"
@@ -98,6 +100,11 @@ static const struct ol_attribute {
   	{0, ATTR_TLS,	"TLS_REQCERT",		NULL,	LDAP_OPT_X_TLS_REQUIRE_CERT},
 	{0, ATTR_TLS,	"TLS_RANDFILE",		NULL,	LDAP_OPT_X_TLS_RANDOM_FILE},
 	{0, ATTR_TLS,	"TLS_CIPHER_SUITE",	NULL,	LDAP_OPT_X_TLS_CIPHER_SUITE},
+
+#ifdef HAVE_OPENSSL_CRL
+	{0, ATTR_TLS,	"TLS_CRLCHECK",		NULL,	LDAP_OPT_X_TLS_CRLCHECK},
+#endif
+        
 #endif
 
 	{0, ATTR_NONE,		NULL,		NULL,	0}
@@ -125,12 +132,7 @@ static void openldap_ldap_init_w_conf(
 		return;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( CONFIG, DETAIL1, 
-		"openldap_init_w_conf: trying %s\n", file, 0, 0 );
-#else
 	Debug(LDAP_DEBUG_TRACE, "ldap_init: trying %s\n", file, 0, 0);
-#endif
 
 	fp = fopen(file, "r");
 	if(fp == NULL) {
@@ -138,11 +140,7 @@ static void openldap_ldap_init_w_conf(
 		return;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( CONFIG, DETAIL1, "openldap_init_w_conf: using %s\n", file, 0, 0 );
-#else
 	Debug(LDAP_DEBUG_TRACE, "ldap_init: using %s\n", file, 0, 0);
-#endif
 
 	while((start = fgets(linebuf, sizeof(linebuf), fp)) != NULL) {
 		/* skip lines starting with '#' */
@@ -270,22 +268,12 @@ static void openldap_ldap_init_w_userconf(const char *file)
 	home = getenv("HOME");
 
 	if (home != NULL) {
-#ifdef NEW_LOGGING
-	LDAP_LOG ( CONFIG, ARGS, 
-		"openldap_init_w_userconf: HOME env is %s\n", home, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_TRACE, "ldap_init: HOME env is %s\n",
 		      home, 0, 0);
-#endif
 		path = LDAP_MALLOC(strlen(home) + strlen(file) + sizeof( LDAP_DIRSEP "."));
 	} else {
-#ifdef NEW_LOGGING
-	LDAP_LOG ( CONFIG, ARGS, "openldap_init_w_userconf: HOME env is NULL\n",
-		0, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_TRACE, "ldap_init: HOME env is NULL\n",
 		      0, 0, 0);
-#endif
 	}
 
 	if(home != NULL && path != NULL) {
@@ -478,6 +466,11 @@ void ldap_int_initialize_global_options( struct ldapoptions *gopts, int *dbglvl
 		SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS;
 #endif
 
+#ifdef HAVE_TLS
+	gopts->ldo_tls_connect_cb = NULL;
+	gopts->ldo_tls_connect_arg = NULL;
+#endif
+
 	gopts->ldo_valid = LDAP_INITIALIZED;
    	return;
 }
@@ -535,8 +528,10 @@ void ldap_int_initialize( struct ldapoptions *gopts, int *dbglvl )
 	|| defined(HAVE_TLS) || defined(HAVE_CYRUS_SASL)
 	ldap_int_hostname = ldap_pvt_get_fqdn( ldap_int_hostname );
 #endif
-	if ( ldap_int_tblsize == 0 )
-		ldap_int_ip_init();
+
+#ifndef HAVE_POLL
+	if ( ldap_int_tblsize == 0 ) ldap_int_ip_init();
+#endif
 
 	ldap_int_initialize_global_options(gopts, NULL);
 
@@ -565,50 +560,26 @@ void ldap_int_initialize( struct ldapoptions *gopts, int *dbglvl )
 		char *altfile = getenv(LDAP_ENV_PREFIX "CONF");
 
 		if( altfile != NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( CONFIG, DETAIL1, 
-				"openldap_init_w_userconf: %sCONF env is %s\n",
-				LDAP_ENV_PREFIX, altfile, 0 );
-#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is %s\n",
 			      LDAP_ENV_PREFIX "CONF", altfile, 0);
-#endif
 			openldap_ldap_init_w_sysconf( altfile );
 		}
 		else
-#ifdef NEW_LOGGING
-			LDAP_LOG ( CONFIG, DETAIL1, 
-				"openldap_init_w_userconf: %sCONF env is NULL\n",
-				LDAP_ENV_PREFIX, 0, 0 );
-#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is NULL\n",
 			      LDAP_ENV_PREFIX "CONF", 0, 0);
-#endif
 	}
 
 	{
 		char *altfile = getenv(LDAP_ENV_PREFIX "RC");
 
 		if( altfile != NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( CONFIG, DETAIL1, 
-				"openldap_init_w_userconf: %sRC env is %s\n",
-				LDAP_ENV_PREFIX, altfile, 0 );
-#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is %s\n",
 			      LDAP_ENV_PREFIX "RC", altfile, 0);
-#endif
 			openldap_ldap_init_w_userconf( altfile );
 		}
 		else
-#ifdef NEW_LOGGING
-			LDAP_LOG ( CONFIG, DETAIL1, 
-				"openldap_init_w_userconf: %sRC env is NULL\n",
-				LDAP_ENV_PREFIX, 0, 0 );
-#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is NULL\n",
 			      LDAP_ENV_PREFIX "RC", 0, 0);
-#endif
 	}
 
 	openldap_ldap_init_w_env(gopts, NULL);
