@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.91.2.5 2004/08/30 00:59:22 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.103.2.3 2005/06/03 19:12:01 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -95,11 +95,7 @@ ldap_send_initial_request(
 	LDAPURLDesc	*servers;
 	int rc;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_send_initial_request\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_send_initial_request\n", 0, 0, 0 );
-#endif
 
 	if ( ber_sockbuf_ctrl( ld->ld_sb, LBER_SB_OPT_GET_FD, NULL ) == -1 ) {
 		/* not connected yet */
@@ -110,15 +106,9 @@ ldap_send_initial_request(
 			return( -1 );
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"ldap_send_initial_request: ldap_open_defconn: successful\n" , 
-			0, 0, 0);
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_open_defconn: successful\n",
 			0, 0, 0 );
-#endif
 	}
 
 	{
@@ -158,8 +148,7 @@ ldap_send_initial_request(
 int
 ldap_int_flush_request(
 	LDAP *ld,
-	LDAPRequest *lr
-)
+	LDAPRequest *lr )
 {
 	LDAPConn *lc = lr->lr_conn;
 
@@ -202,11 +191,7 @@ ldap_send_server_request(
 	LDAPRequest	*lr;
 	int incparent, rc;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_send_server_request\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_send_server_request\n", 0, 0, 0 );
-#endif
 
 	incparent = 0;
 	ld->ld_errno = LDAP_SUCCESS;	/* optimistic */
@@ -306,11 +291,7 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 	LDAPURLDesc	*srv;
 	Sockbuf		*sb = NULL;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_new_connection\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_new_connection\n", 0, 0, 0 );
-#endif
 	/*
 	 * make a new LDAP server connection
 	 * XXX open connection synchronously for now
@@ -372,13 +353,7 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 				++lc->lconn_refcnt;	/* avoid premature free */
 				ld->ld_defconn = lc;
 
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, DETAIL1, 
-					"ldap_new_connection: Call application rebind_proc\n", 
-					0, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE, "Call application rebind_proc\n", 0, 0, 0);
-#endif
 #ifdef LDAP_R_COMPILE
 		ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
 		ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
@@ -406,13 +381,7 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 			++lc->lconn_refcnt;	/* avoid premature free */
 			ld->ld_defconn = lc;
 
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"ldap_new_connection: anonymous rebind via ldap_bind_s\n", 
-				0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "anonymous rebind via ldap_bind_s\n", 0, 0, 0);
-#endif
 #ifdef LDAP_R_COMPILE
 			ldap_pvt_thread_mutex_unlock( &ld->ld_req_mutex );
 			ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
@@ -491,11 +460,7 @@ ldap_free_connection( LDAP *ld, LDAPConn *lc, int force, int unbind )
 {
 	LDAPConn	*tmplc, *prevlc;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_free_connection\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_free_connection\n", 0, 0, 0 );
-#endif
 
 	if ( force || --lc->lconn_refcnt <= 0 ) {
 		if ( lc->lconn_status == LDAP_CONNST_CONNECTED ) {
@@ -541,22 +506,12 @@ ldap_free_connection( LDAP *ld, LDAPConn *lc, int force, int unbind )
 			LDAP_FREE( lc->lconn_rebind_queue);
 		}
 		LDAP_FREE( lc );
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, RESULTS, 
-			"ldap_free_connection: actually freed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "ldap_free_connection: actually freed\n",
 		    0, 0, 0 );
-#endif
 	} else {
 		lc->lconn_lastused = time( NULL );
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, RESULTS, "ldap_free_connection: refcnt %d\n", 
-			lc->lconn_refcnt, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "ldap_free_connection: refcnt %d\n",
 		    lc->lconn_refcnt, 0, 0 );
-#endif
 	}
 }
 
@@ -686,13 +641,8 @@ ldap_free_request( LDAP *ld, LDAPRequest *lr )
 {
 	LDAPRequest     **ttmplr;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ARGS, "ldap_free_request (origid %d, msgid %d)\n",
-		lr->lr_origid, lr->lr_msgid, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_free_request (origid %d, msgid %d)\n",
 		lr->lr_origid, lr->lr_msgid, 0 );
-#endif
 
 	/* free all referrals (child requests) */
 	while ( lr->lr_child )
@@ -707,6 +657,36 @@ ldap_free_request( LDAP *ld, LDAPRequest *lr )
 	ldap_free_request_int( ld, lr );
 }
 
+/*
+ * call first time with *cntp = -1
+ * when returns *cntp == -1, no referrals are left
+ *
+ * NOTE: may replace *refsp, or shuffle the contents
+ * of the original array.
+ */
+static int ldap_int_nextref(
+	LDAP			*ld,
+	char			***refsp,
+	int			*cntp,
+	void			*params )
+{
+	assert( refsp != NULL );
+	assert( *refsp != NULL );
+	assert( cntp != NULL );
+
+	if ( *cntp < -1 ) {
+		*cntp = -1;
+		return -1;
+	}
+
+	(*cntp)++;
+
+	if ( (*refsp)[ *cntp ] == NULL ) {
+		*cntp = -1;
+	}
+
+	return 0;
+}
 
 /*
  * Chase v3 referrals
@@ -738,11 +718,7 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 	ld->ld_errno = LDAP_SUCCESS;	/* optimistic */
 	*hadrefp = 0;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_chase_v3referrals\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_chase_v3referrals\n", 0, 0, 0 );
-#endif
 
 	unfollowed = NULL;
 	rc = count = 0;
@@ -755,14 +731,8 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 
 	/* Check for hop limit exceeded */
 	if ( lr->lr_parentcnt >= ld->ld_refhoplimit ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"ldap_chase_v3referrals: more than %d referral hops (dropping)\n",
-			ld->ld_refhoplimit, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "more than %d referral hops (dropping)\n", ld->ld_refhoplimit, 0, 0 );
-#endif
 		ld->ld_errno = LDAP_REFERRAL_LIMIT_EXCEEDED;
 	    rc = -1;
 		goto done;
@@ -778,8 +748,18 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 
 	refarray = refs;
 	refs = NULL;
+
+	if ( ld->ld_nextref_proc == NULL ) {
+		ld->ld_nextref_proc = ldap_int_nextref;
+	}
+
 	/* parse out & follow referrals */
-	for( i=0; refarray[i] != NULL; i++) {
+	i = -1;
+	for ( ld->ld_nextref_proc( ld, &refarray, &i, ld->ld_nextref_params );
+			i != -1;
+			ld->ld_nextref_proc( ld, &refarray, &i, ld->ld_nextref_params ) )
+	{
+
 		/* Parse the referral URL */
 		if (( rc = ldap_url_parse_ext( refarray[i], &srv)) != LDAP_SUCCESS) {
 			ld->ld_errno = rc;
@@ -810,15 +790,9 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 				 * if two search references come in one behind the other
 				 * for the same server with different contexts.
 				 */
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, DETAIL1, 
-					"ldap_chase_v3referrals: queue referral \"%s\"\n",
-					refarray[i], 0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 					"ldap_chase_v3referrals: queue referral \"%s\"\n",
 					refarray[i], 0, 0);
-#endif
 				if( lc->lconn_rebind_queue == NULL ) {
 					/* Create a referral list */
 					lc->lconn_rebind_queue =
@@ -881,15 +855,9 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 			goto done;
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"ldap_chase_v3referrals: msgid %d, url \"%s\"\n",
-			lr->lr_msgid, refarray[i], 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"ldap_chase_v3referral: msgid %d, url \"%s\"\n",
 			lr->lr_msgid, refarray[i], 0);
-#endif
 
 		/* Send the new request to the server - may require a bind */
 		rinfo.ri_msgid = origreq->lr_origid;
@@ -904,14 +872,8 @@ ldap_chase_v3referrals( LDAP *ld, LDAPRequest *lr, char **refs, int sref, char *
 #endif
 		if ( rc < 0 ) {
 			/* Failure, try next referral in the list */
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, 
-			"ldap_chase_v3referrals: Unable to chase referral \"%s\" (%s)\n",
-			refarray[i], ldap_err2string( ld->ld_errno ), 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "Unable to chase referral \"%s\" (%s)\n", 
 				refarray[i], ldap_err2string( ld->ld_errno ), 0);
-#endif
 			unfollowedcnt += ldap_append_referral( ld, &unfollowed, refarray[i]);
 			ldap_free_urllist(srv);
 			srv = NULL;
@@ -989,11 +951,7 @@ ldap_chase_referrals( LDAP *ld,
 	BerElement	*ber;
 	LDAPreqinfo  rinfo;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_chase_referrals\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_chase_referrals\n", 0, 0, 0 );
-#endif
 
 	ld->ld_errno = LDAP_SUCCESS;	/* optimistic */
 	*hadrefp = 0;
@@ -1016,15 +974,9 @@ ldap_chase_referrals( LDAP *ld,
 	}
 
 	if ( lr->lr_parentcnt >= ld->ld_refhoplimit ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ENTRY, 
-			"ldap_chase_referrals: more than %d referral hops (dropping)\n",
-			ld->ld_refhoplimit, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "more than %d referral hops (dropping)\n",
 		    ld->ld_refhoplimit, 0, 0 );
-#endif
 		    /* XXX report as error in ld->ld_errno? */
 		    return( 0 );
 	}
@@ -1049,14 +1001,8 @@ ldap_chase_referrals( LDAP *ld,
 		rc = ldap_url_parse_ext( ref, &srv );
 
 		if ( rc != LDAP_URL_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"ldap_chase_referrals: ignoring unknown referral <%s>\n",
-				ref, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 			    "ignoring unknown referral <%s>\n", ref, 0, 0 );
-#endif
 			rc = ldap_append_referral( ld, &unfollowed, ref );
 			*hadrefp = 1;
 			continue;
@@ -1067,13 +1013,8 @@ ldap_chase_referrals( LDAP *ld,
 			srv->lud_dn = NULL;
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"ldap_chase_referrals: chasing LDAP referral <%s>\n", ref, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 		    "chasing LDAP referral: <%s>\n", ref, 0, 0 );
-#endif
 
 		*hadrefp = 1;
 
@@ -1104,15 +1045,9 @@ ldap_chase_referrals( LDAP *ld,
 		if( rc >= 0 ) {
 			++count;
 		} else {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, 
-				"ldap_chase_referrals: Unable to chase referral <%s>\n", 
-				ldap_err2string( ld->ld_errno), 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			    "Unable to chase referral (%s)\n", 
 			    ldap_err2string( ld->ld_errno ), 0, 0 );
-#endif
 			rc = ldap_append_referral( ld, &unfollowed, ref );
 		}
 
@@ -1179,17 +1114,10 @@ re_encode_request( LDAP *ld,
 	char		*orig_dn;
 	char		*dn;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ARGS, 
-		"re_encode_request: new msgid %ld, new dn <%s>\n",
-		(long) msgid, 
-		( srv == NULL || srv->lud_dn == NULL ) ? "NONE" : srv->lud_dn, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE,
 	    "re_encode_request: new msgid %ld, new dn <%s>\n",
 	    (long) msgid,
 		( srv == NULL || srv->lud_dn == NULL) ? "NONE" : srv->lud_dn, 0 );
-#endif
 
 	tmpber = *origber;
 
@@ -1223,12 +1151,26 @@ re_encode_request( LDAP *ld,
 			/* use the scope provided in reference */
 			scope = srv->lud_scope;
 
-		} else if ( sref && scope != LDAP_SCOPE_SUBTREE ) {
-			/* use scope implied by previous operation */
-			/*   base -> base */
-			/*   one -> base */
-			/*   subtree -> subtree */
-			scope = LDAP_SCOPE_BASE;
+		} else if ( sref ) {
+			/* use scope implied by previous operation
+			 *   base -> base
+			 *   one -> base
+			 *   subtree -> subtree
+			 *   subordinate -> subtree
+			 */
+			switch( scope ) {
+			default:
+			case LDAP_SCOPE_BASE:
+			case LDAP_SCOPE_ONELEVEL:
+				scope = LDAP_SCOPE_BASE;
+				break;
+			case LDAP_SCOPE_SUBTREE:
+#ifdef LDAP_SCOPE_SUBORDINATE
+			case LDAP_SCOPE_SUBORDINATE:
+#endif
+				scope = LDAP_SCOPE_SUBTREE;
+				break;
+			}
 		}
 
 	} else {
@@ -1280,13 +1222,8 @@ re_encode_request( LDAP *ld,
 
 #ifdef LDAP_DEBUG
 	if ( ldap_debug & LDAP_DEBUG_PACKETS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"re_encode_request: new request is:\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "re_encode_request new request is:\n",
 		    0, 0, 0 );
-#endif
 		ber_log_dump( LDAP_DEBUG_BER, ldap_debug, ber, 0 );
 	}
 #endif /* LDAP_DEBUG */
