@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/unbind.c,v 1.44.2.5 2005/09/30 04:08:05 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/unbind.c,v 1.48.2.1 2005/01/20 18:03:51 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -67,11 +67,7 @@ ldap_unbind_ext_s(
 int
 ldap_unbind( LDAP *ld )
 {
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_unbind\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_unbind\n", 0, 0, 0 );
-#endif
 
 	return( ldap_unbind_ext( ld, NULL, NULL ) );
 }
@@ -144,13 +140,6 @@ ldap_ld_free(
 		ld->ld_options.ldo_defludp = NULL;
 	}
 
-#ifdef LDAP_CONNECTIONLESS
-	if ( ld->ld_options.ldo_peer != NULL ) {
-		LDAP_FREE( ld->ld_options.ldo_peer );
-		ld->ld_options.ldo_peer = NULL;
-	}
-#endif
-
 	if ( ld->ld_options.ldo_tm_api != NULL ) {
 		LDAP_FREE( ld->ld_options.ldo_tm_api );
 		ld->ld_options.ldo_tm_api = NULL;
@@ -183,16 +172,6 @@ ldap_ld_free(
 	}
 #endif
 
-	if ( ld->ld_options.ldo_sctrls != NULL ) {
-		ldap_controls_free( ld->ld_options.ldo_sctrls );
-		ld->ld_options.ldo_sctrls = NULL;
-	}
-
-	if ( ld->ld_options.ldo_cctrls != NULL ) {
-		ldap_controls_free( ld->ld_options.ldo_cctrls );
-		ld->ld_options.ldo_cctrls = NULL;
-	}
-
 	ber_sockbuf_free( ld->ld_sb );   
    
 #ifdef LDAP_R_COMPILE
@@ -221,11 +200,7 @@ ldap_send_unbind(
 	BerElement	*ber;
 	ber_int_t	id;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_send_unbind\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_send_unbind\n", 0, 0, 0 );
-#endif
 
 #ifdef LDAP_CONNECTIONLESS
 	if (LDAP_IS_UDP(ld))
