@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/librewrite/rewrite.c,v 1.6.2.4 2004/01/01 18:16:32 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/librewrite/rewrite.c,v 1.12.2.1 2005/01/20 18:03:53 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2004 The OpenLDAP Foundation.
+ * Copyright 2000-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,12 +30,13 @@
 #include <stdio.h>
 
 #include <rewrite.h>
+#include <ldap.h>
 
 int ldap_debug;
 int ldap_syslog;
 int ldap_syslog_level;
 
-char *
+static void
 apply( 
 		FILE *fin, 
 		const char *rewriteContext,
@@ -57,11 +58,12 @@ apply(
 
 	rewrite_session_init( info, cookie );
 
-	string = strdup( arg );
+	string = (char *)arg;
 	for ( sep = strchr( rewriteContext, ',' );
 			rewriteContext != NULL;
 			rewriteContext = sep,
-			sep ? sep = strchr( rewriteContext, ',' ) : NULL ) {
+			sep ? sep = strchr( rewriteContext, ',' ) : NULL )
+	{
 		char	*errmsg = "";
 
 		if ( sep != NULL ) {
@@ -104,33 +106,46 @@ apply(
 		if ( result == NULL ) {
 			break;
 		}
-		free( string );
+		if ( string != arg && string != result ) {
+			free( string );
+		}
 		string = result;
 	}
 
-	free( string );
+	if ( result && result != arg ) {
+		free( result );
+	}
 
 	rewrite_session_delete( info, cookie );
 
 	rewrite_info_delete( &info );
-
-	return result;
 }
 
 int
 main( int argc, char *argv[] )
 {
-	FILE *fin = NULL;
-	char *rewriteContext = REWRITE_DEFAULT_CONTEXT;
+	FILE	*fin = NULL;
+	char	*rewriteContext = REWRITE_DEFAULT_CONTEXT;
+	int	debug = 0;
+	char	*next;
 
 	while ( 1 ) {
-		int opt = getopt( argc, argv, "f:hr:" );
+		int opt = getopt( argc, argv, "d:f:hr:" );
 
 		if ( opt == EOF ) {
 			break;
 		}
 
 		switch ( opt ) {
+		case 'd':
+			debug = strtol( optarg, &next, 10 );
+			if ( next == NULL || next[0] != '\0' ) {
+				fprintf( stderr, "illegal log level '%s'\n",
+						optarg );
+				exit( EXIT_FAILURE );
+			}
+			break;
+
 		case 'f':
 			fin = fopen( optarg, "r" );
 			if ( fin == NULL ) {
@@ -161,6 +176,11 @@ main( int argc, char *argv[] )
 		}
 	}
 	
+	if ( debug != 0 ) {
+		ber_set_option(NULL, LBER_OPT_DEBUG_LEVEL, &debug);
+		ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &debug);
+	}
+	
 	if ( optind >= argc ) {
 		return -1;
 	}
