@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/librewrite/rule.c,v 1.5.4.9 2005/08/13 16:46:52 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/librewrite/rule.c,v 1.14.2.1 2005/01/20 18:03:53 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2005 The OpenLDAP Foundation.
@@ -346,14 +346,6 @@ rewrite_rule_compile(
 		return REWRITE_ERR;
 	}
 	
-#ifdef USE_REWRITE_LDAP_PVT_THREADS
-        if ( ldap_pvt_thread_mutex_init( &rule->lr_mutex ) ) {
-		regfree( &rule->lr_regex );
-		free( rule );
-		return REWRITE_ERR;
-	}
-#endif /* USE_REWRITE_LDAP_PVT_THREADS */
-
 	/*
 	 * Just to remember them ...
 	 */
@@ -426,17 +418,9 @@ recurse:;
 			rule->lr_pattern, string, strcnt + 1 );
 	
 	op->lo_num_passes++;
-#ifdef USE_REWRITE_LDAP_PVT_THREADS
-	ldap_pvt_thread_mutex_lock( &rule->lr_mutex );
-#endif /* USE_REWRITE_LDAP_PVT_THREADS */
-	rc = regexec( &rule->lr_regex, string, nmatch, match, 0 );
-#ifdef USE_REWRITE_LDAP_PVT_THREADS
-	ldap_pvt_thread_mutex_unlock( &rule->lr_mutex );
-#endif /* USE_REWRITE_LDAP_PVT_THREADS */
-	if ( rc != 0 ) {
-		if ( *result == NULL && strcnt > 0 ) {
+	if ( regexec( &rule->lr_regex, string, nmatch, match, 0 ) != 0 ) {
+		if ( *result == NULL && string != arg ) {
 			free( string );
-			string = NULL;
 		}
 
 		/*
@@ -450,7 +434,7 @@ recurse:;
 
 	*result = val.bv_val;
 	val.bv_val = NULL;
-	if ( strcnt > 0 ) {
+	if ( string != arg ) {
 		free( string );
 		string = NULL;
 	}
@@ -503,9 +487,6 @@ rewrite_rule_destroy(
 	}
 
 	regfree( &rule->lr_regex );
-#ifdef USE_REWRITE_LDAP_PVT_THREADS
-        ldap_pvt_thread_mutex_destroy( &rule->lr_mutex );
-#endif /* USE_REWRITE_LDAP_PVT_THREADS */
 
 	for ( action = rule->lr_action; action; ) {
 		struct rewrite_action *curraction = action;
