@@ -1,5 +1,5 @@
 /* ava.c - routines for dealing with attribute value assertions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/ava.c,v 1.34.2.5 2005/01/20 17:01:06 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/ava.c,v 1.40.2.3 2005/04/29 21:29:04 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -33,18 +33,22 @@
 
 #include "slap.h"
 
+#ifdef LDAP_COMP_MATCH
+#include "component.h"
+#endif
 
 void
 ava_free(
 	Operation *op,
 	AttributeAssertion *ava,
-	int	freeit
-)
+	int	freeit )
 {
+#ifdef LDAP_COMP_MATCH
+	if ( ava->aa_cf && ava->aa_cf->cf_ca->ca_comp_data.cd_mem_op )
+		nibble_mem_free ( ava->aa_cf->cf_ca->ca_comp_data.cd_mem_op );
+#endif
 	op->o_tmpfree( ava->aa_value.bv_val, op->o_tmpmemctx );
-	if ( freeit ) {
-		op->o_tmpfree( (char *) ava, op->o_tmpmemctx );
-	}
+	if ( freeit ) op->o_tmpfree( (char *) ava, op->o_tmpmemctx );
 }
 
 int
@@ -59,15 +63,14 @@ get_ava(
 	ber_tag_t rtag;
 	struct berval type, value;
 	AttributeAssertion *aa;
+#ifdef LDAP_COMP_MATCH
+	AttributeAliasing* a_alias = NULL;
+#endif
 
 	rtag = ber_scanf( ber, "{mm}", &type, &value );
 
 	if( rtag == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, ERR, "get_ava:  ber_scanf failure\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "  get_ava ber_scanf\n", 0, 0, 0 );
-#endif
 		*text = "Error decoding attribute value assertion";
 		return SLAPD_DISCONNECT;
 	}
@@ -75,17 +78,15 @@ get_ava(
 	aa = op->o_tmpalloc( sizeof( AttributeAssertion ), op->o_tmpmemctx );
 	aa->aa_desc = NULL;
 	aa->aa_value.bv_val = NULL;
+#ifdef LDAP_COMP_MATCH
+	aa->aa_cf = NULL;
+#endif
 
 	rc = slap_bv2ad( &type, &aa->aa_desc, text );
 
 	if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, ERR,
-		"get_ava: unknown attributeType %s\n", type.bv_val, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER,
 		"get_ava: unknown attributeType %s\n", type.bv_val, 0, 0 );
-#endif
 		op->o_tmpfree( aa, op->o_tmpmemctx );
 		return rc;
 	}
@@ -95,18 +96,25 @@ get_ava(
 		usage, &value, &aa->aa_value, text, op->o_tmpmemctx );
 
 	if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( FILTER, ERR,
-		"get_ava: illegal value for attributeType %s\n", type.bv_val, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_FILTER,
 		"get_ava: illegal value for attributeType %s\n", type.bv_val, 0, 0 );
-#endif
 		op->o_tmpfree( aa, op->o_tmpmemctx );
 		return rc;
 	}
 
+#ifdef LDAP_COMP_MATCH
+	if( is_aliased_attribute ) {
+		a_alias = is_aliased_attribute ( aa->aa_desc );
+		if ( a_alias ) {
+			rc = get_aliased_filter_aa ( op, aa, a_alias, text );
+			if( rc != LDAP_SUCCESS ) {
+				Debug( LDAP_DEBUG_FILTER,
+						"get_ava:Invalid Attribute Aliasing\n", 0, 0, 0 );
+				return rc;
+			}
+		}
+	}
+#endif
 	*ava = aa;
-
 	return LDAP_SUCCESS;
 }
