@@ -1,5 +1,5 @@
 /* index.c - routines for dealing with attribute indexes */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/index.c,v 1.38.2.10 2005/06/09 05:40:19 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/index.c,v 1.46.2.6 2005/05/18 16:29:22 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2005 The OpenLDAP Foundation.
@@ -28,19 +28,17 @@
 static char presence_keyval[LUTIL_HASH_BYTES] = {0,0,0,1};
 static struct berval presence_key = {LUTIL_HASH_BYTES, presence_keyval};
 
-static slap_mask_t index_mask(
+static AttrInfo *index_mask(
 	Backend *be,
 	AttributeDescription *desc,
 	struct berval *atname )
 {
 	AttributeType *at;
-	slap_mask_t mask = 0;
+	AttrInfo *ai = bdb_attr_mask( be->be_private, desc );
 
-	bdb_attr_mask( be->be_private, desc, &mask );
-
-	if( mask ) {
+	if( ai ) {
 		*atname = desc->ad_cname;
-		return mask;
+		return ai;
 	}
 
 	/* If there is a tagging option, did we ever index the base
@@ -48,11 +46,11 @@ static slap_mask_t index_mask(
 	 */
 	if( slap_ad_is_tagged( desc ) && desc != desc->ad_type->sat_ad ) {
 		/* has tagging option */
-		bdb_attr_mask( be->be_private, desc->ad_type->sat_ad, &mask );
+		ai = bdb_attr_mask( be->be_private, desc->ad_type->sat_ad );
 
-		if ( mask && ( mask ^ SLAP_INDEX_NOTAGS ) ) {
+		if ( ai && ( ai->ai_indexmask ^ SLAP_INDEX_NOTAGS ) ) {
 			*atname = desc->ad_type->sat_cname;
-			return mask;
+			return ai;
 		}
 	}
 
@@ -61,11 +59,11 @@ static slap_mask_t index_mask(
 		/* If no AD, we've never indexed this type */
 		if ( !at->sat_ad ) continue;
 
-		bdb_attr_mask( be->be_private, at->sat_ad, &mask );
+		ai = bdb_attr_mask( be->be_private, at->sat_ad );
 
-		if ( mask && ( mask ^ SLAP_INDEX_NOSUBTYPES ) ) {
+		if ( ai && ( ai->ai_indexmask ^ SLAP_INDEX_NOSUBTYPES ) ) {
 			*atname = at->sat_cname;
-			return mask;
+			return ai;
 		}
 	}
 
@@ -76,18 +74,19 @@ int bdb_index_is_indexed(
 	Backend *be,
 	AttributeDescription *desc )
 {
-	slap_mask_t mask;
+	AttrInfo *ai;
 	struct berval prefix;
 
-	mask = index_mask( be, desc, &prefix );
+	ai = index_mask( be, desc, &prefix );
 
-	if( mask == 0 ) {
+	if( !ai )
 		return LDAP_INAPPROPRIATE_MATCHING;
-	}
 
 	return LDAP_SUCCESS;
 }
 
+/* This function is only called when evaluating search filters.
+ */
 int bdb_index_param(
 	Backend *be,
 	AttributeDescription *desc,
@@ -96,15 +95,17 @@ int bdb_index_param(
 	slap_mask_t *maskp,
 	struct berval *prefixp )
 {
+	AttrInfo *ai;
 	int rc;
 	slap_mask_t mask;
 	DB *db;
 
-	mask = index_mask( be, desc, prefixp );
+	ai = index_mask( be, desc, prefixp );
 
-	if( mask == 0 ) {
+	if( !ai ) {
 		return LDAP_INAPPROPRIATE_MATCHING;
 	}
+	mask = ai->ai_indexmask;
 
 	rc = bdb_db_cache( be, prefixp->bv_val, &db );
 
@@ -166,7 +167,6 @@ static int indexer(
 	slap_mask_t mask )
 {
 	int rc, i;
-	const char *text;
 	DB *db;
 	struct berval *keys;
 
@@ -175,15 +175,9 @@ static int indexer(
 	rc = bdb_db_cache( op->o_bd, atname->bv_val, &db );
 	
 	if ( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( INDEX, ERR, 
-			"bdb_index_read: Could not open DB %s\n",
-			atname->bv_val, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"bdb_index_read: Could not open DB %s\n",
 			atname->bv_val, 0, 0 );
-#endif
 		return LDAP_OTHER;
 	}
 
@@ -285,6 +279,11 @@ static int index_at_values(
 {
 	int rc;
 	slap_mask_t mask = 0;
+	int ixop = opid;
+	AttrInfo *ai = NULL;
+
+	if ( opid == BDB_INDEX_UPDATE_OP )
+		ixop = SLAP_INDEX_ADD_OP;
 
 	if( type->sat_sup ) {
 		/* recurse */
@@ -297,35 +296,59 @@ static int index_at_values(
 
 	/* If this type has no AD, we've never used it before */
 	if( type->sat_ad ) {
-		bdb_attr_mask( op->o_bd->be_private, type->sat_ad, &mask );
-		ad = type->sat_ad;
-	}
+		ai = bdb_attr_mask( op->o_bd->be_private, type->sat_ad );
+		if ( ai ) {
+#ifdef LDAP_COMP_MATCH
+			/* component indexing */
+			if ( ai->ai_cr ) {
+				ComponentReference *cr;
+				for( cr = ai->ai_cr ; cr ; cr = cr->cr_next ) {
+					rc = indexer( op, txn, cr->cr_ad, &type->sat_cname,
+						cr->cr_nvals, id, ixop,
+						cr->cr_indexmask );
+				}
+			}
+#endif
+			ad = type->sat_ad;
+			/* If we're updating the index, just set the new bits that aren't
+			 * already in the old mask.
+			 */
+			if ( opid == BDB_INDEX_UPDATE_OP )
+				mask = ai->ai_newmask & ~ai->ai_indexmask;
+			else
+			/* For regular updates, if there is a newmask use it. Otherwise
+			 * just use the old mask.
+			 */
+				mask = ai->ai_newmask ? ai->ai_newmask : ai->ai_indexmask;
+			if( mask ) {
+				rc = indexer( op, txn, ad, &type->sat_cname,
+					vals, id, ixop, mask );
 
-	if( mask ) {
-		rc = indexer( op, txn, ad, &type->sat_cname,
-			vals, id, opid,
-			mask );
-
-		if( rc ) return rc;
+				if( rc ) return rc;
+			}
+		}
 	}
 
 	if( tags->bv_len ) {
 		AttributeDescription *desc;
 
-		mask = 0;
-
 		desc = ad_find_tags( type, tags );
 		if( desc ) {
-			bdb_attr_mask( op->o_bd->be_private, desc, &mask );
-		}
+			ai = bdb_attr_mask( op->o_bd->be_private, desc );
 
-		if( mask ) {
-			rc = indexer( op, txn, desc, &desc->ad_cname,
-				vals, id, opid,
-				mask );
+			if( ai ) {
+				if ( opid == BDB_INDEX_UPDATE_OP )
+					mask = ai->ai_newmask & ~ai->ai_indexmask;
+				else
+					mask = ai->ai_newmask ? ai->ai_newmask : ai->ai_indexmask;
+				if ( mask ) {
+					rc = indexer( op, txn, desc, &desc->ad_cname,
+						vals, id, ixop, mask );
 
-			if( rc ) {
-				return rc;
+					if( rc ) {
+						return rc;
+					}
+				}
 			}
 		}
 	}
@@ -343,6 +366,10 @@ int bdb_index_values(
 {
 	int rc;
 
+	/* Never index ID 0 */
+	if ( id == 0 )
+		return 0;
+
 	rc = index_at_values( op, txn, desc,
 		desc->ad_type, &desc->ad_tags,
 		vals, id, opid );
@@ -359,42 +386,94 @@ bdb_index_entry(
 {
 	int rc;
 	Attribute *ap = e->e_attrs;
-
-#ifdef NEW_LOGGING
-	LDAP_LOG( INDEX, ENTRY, "index_entry: %s (%s) %ld\n",
-		opid == SLAP_INDEX_ADD_OP ? "add" : "del", e->e_dn, (long) e->e_id );
-#else
-	Debug( LDAP_DEBUG_TRACE, "=> index_entry_%s( %ld, \"%s\" )\n",
-		opid == SLAP_INDEX_ADD_OP ? "add" : "del",
-		(long) e->e_id, e->e_dn );
+#ifdef LDAP_COMP_MATCH
+	ComponentReference *cr_list = NULL;
+	ComponentReference *cr = NULL, *dupped_cr = NULL;
+	void* decoded_comp, *extracted_comp;
+	ComponentSyntaxInfo* csi_attr;
+	Syntax* syn;
+	AttributeType* at;
+	int i, num_attr;
+	void* mem_op;
+	struct berval value = {0};
 #endif
 
+	Debug( LDAP_DEBUG_TRACE, "=> index_entry_%s( %ld, \"%s\" )\n",
+		opid == SLAP_INDEX_DELETE_OP ? "del" : "add",
+		(long) e->e_id, e->e_dn );
+
 	/* add each attribute to the indexes */
 	for ( ; ap != NULL; ap = ap->a_next ) {
+#ifdef LDAP_COMP_MATCH
+		AttrInfo *ai;
+		/* see if attribute has components to be indexed */
+		ai = bdb_attr_mask( op->o_bd->be_private, ap->a_desc->ad_type->sat_ad );
+		if ( ai ) cr_list = ai->ai_cr;
+		else cr_list = NULL;
+		if ( attr_converter && cr_list ) {
+			syn = ap->a_desc->ad_type->sat_syntax;
+			ap->a_comp_data = op->o_tmpalloc( sizeof( ComponentData ), op->o_tmpmemctx );
+                	/* Memory chunk(nibble) pre-allocation for decoders */
+                	mem_op = nibble_mem_allocator ( 1024*16, 1024*4 );
+			ap->a_comp_data->cd_mem_op = mem_op;
+			for( cr = cr_list ; cr ; cr = cr->cr_next ) {
+				/* count how many values in an attribute */
+				for( num_attr=0; ap->a_vals[num_attr].bv_val != NULL; num_attr++ );
+				num_attr++;
+				cr->cr_nvals = (BerVarray)op->o_tmpalloc( sizeof( struct berval )*num_attr, op->o_tmpmemctx );
+				for( i=0; ap->a_vals[i].bv_val != NULL; i++ ) {
+					/* decoding attribute value */
+					decoded_comp = attr_converter ( ap, syn, &ap->a_vals[i] );
+					if ( !decoded_comp )
+						return LDAP_DECODING_ERROR;
+					/* extracting the referenced component */
+					dupped_cr = dup_comp_ref( op, cr );
+					csi_attr = ((ComponentSyntaxInfo*)decoded_comp)->csi_comp_desc->cd_extract_i( mem_op, dupped_cr, decoded_comp );
+					if ( !csi_attr )
+						return LDAP_DECODING_ERROR;
+					cr->cr_asn_type_id = csi_attr->csi_comp_desc->cd_type_id;
+					cr->cr_ad = (AttributeDescription*)get_component_description ( cr->cr_asn_type_id );
+					if ( !cr->cr_ad )
+						return LDAP_INVALID_SYNTAX;
+					at = cr->cr_ad->ad_type;
+					/* encoding the value of component in GSER */
+					rc = component_encoder( mem_op, csi_attr, &value );
+					if ( rc != LDAP_SUCCESS )
+						return LDAP_ENCODING_ERROR;
+					/* Normalize the encoded component values */
+					if ( at->sat_equality && at->sat_equality->smr_normalize ) {
+						rc = at->sat_equality->smr_normalize (
+							SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+							at->sat_syntax, at->sat_equality,
+							&value, &cr->cr_nvals[i], op->o_tmpmemctx );
+					} else {
+						cr->cr_nvals[i] = value;
+					}
+				}
+				/* The end of BerVarray */
+				cr->cr_nvals[num_attr-1].bv_val = NULL;
+				cr->cr_nvals[num_attr-1].bv_len = 0;
+			}
+			op->o_tmpfree( ap->a_comp_data, op->o_tmpmemctx );
+			nibble_mem_free ( mem_op );
+			ap->a_comp_data = NULL;
+		}
+#endif
 		rc = bdb_index_values( op, txn, ap->a_desc,
 			ap->a_nvals, e->e_id, opid );
 
 		if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( INDEX, ENTRY, 
-				"index_entry: failure (%d)\n", rc, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"<= index_entry_%s( %ld, \"%s\" ) failure\n",
 				opid == SLAP_INDEX_ADD_OP ? "add" : "del",
 				(long) e->e_id, e->e_dn );
-#endif
 			return rc;
 		}
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( INDEX, ENTRY, "index_entry: success\n", 0, 0, 0  );
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= index_entry_%s( %ld, \"%s\" ) success\n",
-		opid == SLAP_INDEX_ADD_OP ? "add" : "del",
+		opid == SLAP_INDEX_DELETE_OP ? "del" : "add",
 		(long) e->e_id, e->e_dn );
-#endif
 
 	return LDAP_SUCCESS;
 }
