@@ -1,5 +1,5 @@
 /* tools.c - tools for slap tools */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/tools.c,v 1.53.2.10 2005/04/21 08:25:12 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/tools.c,v 1.72.2.5 2005/04/29 21:29:07 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2005 The OpenLDAP Foundation.
@@ -19,8 +19,8 @@
 #include <stdio.h>
 #include <ac/string.h>
 
+#define AVL_INTERNAL
 #include "back-bdb.h"
-#include "external.h"
 
 static DBC *cursor = NULL;
 static DBT key, data;
@@ -35,6 +35,8 @@ static dn_id hbuf[HOLE_SIZE], *holes = hbuf;
 static unsigned nhmax = HOLE_SIZE;
 static unsigned nholes;
 
+Avlnode *index_attrs, index_dummy;
+
 int bdb_tool_entry_open(
 	BackendDB *be, int mode )
 {
@@ -79,6 +81,8 @@ int bdb_tool_entry_close(
 	return 0;
 }
 
+static int bdb_reindex_cmp(const void *a, const void *b) { return 0; }
+
 ID bdb_tool_entry_next(
 	BackendDB *be )
 {
@@ -90,6 +94,7 @@ ID bdb_tool_entry_next(
 	assert( slapMode & SLAP_TOOL_MODE );
 	assert( bdb != NULL );
 	
+	/* Initialization */
 	if (cursor == NULL) {
 		rc = bdb->bi_id2entry->bdi_db->cursor(
 			bdb->bi_id2entry->bdi_db, NULL, &cursor,
@@ -102,14 +107,29 @@ ID bdb_tool_entry_next(
 	rc = cursor->c_get( cursor, &key, &data, DB_NEXT );
 
 	if( rc != 0 ) {
-		return NOID;
+		/* If we're doing linear indexing and there are more attrs to
+		 * index, and we're at the end of the database, start over.
+		 */
+		if ( bdb->bi_attrs == &index_dummy ) {
+			if ( index_attrs && rc == DB_NOTFOUND ) {
+				/* optional - do a checkpoint here? */
+				index_dummy.avl_data = avl_delete(&index_attrs, NULL, bdb_reindex_cmp);
+				rc = cursor->c_get( cursor, &key, &data, DB_FIRST );
+			}
+			if ( rc ) {
+				bdb->bi_attrs = NULL;
+				return NOID;
+			}
+		} else {
+			return NOID;
+		}
 	}
 
 	if( data.data == NULL ) {
 		return NOID;
 	}
 
-	AC_MEMCPY( &id, key.data, key.size );
+	BDB_DISK2ID( key.data, &id );
 	return id;
 }
 
@@ -118,34 +138,23 @@ ID bdb_tool_dn2id_get(
 	struct berval *dn
 )
 {
-	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
-	DB *db = bdb->bi_dn2id->bdi_db;
-	int rc;
-	DBT	key, data;
-	ID	id;
+	Operation op = {0};
+	Opheader ohdr = {0};
+	EntryInfo ei = {0};
 
-	DBTzero( &key );
-	key.size = dn->bv_len + 2;
-	key.data = ch_malloc( key.size );
-	((char*)key.data)[0] = DN_BASE_PREFIX;
-	AC_MEMCPY( &((char*)key.data)[1], dn->bv_val, key.size - 1 );
+	if ( BER_BVISEMPTY(dn) )
+		return 0;
 
-	DBTzero( &data );
-	data.data = &id;
-	data.ulen = sizeof(ID);
-	data.flags = DB_DBT_USERMEM;
+	op.o_hdr = &ohdr;
+	op.o_bd = be;
+	op.o_tmpmemctx = NULL;
+	op.o_tmpmfuncs = &ch_mfuncs;
 
-	rc = db->get( db, NULL, &key, &data, bdb->bi_db_opflags );
+	ei.bei_id = NOID;
 
-    if( rc != 0 ) {
-		Debug( LDAP_DEBUG_TRACE, LDAP_XSTRING(bdb_tool_dn2id_get)
-				": get failed: %s (%d)\n",
-				db_strerror( rc ), rc, 0 );
-		id = NOID;
-	}
-
-	ch_free( key.data );
-	return id;
+	bdb_dn2id( &op, NULL, dn, &ei );
+	
+	return ei.bei_id;
 }
 
 int bdb_tool_id2entry_get(
@@ -154,7 +163,20 @@ int bdb_tool_id2entry_get(
 	Entry **e
 )
 {
-	return bdb_id2entry( be, NULL, id, e );
+	int rc = bdb_id2entry( be, NULL, id, e );
+
+	if ( rc == DB_NOTFOUND && id == 0 ) {
+		Entry *dummy = ch_calloc( 1, sizeof(Entry) );
+		struct berval gluebv = BER_BVC("glue");
+		dummy->e_name.bv_val = ch_strdup( "" );
+		dummy->e_nname.bv_val = ch_strdup( "" );
+		attr_merge_one( dummy, slap_schema.si_ad_objectClass, &gluebv, NULL );
+		attr_merge_one( dummy, slap_schema.si_ad_structuralObjectClass,
+			&gluebv, NULL );
+		*e = dummy;
+		rc = LDAP_SUCCESS;
+	}
+	return rc;
 }
 
 Entry* bdb_tool_entry_get( BackendDB *be, ID id )
@@ -170,7 +192,12 @@ Entry* bdb_tool_entry_get( BackendDB *be, ID id )
 #ifndef BDB_HIER
 	DBT2bv( &data, &bv );
 
+#ifdef SLAP_ZONE_ALLOC
+	/* FIXME: will add ctx later */
+	rc = entry_decode( &bv, &e, NULL );
+#else
 	rc = entry_decode( &bv, &e );
+#endif
 
 	if( rc == LDAP_SUCCESS ) {
 		e->e_id = id;
@@ -179,7 +206,9 @@ Entry* bdb_tool_entry_get( BackendDB *be, ID id )
 	{
 		EntryInfo *ei = NULL;
 		Operation op = {0};
+		Opheader ohdr = {0};
 
+		op.o_hdr = &ohdr;
 		op.o_bd = be;
 		op.o_tmpmemctx = NULL;
 		op.o_tmpmfuncs = &ch_mfuncs;
@@ -199,14 +228,16 @@ static int bdb_tool_next_id(
 	struct berval *text,
 	int hole )
 {
-	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	struct berval dn = e->e_name;
 	struct berval ndn = e->e_nname;
 	struct berval pdn, npdn;
 	EntryInfo *ei = NULL, eidummy;
 	int rc;
 
-	if (ndn.bv_len == 0) return 0;
+	if (ndn.bv_len == 0) {
+		e->e_id = 0;
+		return 0;
+	}
 
 	rc = bdb_cache_find_ndn( op, tid, &ndn, &ei );
 	if ( ei ) bdb_cache_entryinfo_unlock( ei );
@@ -291,6 +322,7 @@ ID bdb_tool_entry_put(
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 	DB_TXN *tid = NULL;
 	Operation op = {0};
+	Opheader ohdr = {0};
 
 	assert( be != NULL );
 	assert( slapMode & SLAP_TOOL_MODE );
@@ -302,6 +334,7 @@ ID bdb_tool_entry_put(
 	Debug( LDAP_DEBUG_TRACE, "=> " LDAP_XSTRING(bdb_tool_entry_put)
 		"( %ld, \"%s\" )\n", (long) e->e_id, e->e_dn, 0 );
 
+	if (! (slapMode & SLAP_TOOL_QUICK)) {
 	rc = TXN_BEGIN( bdb->bi_dbenv, NULL, &tid, 
 		bdb->bi_db_opflags );
 	if( rc != 0 ) {
@@ -313,7 +346,9 @@ ID bdb_tool_entry_put(
 			 text->bv_val, 0, 0 );
 		return NOID;
 	}
+	}
 
+	op.o_hdr = &ohdr;
 	op.o_bd = be;
 	op.o_tmpmemctx = NULL;
 	op.o_tmpmfuncs = &ch_mfuncs;
@@ -336,7 +371,8 @@ ID bdb_tool_entry_put(
 		goto done;
 	}
 
-	rc = bdb_index_entry_add( &op, tid, e );
+	if ( !bdb->bi_linear_index )
+		rc = bdb_index_entry_add( &op, tid, e );
 	if( rc != 0 ) {
 		snprintf( text->bv_val, text->bv_len,
 				"index_entry_add failed: %s (%d)",
@@ -349,6 +385,7 @@ ID bdb_tool_entry_put(
 
 done:
 	if( rc == 0 ) {
+		if ( !( slapMode & SLAP_TOOL_QUICK )) {
 		rc = TXN_COMMIT( tid, 0 );
 		if( rc != 0 ) {
 			snprintf( text->bv_val, text->bv_len,
@@ -359,8 +396,10 @@ done:
 				text->bv_val, 0, 0 );
 			e->e_id = NOID;
 		}
+		}
 
 	} else {
+		if ( !( slapMode & SLAP_TOOL_QUICK )) {
 		TXN_ABORT( tid );
 		snprintf( text->bv_val, text->bv_len,
 			"txn_aborted! %s (%d)",
@@ -368,6 +407,7 @@ done:
 		Debug( LDAP_DEBUG_ANY,
 			"=> " LDAP_XSTRING(bdb_tool_entry_put) ": %s\n",
 			text->bv_val, 0, 0 );
+		}
 		e->e_id = NOID;
 	}
 
@@ -383,11 +423,26 @@ int bdb_tool_entry_reindex(
 	Entry *e;
 	DB_TXN *tid = NULL;
 	Operation op = {0};
+	Opheader ohdr = {0};
 
 	Debug( LDAP_DEBUG_ARGS,
 		"=> " LDAP_XSTRING(bdb_tool_entry_reindex) "( %ld )\n",
 		(long) id, 0, 0 );
 
+	/* No indexes configured, nothing to do. Could return an
+	 * error here to shortcut things.
+	 */
+	if (!bi->bi_attrs) {
+		return 0;
+	}
+
+	/* Get the first attribute to index */
+	if (bi->bi_linear_index && !index_attrs && bi->bi_attrs != &index_dummy) {
+		index_attrs = bi->bi_attrs;
+		bi->bi_attrs = &index_dummy;
+		index_dummy.avl_data = avl_delete(&index_attrs, NULL, bdb_reindex_cmp);
+	}
+
 	e = bdb_tool_entry_get( be, id );
 
 	if( e == NULL ) {
@@ -398,6 +453,7 @@ int bdb_tool_entry_reindex(
 		return -1;
 	}
 
+	if (! (slapMode & SLAP_TOOL_QUICK)) {
 	rc = TXN_BEGIN( bi->bi_dbenv, NULL, &tid, bi->bi_db_opflags );
 	if( rc != 0 ) {
 		Debug( LDAP_DEBUG_ANY,
@@ -406,6 +462,7 @@ int bdb_tool_entry_reindex(
 			db_strerror(rc), rc, 0 );
 		goto done;
 	}
+	}
  	
 	/*
 	 * just (re)add them for now
@@ -418,26 +475,16 @@ int bdb_tool_entry_reindex(
 		"=> " LDAP_XSTRING(bdb_tool_entry_reindex) "( %ld, \"%s\" )\n",
 		(long) id, e->e_dn, 0 );
 
+	op.o_hdr = &ohdr;
 	op.o_bd = be;
 	op.o_tmpmemctx = NULL;
 	op.o_tmpmfuncs = &ch_mfuncs;
 
-#ifndef BDB_HIER
-	/* add dn2id indices */
-	rc = bdb_dn2id_add( &op, tid, NULL, e );
-	if( rc != 0 && rc != DB_KEYEXIST ) {
-		Debug( LDAP_DEBUG_ANY,
-			"=> " LDAP_XSTRING(bdb_tool_entry_reindex)
-			": dn2id_add failed: %s (%d)\n",
-			db_strerror(rc), rc, 0 );
-		goto done;
-	}
-#endif
-
 	rc = bdb_index_entry_add( &op, tid, e );
 
 done:
 	if( rc == 0 ) {
+		if (! (slapMode & SLAP_TOOL_QUICK)) {
 		rc = TXN_COMMIT( tid, 0 );
 		if( rc != 0 ) {
 			Debug( LDAP_DEBUG_ANY,
@@ -446,13 +493,16 @@ done:
 				db_strerror(rc), rc, 0 );
 			e->e_id = NOID;
 		}
+		}
 
 	} else {
+		if (! (slapMode & SLAP_TOOL_QUICK)) {
 		TXN_ABORT( tid );
 		Debug( LDAP_DEBUG_ANY,
 			"=> " LDAP_XSTRING(bdb_tool_entry_reindex)
 			": txn_aborted! %s (%d)\n",
 			db_strerror(rc), rc, 0 );
+		}
 		e->e_id = NOID;
 	}
 	bdb_entry_release( &op, e, 0 );
@@ -469,6 +519,7 @@ ID bdb_tool_entry_modify(
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 	DB_TXN *tid = NULL;
 	Operation op = {0};
+	Opheader ohdr = {0};
 
 	assert( be != NULL );
 	assert( slapMode & SLAP_TOOL_MODE );
@@ -478,12 +529,12 @@ ID bdb_tool_entry_modify(
 	assert( text->bv_val[0] == '\0' );	/* overconservative? */
 
 	assert ( e->e_id != NOID );
-	assert ( e->e_id != 0 );
 
 	Debug( LDAP_DEBUG_TRACE,
 		"=> " LDAP_XSTRING(bdb_tool_entry_modify) "( %ld, \"%s\" )\n",
 		(long) e->e_id, e->e_dn, 0 );
 
+	if (! (slapMode & SLAP_TOOL_QUICK)) {
 	rc = TXN_BEGIN( bdb->bi_dbenv, NULL, &tid, 
 		bdb->bi_db_opflags );
 	if( rc != 0 ) {
@@ -495,7 +546,9 @@ ID bdb_tool_entry_modify(
 			 text->bv_val, 0, 0 );
 		return NOID;
 	}
+	}
 
+	op.o_hdr = &ohdr;
 	op.o_bd = be;
 	op.o_tmpmemctx = NULL;
 	op.o_tmpmfuncs = &ch_mfuncs;
@@ -512,6 +565,10 @@ ID bdb_tool_entry_modify(
 		goto done;
 	}
 
+#if 0
+	/* FIXME: this is bogus, we don't have the old values to delete
+	 * from the index because the given entry has already been modified.
+	 */
 	rc = bdb_index_entry_del( &op, tid, e );
 	if( rc != 0 ) {
 		snprintf( text->bv_val, text->bv_len,
@@ -522,6 +579,7 @@ ID bdb_tool_entry_modify(
 			text->bv_val, 0, 0 );
 		goto done;
 	}
+#endif
 
 	rc = bdb_index_entry_add( &op, tid, e );
 	if( rc != 0 ) {
@@ -536,6 +594,7 @@ ID bdb_tool_entry_modify(
 
 done:
 	if( rc == 0 ) {
+		if (! (slapMode & SLAP_TOOL_QUICK)) {
 		rc = TXN_COMMIT( tid, 0 );
 		if( rc != 0 ) {
 			snprintf( text->bv_val, text->bv_len,
@@ -546,8 +605,10 @@ done:
 				"%s\n", text->bv_val, 0, 0 );
 			e->e_id = NOID;
 		}
+		}
 
 	} else {
+		if (! (slapMode & SLAP_TOOL_QUICK)) {
 		TXN_ABORT( tid );
 		snprintf( text->bv_val, text->bv_len,
 			"txn_aborted! %s (%d)",
@@ -555,6 +616,7 @@ done:
 		Debug( LDAP_DEBUG_ANY,
 			"=> " LDAP_XSTRING(bdb_tool_entry_modify) ": %s\n",
 			text->bv_val, 0, 0 );
+		}
 		e->e_id = NOID;
 	}
 
