@@ -1,5 +1,5 @@
 /* unbind.c - ldap backend unbind function */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/unbind.c,v 1.18.2.4 2005/01/20 17:01:12 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/unbind.c,v 1.24.2.1 2005/01/20 18:03:59 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2005 The OpenLDAP Foundation.
@@ -33,46 +33,28 @@
 
 int
 ldap_back_conn_destroy(
-    Backend		*be,
-    Connection		*conn
+		Backend		*be,
+		Connection	*conn
 )
 {
 	struct ldapinfo	*li = (struct ldapinfo *) be->be_private;
-	struct ldapconn *lc, lc_curr;
+	struct ldapconn *lc = NULL, lc_curr;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDAP, INFO,
-		"ldap_back_conn_destroy: fetching conn %ld\n", conn->c_connid, 0, 0 );
-#else /* !NEW_LOGGING */
 	Debug( LDAP_DEBUG_TRACE,
 		"=>ldap_back_conn_destroy: fetching conn %ld\n",
 		conn->c_connid, 0, 0 );
-#endif /* !NEW_LOGGING */
 
-	lc_curr.conn = conn;
-	lc_curr.local_dn = conn->c_ndn;
+	lc_curr.lc_conn = conn;
+	lc_curr.lc_local_ndn = conn->c_ndn;
 	
 	ldap_pvt_thread_mutex_lock( &li->conn_mutex );
 	lc = avl_delete( &li->conntree, (caddr_t)&lc_curr, ldap_back_conn_cmp );
 	ldap_pvt_thread_mutex_unlock( &li->conn_mutex );
 
-	if (lc) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDAP, DETAIL1, 
-			"ldap_back_conn_destroy: destroying conn %ld\n", 
-			conn->c_connid, 0, 0 );
-#else /* !NEW_LOGGING */
+	if ( lc ) {
 		Debug( LDAP_DEBUG_TRACE,
 			"=>ldap_back_conn_destroy: destroying conn %ld\n",
-			lc->conn->c_connid, 0, 0 );
-#endif
-
-#ifdef ENABLE_REWRITE
-		/*
-		 * Cleanup rewrite session
-		 */
-		rewrite_session_delete( li->rwmap.rwm_rw, conn );
-#endif /* ENABLE_REWRITE */
+			lc->lc_conn->c_connid, 0, 0 );
 
 		/*
 		 * Needs a test because the handler may be corrupted,
