@@ -1,5 +1,5 @@
 /* close.c - close ldbm backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/close.c,v 1.16.2.4 2005/04/21 02:41:24 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/close.c,v 1.19.2.2 2005/04/29 21:29:07 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -27,20 +27,11 @@ int
 ldbm_back_db_close( Backend *be )
 {
 	struct ldbminfo *li = (struct ldbminfo *) be->be_private;
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, CRIT,
-		   "ldbm_back_db_close: ldbm backend syncing\n", 0, 0, 0 );
-#else
+
 	Debug( LDAP_DEBUG_TRACE, "ldbm backend syncing\n", 0, 0, 0 );
-#endif
 
 	ldbm_cache_flush_all( be );
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, CRIT,
-		   "ldbm_back_db_close: ldbm backend synch'ed\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldbm backend done syncing\n", 0, 0, 0 );
-#endif
 
 	cache_release_all( &((struct ldbminfo *) be->be_private)->li_cache );
 
