@@ -1,5 +1,5 @@
 /* ldbmcache.c - maintain a cache of open ldbm files */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/dbcache.c,v 1.52.2.4 2005/04/21 02:41:24 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/dbcache.c,v 1.57.2.2 2005/04/29 21:29:08 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -63,13 +63,8 @@ ldbm_cache_open(
 		flags |= LDBM_NOSYNC;
 	}
 	
-#ifdef NEW_LOGGING
-	LDAP_LOG( CACHE, ENTRY, 
-		"ldbm_cache_open: \"%s\", %d, %o\n", buf, flags, li->li_mode );
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> ldbm_cache_open( \"%s\", %d, %o )\n", buf,
 	    flags, li->li_mode );
-#endif
 
 
 	empty = MAXDBCACHE;
@@ -110,13 +105,8 @@ ldbm_cache_open(
 					break;
 				}
 				li->li_dbcache[i].dbc_refcnt++;
-#ifdef NEW_LOGGING
-				LDAP_LOG( CACHE, DETAIL1, 
-					"ldbm_cache_open: cache %d\n", i, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 				    "<= ldbm_cache_open (cache %d)\n", i, 0, 0 );
-#endif
 
 				ldap_pvt_thread_mutex_unlock( &li->li_dbcache_mutex );
 				return( &li->li_dbcache[i] );
@@ -141,15 +131,9 @@ ldbm_cache_open(
 				free( li->li_dbcache[i].dbc_name );
 				li->li_dbcache[i].dbc_name = NULL;
 			} else {
-#ifdef NEW_LOGGING
-				LDAP_LOG( CACHE, INFO,
-					"ldbm_cache_open: no unused db to close - waiting\n", 
-					0, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 				    "ldbm_cache_open no unused db to close - waiting\n",
 				    0, 0, 0 );
-#endif
 
 				ldap_pvt_thread_cond_wait( &li->li_dbcache_cv,
 					    &li->li_dbcache_mutex );
@@ -165,17 +149,10 @@ ldbm_cache_open(
 	    li->li_dbcachesize )) == NULL )
 	{
 		int err = errno;
-#ifdef NEW_LOGGING
-		LDAP_LOG( CACHE, ERR, 
-			"ldbm_cache_open: \"%s\" failed, errono=%d, reason=%s\n",
-			buf, err, err > -1 && err < sys_nerr ? sys_errlist[err] :
-			"unknown" );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 		    "<= ldbm_cache_open NULL \"%s\" errno=%d reason=\"%s\")\n",
 		    buf, err, err > -1 && err < sys_nerr ?
 		    sys_errlist[err] : "unknown" );
-#endif
 
 		ldap_pvt_thread_mutex_unlock( &li->li_dbcache_mutex );
 		return( NULL );
@@ -200,23 +177,12 @@ ldbm_cache_open(
 
 	assert( li->li_dbcache[i].dbc_maxindirect < 256 );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( CACHE, ARGS, 
-		   "ldbm_cache_open: blksize:%ld  maxids:%d  maxindirect:%d\n",
-		   li->li_dbcache[i].dbc_blksize, li->li_dbcache[i].dbc_maxids,
-		   li->li_dbcache[i].dbc_maxindirect );
-#else
 	Debug( LDAP_DEBUG_ARGS,
 	    "ldbm_cache_open (blksize %ld) (maxids %d) (maxindirect %d)\n",
 	    li->li_dbcache[i].dbc_blksize, li->li_dbcache[i].dbc_maxids,
 	    li->li_dbcache[i].dbc_maxindirect );
-#endif
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( CACHE, DETAIL1, "<= ldbm_cache_open: (opened %d)\n", i, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= ldbm_cache_open (opened %d)\n", i, 0, 0 );
-#endif
 
 	ldap_pvt_thread_mutex_init( &li->li_dbcache[i].dbc_write_mutex );
 
@@ -268,39 +234,21 @@ ldbm_cache_flush_all( Backend *be )
 	ldap_pvt_thread_mutex_lock( &li->li_dbcache_mutex );
 	for ( i = 0; i < MAXDBCACHE; i++ ) {
 		if ( li->li_dbcache[i].dbc_name != NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( CACHE, DETAIL1, 
-				   "ldbm_cache_flush_all: flushing db (%s)\n",
-				   li->li_dbcache[i].dbc_name, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "ldbm flushing db (%s)\n",
 			    li->li_dbcache[i].dbc_name, 0, 0 );
-#endif
 
 			ldbm_sync( li->li_dbcache[i].dbc_db );
 			li->li_dbcache[i].dbc_dirty = 0;
 			if ( li->li_dbcache[i].dbc_refcnt != 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( CACHE, INFO, 
-					"ldbm_cache_flush_all: couldn't close db (%s), refcnt=%d\n",
-					li->li_dbcache[i].dbc_name, li->li_dbcache[i].dbc_refcnt,0);
-#else
 				Debug( LDAP_DEBUG_TRACE,
 				       "refcnt = %d, couldn't close db (%s)\n",
 				       li->li_dbcache[i].dbc_refcnt,
 				       li->li_dbcache[i].dbc_name, 0 );
-#endif
 
 			} else {
-#ifdef NEW_LOGGING
-				LDAP_LOG( CACHE, DETAIL1, 
-					   "ldbm_cache_flush_all: ldbm closing db (%s)\n",
-					   li->li_dbcache[i].dbc_name, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 				       "ldbm closing db (%s)\n",
 				       li->li_dbcache[i].dbc_name, 0, 0 );
-#endif
 
 				ldap_pvt_thread_cond_signal( &li->li_dbcache_cv );
 				ldbm_close( li->li_dbcache[i].dbc_db );
@@ -322,15 +270,6 @@ ldbm_cache_sync( Backend *be )
 	ldap_pvt_thread_mutex_lock( &li->li_dbcache_mutex );
 	for ( i = 0; i < MAXDBCACHE; i++ ) {
 		if ( li->li_dbcache[i].dbc_name != NULL && li->li_dbcache[i].dbc_dirty ) {
-#ifdef NEW_LOGGING
-			if ( do_log ) {
-				do_log = 0;
-				LDAP_LOG( CACHE, ARGS, "syncing %s\n",
-					li->li_directory, 0, 0 );
-			}
-			LDAP_LOG ( CACHE, DETAIL1, "ldbm_cache_sync: "
-				"ldbm syncing db (%s)\n", li->li_dbcache[i].dbc_name, 0, 0 );
-#else
 			if ( do_log ) {
 				do_log = 0;
 				Debug( LDAP_DEBUG_TRACE, "syncing %s\n",
@@ -338,7 +277,6 @@ ldbm_cache_sync( Backend *be )
 			}
 			Debug(	LDAP_DEBUG_TRACE, "ldbm syncing db (%s)\n",
 				li->li_dbcache[i].dbc_name, 0, 0 );
-#endif
 			ldbm_sync( li->li_dbcache[i].dbc_db );
 			li->li_dbcache[i].dbc_dirty = 0;
 		}
@@ -410,9 +348,9 @@ ldbm_cache_sync_daemon(
 			li->li_dbsyncwaitcount, li->li_directory, 0 );
 	}
 
-	ldap_pvt_thread_mutex_lock( &syncrepl_rq.rq_mutex );
-	ldap_pvt_runqueue_stoptask( &syncrepl_rq, rtask );
-	ldap_pvt_runqueue_resched( &syncrepl_rq, rtask, 0 );
-	ldap_pvt_thread_mutex_unlock( &syncrepl_rq.rq_mutex );
+	ldap_pvt_thread_mutex_lock( &slapd_rq.rq_mutex );
+	ldap_pvt_runqueue_stoptask( &slapd_rq, rtask );
+	ldap_pvt_runqueue_resched( &slapd_rq, rtask, 0 );
+	ldap_pvt_thread_mutex_unlock( &slapd_rq.rq_mutex );
 	return NULL;
 }
