@@ -1,5 +1,5 @@
 /* tools.c - tools for slap tools */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/tools.c,v 1.39.2.3 2005/01/20 17:01:14 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/tools.c,v 1.43.2.2 2005/01/24 21:28:52 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -55,13 +55,8 @@ int ldbm_tool_entry_open(
 
 	if ( (id2entry = ldbm_cache_open( be, "id2entry", LDBM_SUFFIX, flags ))
 	    == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, CRIT,
-			   "Could not open/create id2entry%s\n", LDBM_SUFFIX, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "Could not open/create id2entry" LDBM_SUFFIX "\n",
 		    0, 0, 0 );
-#endif
 
 		return( -1 );
 	}
@@ -163,7 +158,7 @@ Entry* ldbm_tool_entry_get( BackendDB *be, ID id )
 		return NULL;
 	}
 
-	e = str2entry( data.dptr );
+	e = str2entry2( data.dptr, 0 );
 	ldbm_datum_free( id2entry->dbc_db, data );
 
 	if( e != NULL ) {
@@ -183,6 +178,7 @@ ID ldbm_tool_entry_put(
 	int rc, len;
 	ID id;
 	Operation op = {0};
+	Opheader ohdr = {0};
 
 	assert( slapMode & SLAP_TOOL_MODE );
 	assert( id2entry != NULL );
@@ -198,13 +194,8 @@ ID ldbm_tool_entry_put(
 
 	e->e_id = li->li_nextid++;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, ENTRY,
-		"ldbm_tool_entry_put: (%s)%ld\n", e->e_dn, e->e_id ,0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> ldbm_tool_entry_put( %ld, \"%s\" )\n",
 		e->e_id, e->e_dn, 0 );
-#endif
 
 	if ( dn2id( be, &e->e_nname, &id ) ) {
 		/* something bad happened to ldbm cache */
@@ -213,19 +204,14 @@ ID ldbm_tool_entry_put(
 	}
 
 	if( id != NOID ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ENTRY,
-			"ldbm_tool_entry_put: \"%s\" already exists (id=%ld)\n",
-			e->e_dn, id, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"<= ldbm_tool_entry_put: \"%s\" already exists (id=%ld)\n",
 			e->e_ndn, id, 0 );
-#endif
 		strncpy( text->bv_val, "already exists", text->bv_len );
 		return NOID;
 	}
 
+	op.o_hdr = &ohdr;
 	op.o_bd = be;
 	op.o_tmpmemctx = NULL;
 	op.o_tmpmfuncs = &ch_mfuncs;
@@ -275,28 +261,18 @@ int ldbm_tool_entry_reindex(
 	int rc;
 	Entry *e;
 	Operation op = {0};
+	Opheader ohdr = {0};
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, ENTRY, "ldbm_tool_entry_reindex: ID=%ld\n", 
-		(long)id, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ARGS, "=> ldbm_tool_entry_reindex( %ld )\n",
 		(long) id, 0, 0 );
-#endif
 
 
 	e = ldbm_tool_entry_get( be, id );
 
 	if( e == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, INFO,
-		   "ldbm_tool_entry_reindex: could not locate id %ld\n", 
-		   (long)id, 0, 0  );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"ldbm_tool_entry_reindex:: could not locate id=%ld\n",
 			(long) id, 0, 0 );
-#endif
 
 		return -1;
 	}
@@ -308,16 +284,12 @@ int ldbm_tool_entry_reindex(
 	 *
 	 */
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, ENTRY,
-		   "ldbm_tool_entry_reindex: (%s) %ld\n", e->e_dn, id, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> ldbm_tool_entry_reindex( %ld, \"%s\" )\n",
 		id, e->e_dn, 0 );
-#endif
 
 	dn2id_add( be, &e->e_nname, e->e_id );
 
+	op.o_hdr = &ohdr;
 	op.o_bd = be;
 	op.o_tmpmemctx = NULL;
 	op.o_tmpmfuncs = &ch_mfuncs;
