@@ -1,5 +1,5 @@
 /* null.c - the null backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-null/null.c,v 1.5.2.5 2005/01/20 17:01:15 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-null/null.c,v 1.12.2.2 2005/01/24 21:28:53 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2002-2005 The OpenLDAP Foundation.
@@ -14,7 +14,7 @@
  * <http://www.OpenLDAP.org/license.html>.
  */
 /* ACKNOWLEDGEMENTS:
- * This work was originally developed by Howard Chu for inclusion
+ * This work was originally developed by Hallvard Furuseth for inclusion
  * in OpenLDAP Software.
  */
 
@@ -24,30 +24,29 @@
 #include <ac/string.h>
 
 #include "slap.h"
-#include "external.h"
 
 struct null_info {
 	int bind_allowed;
 };
 
-int
+static int
 null_back_bind( Operation *op, SlapReply *rs )
 {
 	struct null_info *ni = (struct null_info *) op->o_bd->be_private;
 
 	if ( ni->bind_allowed ) {
 		/* front end will send result on success (0) */
-		return 0;
+		return LDAP_SUCCESS;
 	}
 
 	rs->sr_err = LDAP_INVALID_CREDENTIALS;
 	send_ldap_result( op, rs );
 
-	return 1;
+	return rs->sr_err;
 }
 
 /* add, delete, modify, modrdn, search */
-int
+static int
 null_back_success( Operation *op, SlapReply *rs )
 {
 	rs->sr_err = LDAP_SUCCESS;
@@ -56,7 +55,7 @@ null_back_success( Operation *op, SlapReply *rs )
 }
 
 /* compare */
-int
+static int
 null_back_false( Operation *op, SlapReply *rs )
 {
 	rs->sr_err = LDAP_COMPARE_FALSE;
@@ -64,12 +63,12 @@ null_back_false( Operation *op, SlapReply *rs )
 	return 0;
 }
 
-int
+static int
 null_back_db_config(
 	BackendDB	*be,
 	const char	*fname,
-	int			lineno,
-	int			argc,
+	int		lineno,
+	int		argc,
 	char		**argv )
 {
 	struct null_info *ni = (struct null_info *) be->be_private;
@@ -98,8 +97,7 @@ null_back_db_config(
 	return 0;
 }
 
-
-int
+static int
 null_back_db_init( BackendDB *be )
 {
 	struct null_info *ni;
@@ -110,10 +108,8 @@ null_back_db_init( BackendDB *be )
 	return 0;
 }
 
-int
-null_back_db_destroy(
-    Backend	*be
-)
+static int
+null_back_db_destroy( Backend *be )
 {
 	free( be->be_private );
 	return 0;
@@ -121,9 +117,7 @@ null_back_db_destroy(
 
 
 int
-null_back_initialize(
-    BackendInfo	*bi
-)
+null_back_initialize( BackendInfo *bi )
 {
 	bi->bi_open = 0;
 	bi->bi_close = 0;
@@ -157,17 +151,8 @@ null_back_initialize(
 }
 
 #if SLAPD_NULL == SLAPD_MOD_DYNAMIC
-int init_module(
-	int argc,
-	char *argv[] )
-{
-    BackendInfo bi;
 
-    memset( &bi, '\0', sizeof(bi) );
-    bi.bi_type = "null";
-    bi.bi_init = null_back_initialize;
+/* conditionally define the init_module() function */
+SLAP_BACKEND_INIT_MODULE( null )
 
-    backend_add(&bi);
-    return 0;
-}
-#endif /* SLAPD_NULL */
+#endif /* SLAPD_NULL == SLAPD_MOD_DYNAMIC */
