@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/init.c,v 1.30.2.5 2004/04/12 18:20:14 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/init.c,v 1.40.2.1 2005/01/20 18:04:01 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Copyright 1999-2005 The OpenLDAP Foundation.
  * Portions Copyright 1999 John C. Quillan.
  * Portions Copyright 2002 myinternet Limited.
  * All rights reserved.
@@ -15,45 +15,14 @@
  * <http://www.OpenLDAP.org/license.html>.
  */
 
-#include <EXTERN.h>
-#include <perl.h>
-#undef _ /* #defined by both Perl and ac/localize.h */
-
-#ifdef HAVE_WIN32_ASPERL
-#include "asperl_undefs.h"
-#endif
-
-#include "portable.h"
-	
-#include <stdio.h>
-
-#include "slap.h"
-
 #include "perl_back.h"
 
-
 static void perl_back_xs_init LDAP_P((PERL_BACK_XS_INIT_PARAMS));
 EXT void boot_DynaLoader LDAP_P((PERL_BACK_BOOT_DYNALOADER_PARAMS));
 
 PerlInterpreter *PERL_INTERPRETER = NULL;
 ldap_pvt_thread_mutex_t	perl_interpreter_mutex;
 
-#if SLAPD_PERL == SLAPD_MOD_DYNAMIC
-
-int init_module(int argc, char *argv[])
-{
-	BackendInfo bi;
-
-	memset( &bi, '\0', sizeof(bi) );
-	bi.bi_type = "perl";
-	bi.bi_init = perl_back_initialize;
-
-	backend_add(&bi);
-	return 0;
-}
-
-#endif /* SLAPD_PERL */
-
 
 /**********************************************************
  *
@@ -186,3 +155,12 @@ perl_back_xs_init(PERL_BACK_XS_INIT_PARAMS)
 	dXSUB_SYS;
 	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
 }
+
+#if SLAPD_PERL == SLAPD_MOD_DYNAMIC
+
+/* conditionally define the init_module() function */
+SLAP_BACKEND_INIT_MODULE( perl )
+
+#endif /* SLAPD_PERL == SLAPD_MOD_DYNAMIC */
+
+
