@@ -1,8 +1,10 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/schema-map.c,v 1.19.2.13 2005/10/04 13:18:51 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/schema-map.c,v 1.44.2.4 2005/06/09 16:31:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2005 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
+ * Portions Copyright 2002 Pierangelo Masarati.
+ * Portions Copyright 2004 Mark Adamson.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -15,20 +17,17 @@
  */
 /* ACKNOWLEDGEMENTS:
  * This work was initially developed by Dmitry Kovalev for inclusion
- * by OpenLDAP Software.
+ * by OpenLDAP Software.  Additional significant contributors include
+ * Pierangelo Masarati and Mark Adamson.
  */
 
 #include "portable.h"
 
-#ifdef SLAPD_SQL
-
 #include <stdio.h>
 #include <sys/types.h>
 #include "ac/string.h"
 
 #include "slap.h"
-#include "lber_pvt.h"
-#include "ldap_pvt.h"
 #include "proto-sql.h"
 
 #define BACKSQL_DUPLICATE	(-1)
@@ -101,19 +100,20 @@ backsql_dup_attr( void *v_m1, void *v_m2 )
 
 static int
 backsql_make_attr_query( 
+	backsql_info		*bi,
 	backsql_oc_map_rec 	*oc_map,
 	backsql_at_map_rec 	*at_map )
 {
 	struct berbuf	bb = BB_NULL;
 
-#ifdef BACKSQL_ALIASING_QUOTE
-	backsql_strfcat( &bb, "lblcbclblbcbl", 
+	backsql_strfcat( &bb, "lblbbbblblbcbl", 
 			(ber_len_t)STRLENOF( "SELECT " ), "SELECT ", 
 			&at_map->bam_sel_expr, 
-			(ber_len_t)STRLENOF( " " BACKSQL_ALIASING ), " " BACKSQL_ALIASING, 
-			BACKSQL_ALIASING_QUOTE,
+			(ber_len_t)STRLENOF( " " ), " ",
+			&bi->sql_aliasing,
+			&bi->sql_aliasing_quote, 
 			&at_map->bam_ad->ad_cname,
-			BACKSQL_ALIASING_QUOTE,
+			&bi->sql_aliasing_quote,
 			(ber_len_t)STRLENOF( " FROM " ), " FROM ", 
 			&at_map->bam_from_tbls, 
 			(ber_len_t)STRLENOF( " WHERE " ), " WHERE ", 
@@ -121,20 +121,6 @@ backsql_make_attr_query(
 			'.', 
 			&oc_map->bom_keycol,
 			(ber_len_t)STRLENOF( "=?" ), "=?" );
-#else /* ! BACKSQL_ALIASING_QUOTE */
-	backsql_strfcat( &bb, "lblblblbcbl", 
-			(ber_len_t)STRLENOF( "SELECT " ), "SELECT ", 
-			&at_map->bam_sel_expr, 
-			(ber_len_t)STRLENOF( " " BACKSQL_ALIASING ), " " BACKSQL_ALIASING, 
-			&at_map->bam_ad->ad_cname,
-			(ber_len_t)STRLENOF( " FROM " ), " FROM ", 
-			&at_map->bam_from_tbls, 
-			(ber_len_t)STRLENOF( " WHERE " ), " WHERE ", 
-			&oc_map->bom_keytbl,
-			'.', 
-			&oc_map->bom_keycol,
-			(ber_len_t)STRLENOF( "=?" ), "=?" );
-#endif /* ! BACKSQL_ALIASING_QUOTE */
 
 	if ( !BER_BVISNULL( &at_map->bam_join_where ) ) {
 		backsql_strfcat( &bb, "lb",
@@ -142,25 +128,42 @@ backsql_make_attr_query(
 				&at_map->bam_join_where );
 	}
 
-#ifdef BACKSQL_ALIASING_QUOTE
-	backsql_strfcat( &bb, "lcbc", 
+	backsql_strfcat( &bb, "lbbb", 
 			(ber_len_t)STRLENOF( " ORDER BY " ), " ORDER BY ",
-			BACKSQL_ALIASING_QUOTE,
-			&at_map->bam_sel_expr,
-			BACKSQL_ALIASING_QUOTE );
-#else /* ! BACKSQL_ALIASING_QUOTE */
-	backsql_strfcat( &bb, "lb", 
-			(ber_len_t)STRLENOF( " ORDER BY " ), " ORDER BY ",
-			&at_map->bam_sel_expr );
-#endif /* ! BACKSQL_ALIASING_QUOTE */
+			&bi->sql_aliasing_quote,
+			&at_map->bam_ad->ad_cname,
+			&bi->sql_aliasing_quote );
 
 	at_map->bam_query = bb.bb_val.bv_val;
-	
+
+#ifdef BACKSQL_COUNTQUERY
+	/* Query to count how many rows will be returned. */
+	BER_BVZERO( &bb.bb_val );
+	bb.bb_len = 0;
+	backsql_strfcat( &bb, "lblbcbl", 
+			(ber_len_t)STRLENOF( "SELECT COUNT(*) FROM " ),
+				"SELECT COUNT(*) FROM ", 
+			&at_map->bam_from_tbls, 
+			(ber_len_t)STRLENOF( " WHERE " ), " WHERE ", 
+			&oc_map->bom_keytbl,
+			'.', 
+			&oc_map->bom_keycol,
+			(ber_len_t)STRLENOF( "=?" ), "=?" );
+
+	if ( !BER_BVISNULL( &at_map->bam_join_where ) ) {
+		backsql_strfcat( &bb, "lb",
+				(ber_len_t)STRLENOF( " AND " ), " AND ", 
+				&at_map->bam_join_where );
+	}
+
+	at_map->bam_countquery = bb.bb_val.bv_val;
+#endif /* BACKSQL_COUNTQUERY */
+
 	return 0;
 }
 
 static int
-backsql_add_sysmaps( backsql_oc_map_rec *oc_map )
+backsql_add_sysmaps( backsql_info *bi, backsql_oc_map_rec *oc_map )
 {
 	backsql_at_map_rec	*at_map;
 	char			s[] = "+9223372036854775807L";
@@ -168,8 +171,7 @@ backsql_add_sysmaps( backsql_oc_map_rec *oc_map )
 	struct berbuf		bb;
 	
 	sbv.bv_val = s;
-	snprintf( s, sizeof( s ), "%ld", oc_map->bom_id );
-	sbv.bv_len = strlen( s );
+	sbv.bv_len = snprintf( s, sizeof( s ), "%ld", oc_map->bom_id );
 
 	/* extra objectClasses */
 	at_map = (backsql_at_map_rec *)ch_calloc(1, 
@@ -182,7 +184,7 @@ backsql_add_sysmaps( backsql_oc_map_rec *oc_map )
 	
 	bb.bb_len = at_map->bam_from_tbls.bv_len + 1;
 	bb.bb_val = at_map->bam_from_tbls;
-	backsql_merge_from_clause( &bb, &oc_map->bom_keytbl );
+	backsql_merge_from_clause( bi, &bb, &oc_map->bom_keytbl );
 	at_map->bam_from_tbls = bb.bb_val;
 
 	BER_BVZERO( &bb.bb_val );
@@ -229,7 +231,7 @@ backsql_add_sysmaps( backsql_oc_map_rec *oc_map )
 		snprintf( tmp, sizeof(tmp), 
 			"DELETE FROM ldap_entry_objclasses "
 			"WHERE entry_id=(SELECT id FROM ldap_entries "
-			"WHERE oc_map_id=%lu "
+			"WHERE oc_map_id=%lu"
 			"AND keyval=?) AND oc_name=?",
 			oc_map->bom_id );
 		at_map->bam_delete_proc = ch_strdup( tmp );
@@ -239,7 +241,7 @@ backsql_add_sysmaps( backsql_oc_map_rec *oc_map )
 	at_map->bam_expect_return = 0;
 	at_map->bam_next = NULL;
 
-	backsql_make_attr_query( oc_map, at_map );
+	backsql_make_attr_query( bi, oc_map, at_map );
 	if ( avl_insert( &oc_map->bom_attrs, at_map, backsql_cmp_attr, backsql_dup_attr ) == BACKSQL_DUPLICATE ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_add_sysmaps(): "
 				"duplicate attribute \"%s\" in objectClass \"%s\" map\n",
@@ -263,86 +265,11 @@ backsql_add_sysmaps( backsql_oc_map_rec *oc_map )
 			&sbv );
 	at_map->bam_join_where = bb.bb_val;
 
-	/* referral attribute */
-	at_map = (backsql_at_map_rec *)ch_calloc( 1, 
-			sizeof( backsql_at_map_rec ) );
-	at_map->bam_ad = slap_schema.si_ad_ref;
-	ber_str2bv( "ldap_referrals.url", 0, 1, &at_map->bam_sel_expr );
-	ber_str2bv( "ldap_referrals,ldap_entries", 0, 1, &at_map->bam_from_tbls );
-
-	bb.bb_len = at_map->bam_from_tbls.bv_len + 1;
-	bb.bb_val = at_map->bam_from_tbls;
-	backsql_merge_from_clause( &bb, &oc_map->bom_keytbl );
-	at_map->bam_from_tbls = bb.bb_val;
-
-	BER_BVZERO( &bb.bb_val );
-	bb.bb_len = 0;
-	backsql_strfcat( &bb, "lbcblb",
-			(ber_len_t)STRLENOF( "ldap_entries.id=ldap_referrals.entry_id AND ldap_entries.keyval=" ),
-				"ldap_entries.id=ldap_referrals.entry_id AND ldap_entries.keyval=",
-			&oc_map->bom_keytbl, 
-			'.', 
-			&oc_map->bom_keycol,
-			(ber_len_t)STRLENOF( " AND ldap_entries.oc_map_id=" ), 
-				" AND ldap_entries.oc_map_id=", 
-			&sbv );
-
-	at_map->bam_join_where = bb.bb_val;
-
-	at_map->bam_oc = NULL;
-
-	at_map->bam_add_proc = NULL;
-	{
-		char	tmp[] =
-			"INSERT INTO ldap_referrals "
-			"(entry_id,url) VALUES "
-			"((SELECT id FROM ldap_entries "
-			"WHERE oc_map_id="
-			"18446744073709551615UL "	/* 64 bit ULONG */
-			"AND keyval=?),?)";
-		snprintf( tmp, sizeof(tmp), 
-			"INSERT INTO ldap_referrals "
-			"(entry_id,url) VALUES "
-			"((SELECT id FROM ldap_entries "
-			"WHERE oc_map_id=%lu "
-			"AND keyval=?),?)", oc_map->bom_id );
-		at_map->bam_add_proc = ch_strdup( tmp );
-	}
-
-	at_map->bam_delete_proc = NULL;
-	{
-		char	tmp[] =
-			"DELETE FROM ldap_referrals "
-			"WHERE entry_id=(SELECT id FROM ldap_entries "
-			"WHERE oc_map_id="
-			"18446744073709551615UL "	/* 64 bit ULONG */
-			"AND keyval=?) and url=?";
-		snprintf( tmp, sizeof(tmp), 
-			"DELETE FROM ldap_referrals "
-			"WHERE entry_id=(SELECT id FROM ldap_entries "
-			"WHERE oc_map_id=%lu "
-			"AND keyval=?) and url=?",
-			oc_map->bom_id );
-		at_map->bam_delete_proc = ch_strdup( tmp );
-	}
-
-	at_map->bam_param_order = 0;
-	at_map->bam_expect_return = 0;
-	at_map->bam_next = NULL;
-
-	backsql_make_attr_query( oc_map, at_map );
-	if ( avl_insert( &oc_map->bom_attrs, at_map, backsql_cmp_attr, backsql_dup_attr ) == BACKSQL_DUPLICATE ) {
-		Debug( LDAP_DEBUG_TRACE, "backsql_add_sysmaps(): "
-				"duplicate attribute \"%s\" in objectClass \"%s\" map\n",
-				at_map->bam_ad->ad_cname.bv_val,
-				oc_map->bom_oc->soc_cname.bv_val, 0 );
-	}
-
 	return 1;
 }
 
 struct backsql_attr_schema_info {
-	backsql_info	*bas_si;
+	backsql_info	*bas_bi;
 	SQLHDBC		bas_dbh;
 	SQLHSTMT	bas_sth;
 	unsigned long	*bas_oc_id;
@@ -366,7 +293,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 		"    \"%s\"\n"
 		"    for objectClass \"%s\"\n"
 		"    with param oc_id=\"%lu\"\n",
-		bas->bas_si->at_query,
+		bas->bas_bi->sql_at_query,
 		BACKSQL_OC_NAME( oc_map ),
 		*bas->bas_oc_id );
 
@@ -377,10 +304,11 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 			"    \"%s\"\n"
 			"    for objectClass \"%s\"\n"
 			"    with param oc_id=\"%lu\"\n",
-			bas->bas_si->at_query,
+			bas->bas_bi->sql_at_query,
 			BACKSQL_OC_NAME( oc_map ),
 			*bas->bas_oc_id );
-		backsql_PrintErrors( SQL_NULL_HENV, bas->bas_dbh, bas->bas_sth, rc );
+		backsql_PrintErrors( bas->bas_bi->sql_db_env,
+				bas->bas_dbh, bas->bas_sth, rc );
 		bas->bas_rc = LDAP_OTHER;
 		return BACKSQL_AVL_STOP;
 	}
@@ -388,6 +316,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 	backsql_BindRowAsStrings( bas->bas_sth, &at_row );
 	for ( ; rc = SQLFetch( bas->bas_sth ), BACKSQL_SUCCESS( rc ); ) {
 		const char	*text = NULL;
+		char		*next = NULL;
 		struct berval	bv;
 		struct berbuf	bb = BB_NULL;
 
@@ -431,7 +360,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 		}
 
 		ber_str2bv( at_row.cols[ 2 ], 0, 0, &bv );
-		backsql_merge_from_clause( &bb, &bv );
+		backsql_merge_from_clause( bas->bas_bi, &bb, &bv );
 		at_map->bam_from_tbls = bb.bb_val;
 		if ( at_row.value_len[ 3 ] < 0 ) {
 			BER_BVZERO( &at_map->bam_join_where );
@@ -449,10 +378,16 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 			at_map->bam_delete_proc = ch_strdup( at_row.cols[ 5 ] );
 		}
 		at_map->bam_param_order = strtol( at_row.cols[ 6 ], 
-				NULL, 0 );
+				&next, 0 );
+		if ( next == at_row.cols[ 6 ] || next[0] != '\0' ) {
+			/* error */
+		}
 		at_map->bam_expect_return = strtol( at_row.cols[ 7 ],
-				NULL, 0 );
-		backsql_make_attr_query( oc_map, at_map );
+				&next, 0 );
+		if ( next == at_row.cols[ 7 ] || next[0] != '\0' ) {
+			/* error */
+		}
+		backsql_make_attr_query( bas->bas_bi, oc_map, at_map );
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_attr_mapping(): "
 			"preconstructed query \"%s\"\n",
 			at_map->bam_query, 0, 0 );
@@ -465,13 +400,13 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 					oc_map->bom_oc->soc_cname.bv_val, 0 );
 		}
 
-		if ( !BER_BVISNULL( &bas->bas_si->upper_func ) &&
+		if ( !BER_BVISNULL( &bas->bas_bi->sql_upper_func ) &&
 				BER_BVISNULL( &at_map->bam_sel_expr_u ) )
 		{
 			struct berbuf	bb = BB_NULL;
 
 			backsql_strfcat( &bb, "bcbc",
-					&bas->bas_si->upper_func,
+					&bas->bas_bi->sql_upper_func,
 					'(' /* ) */ ,
 					&at_map->bam_sel_expr,
 					/* ( */ ')' );
@@ -485,16 +420,16 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 		"autoadding 'objectClass' and 'ref' mappings\n",
 		BACKSQL_OC_NAME( oc_map ), 0, 0 );
 
-	(void)backsql_add_sysmaps( oc_map );
+	(void)backsql_add_sysmaps( bas->bas_bi, oc_map );
 
 	return BACKSQL_AVL_CONTINUE;
 }
 
 
 int
-backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
+backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 {
-	SQLHSTMT 			sth;
+	SQLHSTMT 			sth = SQL_NULL_HSTMT;
 	RETCODE				rc;
 	BACKSQL_ROW_NTS			oc_row;
 	unsigned long			oc_id;
@@ -506,31 +441,31 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 	/* 
 	 * TimesTen : See if the ldap_entries.dn_ru field exists in the schema
 	 */
-	if ( !BACKSQL_DONTCHECK_LDAPINFO_DN_RU( si ) ) {
+	if ( !BACKSQL_DONTCHECK_LDAPINFO_DN_RU( bi ) ) {
 		rc = backsql_Prepare( dbh, &sth, 
 				backsql_check_dn_ru_query, 0 );
 		if ( rc == SQL_SUCCESS ) {
 			/* Yes, the field exists */
-			si->bsql_flags |= BSQLF_HAS_LDAPINFO_DN_RU;
+			bi->sql_flags |= BSQLF_HAS_LDAPINFO_DN_RU;
    			Debug( LDAP_DEBUG_TRACE, "ldapinfo.dn_ru field exists "
 				"in the schema\n", 0, 0, 0 );
 		} else {
 			/* No such field exists */
-			si->bsql_flags &= ~BSQLF_HAS_LDAPINFO_DN_RU;
+			bi->sql_flags &= ~BSQLF_HAS_LDAPINFO_DN_RU;
 		}
 
 		SQLFreeStmt( sth, SQL_DROP );
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): oc_query \"%s\"\n", 
-			si->oc_query, 0, 0 );
+			bi->sql_oc_query, 0, 0 );
 
-	rc = backsql_Prepare( dbh, &sth, si->oc_query, 0 );
+	rc = backsql_Prepare( dbh, &sth, bi->sql_oc_query, 0 );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 			"error preparing oc_query: \"%s\"\n", 
-			si->oc_query, 0, 0 );
-		backsql_PrintErrors( si->db_env, dbh, sth, rc );
+			bi->sql_oc_query, 0, 0 );
+		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
 		return LDAP_OTHER;
 	}
 
@@ -538,7 +473,7 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 			"error executing oc_query: \n", 0, 0, 0 );
-		backsql_PrintErrors( si->db_env, dbh, sth, rc );
+		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
 		return LDAP_OTHER;
 	}
 
@@ -566,7 +501,7 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 			: ch_strdup( oc_row.cols[ 4 ] );
 
 		colnum = 5;
-		if ( BACKSQL_CREATE_NEEDS_SELECT( si ) ) {
+		if ( BACKSQL_CREATE_NEEDS_SELECT( bi ) ) {
 			colnum = 6;
 			oc_map->bom_create_keyval = ( oc_row.value_len[ 5 ] < 0 ) 
 				? NULL : ch_strdup( oc_row.cols[ 5 ] );
@@ -576,19 +511,41 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 		oc_map->bom_expect_return = strtol( oc_row.cols[ colnum + 1 ], 
 				NULL, 0 );
 
+		colnum += 2;
+		if ( ( oc_row.ncols > colnum ) &&
+				( oc_row.value_len[ colnum ] > 0 ) )
+		{
+			const char	*text;
+
+			oc_map->bom_create_hint = NULL;
+			rc = slap_str2ad( oc_row.cols[ colnum ],
+					&oc_map->bom_create_hint, &text );
+			if ( rc != SQL_SUCCESS ) {
+				Debug( LDAP_DEBUG_TRACE, "load_schema_map(): "
+						"error matching "
+						"AttributeDescription %s "
+						"in create_hint: %s (%d)\n",
+						oc_row.cols[ colnum ],
+						text, rc );
+				backsql_PrintErrors( bi->sql_db_env, dbh,
+						sth, rc );
+				return LDAP_OTHER;
+			}
+		}
+
 		/*
 		 * FIXME: first attempt to check for offending
 		 * instructions in {create|delete}_proc
 		 */
 
 		oc_map->bom_attrs = NULL;
-		if ( avl_insert( &si->oc_by_oc, oc_map, backsql_cmp_oc, avl_dup_error ) == -1 ) {
+		if ( avl_insert( &bi->sql_oc_by_oc, oc_map, backsql_cmp_oc, avl_dup_error ) == -1 ) {
 			Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 					"duplicate objectClass \"%s\" in objectClass map\n",
 					oc_map->bom_oc->soc_cname.bv_val, 0, 0 );
 			return LDAP_OTHER;
 		}
-		if ( avl_insert( &si->oc_by_id, oc_map, backsql_cmp_oc_id, avl_dup_error ) == -1 ) {
+		if ( avl_insert( &bi->sql_oc_by_id, oc_map, backsql_cmp_oc_id, avl_dup_error ) == -1 ) {
 			Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 					"duplicate objectClass \"%s\" in objectClass by ID map\n",
 					oc_map->bom_oc->soc_cname.bv_val, 0, 0 );
@@ -596,22 +553,27 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 		}
 		oc_id = oc_map->bom_id;
 		Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
-			"objectClass \"%s\": keytbl=\"%s\" keycol=\"%s\"\n",
+			"objectClass \"%s\":\n    keytbl=\"%s\" keycol=\"%s\"\n",
 			BACKSQL_OC_NAME( oc_map ),
 			oc_map->bom_keytbl.bv_val, oc_map->bom_keycol.bv_val );
 		if ( oc_map->bom_create_proc ) {
-			Debug( LDAP_DEBUG_TRACE, "create_proc=\"%s\"\n",
+			Debug( LDAP_DEBUG_TRACE, "    create_proc=\"%s\"\n",
 				oc_map->bom_create_proc, 0, 0 );
 		}
 		if ( oc_map->bom_create_keyval ) {
-			Debug( LDAP_DEBUG_TRACE, "create_keyval=\"%s\"\n",
+			Debug( LDAP_DEBUG_TRACE, "    create_keyval=\"%s\"\n",
 				oc_map->bom_create_keyval, 0, 0 );
 		}
+		if ( oc_map->bom_create_hint ) {
+			Debug( LDAP_DEBUG_TRACE, "    create_hint=\"%s\"\n", 
+				oc_map->bom_create_hint->ad_cname.bv_val,
+				0, 0 );
+		}
 		if ( oc_map->bom_delete_proc ) {
-			Debug( LDAP_DEBUG_TRACE, "delete_proc=\"%s\"\n", 
+			Debug( LDAP_DEBUG_TRACE, "    delete_proc=\"%s\"\n", 
 				oc_map->bom_delete_proc, 0, 0 );
 		}
-		Debug( LDAP_DEBUG_TRACE, "expect_return: "
+		Debug( LDAP_DEBUG_TRACE, "    expect_return: "
 			"add=%d, del=%d; attributes:\n",
 			BACKSQL_IS_ADD( oc_map->bom_expect_return ), 
 			BACKSQL_IS_DEL( oc_map->bom_expect_return ), 0 );
@@ -622,37 +584,38 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 
 	/* prepare for attribute fetching */
 	Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): at_query \"%s\"\n", 
-			si->at_query, 0, 0 );
+			bi->sql_at_query, 0, 0 );
 
-	rc = backsql_Prepare( dbh, &sth, si->at_query, 0 );
+	rc = backsql_Prepare( dbh, &sth, bi->sql_at_query, 0 );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 			"error preparing at_query: \"%s\"\n", 
-			si->at_query, 0, 0 );
-		backsql_PrintErrors( si->db_env, dbh, sth, rc );
+			bi->sql_at_query, 0, 0 );
+		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
 		return LDAP_OTHER;
 	}
 
-	rc = backsql_BindParamID( sth, 1, &oc_id );
+	rc = backsql_BindParamInt( sth, 1, SQL_PARAM_INPUT, &oc_id );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 			"error binding param \"oc_id\" for at_query\n", 0, 0, 0 );
-		backsql_PrintErrors( si->db_env, dbh, sth, rc );
+		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
+		SQLFreeStmt( sth, SQL_DROP );
 		return LDAP_OTHER;
 	}
 
-	bas.bas_si = si;
+	bas.bas_bi = bi;
 	bas.bas_dbh = dbh;
 	bas.bas_sth = sth;
 	bas.bas_oc_id = &oc_id;
 	bas.bas_rc = LDAP_SUCCESS;
 
-	(void)avl_apply( si->oc_by_oc, backsql_oc_get_attr_mapping,
+	(void)avl_apply( bi->sql_oc_by_oc, backsql_oc_get_attr_mapping,
 			&bas, BACKSQL_AVL_STOP, AVL_INORDER );
 
 	SQLFreeStmt( sth, SQL_DROP );
 
-	si->bsql_flags |= BSQLF_SCHEMA_LOADED;
+	bi->sql_flags |= BSQLF_SCHEMA_LOADED;
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_load_schema_map()\n", 0, 0, 0 );
 
@@ -660,7 +623,7 @@ backsql_load_schema_map( backsql_info *si, SQLHDBC dbh )
 }
 
 backsql_oc_map_rec *
-backsql_oc2oc( backsql_info *si, ObjectClass *oc )
+backsql_oc2oc( backsql_info *bi, ObjectClass *oc )
 {
 	backsql_oc_map_rec	tmp, *res;
 
@@ -671,7 +634,7 @@ backsql_oc2oc( backsql_info *si, ObjectClass *oc )
 #endif /* BACKSQL_TRACE */
 
 	tmp.bom_oc = oc;
-	res = (backsql_oc_map_rec *)avl_find( si->oc_by_oc, &tmp, backsql_cmp_oc );
+	res = (backsql_oc_map_rec *)avl_find( bi->sql_oc_by_oc, &tmp, backsql_cmp_oc );
 #ifdef BACKSQL_TRACE
 	if ( res != NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "<==backsql_oc2oc(): "
@@ -687,7 +650,7 @@ backsql_oc2oc( backsql_info *si, ObjectClass *oc )
 }
 
 backsql_oc_map_rec *
-backsql_name2oc( backsql_info *si, struct berval *oc_name )
+backsql_name2oc( backsql_info *bi, struct berval *oc_name )
 {
 	backsql_oc_map_rec	tmp, *res;
 
@@ -702,7 +665,7 @@ backsql_name2oc( backsql_info *si, struct berval *oc_name )
 		return NULL;
 	}
 
-	res = (backsql_oc_map_rec *)avl_find( si->oc_by_oc, &tmp, backsql_cmp_oc );
+	res = (backsql_oc_map_rec *)avl_find( bi->sql_oc_by_oc, &tmp, backsql_cmp_oc );
 #ifdef BACKSQL_TRACE
 	if ( res != NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "<==oc_with_name(): "
@@ -718,7 +681,7 @@ backsql_name2oc( backsql_info *si, struct berval *oc_name )
 }
 
 backsql_oc_map_rec *
-backsql_id2oc( backsql_info *si, unsigned long id )
+backsql_id2oc( backsql_info *bi, unsigned long id )
 {
 	backsql_oc_map_rec	tmp, *res;
  
@@ -728,7 +691,7 @@ backsql_id2oc( backsql_info *si, unsigned long id )
 #endif /* BACKSQL_TRACE */
 
 	tmp.bom_id = id;
-	res = (backsql_oc_map_rec *)avl_find( si->oc_by_id, &tmp,
+	res = (backsql_oc_map_rec *)avl_find( bi->sql_oc_by_id, &tmp,
 			backsql_cmp_oc_id );
 
 #ifdef BACKSQL_TRACE
@@ -791,7 +754,7 @@ supad2at_f( void *v_at, void *v_arg )
 	struct supad2at_t	*va = (struct supad2at_t *)v_arg;
 
 	if ( is_at_subtype( at->bam_ad->ad_type, va->ad->ad_type ) ) {
-		backsql_at_map_rec	**ret;
+		backsql_at_map_rec	**ret = NULL;
 		unsigned		i;
 
 		/* if already listed, holler! (should never happen) */
@@ -808,9 +771,11 @@ supad2at_f( void *v_at, void *v_arg )
 		}
 
 		ret = ch_realloc( va->ret,
-				sizeof( backsql_at_map_rec *) * ( va->n + 2 ) );
+				sizeof( backsql_at_map_rec * ) * ( va->n + 2 ) );
 		if ( ret == NULL ) {
 			ch_free( va->ret );
+			va->ret = NULL;
+			va->n = 0;
 			return SUPAD2AT_STOP;
 		}
 
@@ -832,7 +797,7 @@ int
 backsql_supad2at( backsql_oc_map_rec *objclass, AttributeDescription *supad,
 		backsql_at_map_rec ***pret )
 {
-	struct supad2at_t	va;
+	struct supad2at_t	va = { 0 };
 	int			rc;
 
 	assert( objclass );
@@ -841,9 +806,7 @@ backsql_supad2at( backsql_oc_map_rec *objclass, AttributeDescription *supad,
 
 	*pret = NULL;
 
-	va.ret = NULL;
 	va.ad = supad;
-	va.n = 0;
 
 	rc = avl_apply( objclass->bom_attrs, supad2at_f, &va,
 			SUPAD2AT_STOP, AVL_INORDER );
@@ -876,7 +839,7 @@ backsql_free_attr( void *v_at )
 	if ( at->bam_delete_proc != NULL ) {
 		ch_free( at->bam_delete_proc );
 	}
-	if ( at->bam_query ) {
+	if ( at->bam_query != NULL ) {
 		ch_free( at->bam_query );
 	}
 
@@ -919,14 +882,12 @@ backsql_free_oc( void *v_oc )
 }
 
 int
-backsql_destroy_schema_map( backsql_info *si )
+backsql_destroy_schema_map( backsql_info *bi )
 {
 	Debug( LDAP_DEBUG_TRACE, "==>destroy_schema_map()\n", 0, 0, 0 );
-	avl_free( si->oc_by_oc, 0 );
-	avl_free( si->oc_by_id, backsql_free_oc );
+	avl_free( bi->sql_oc_by_oc, 0 );
+	avl_free( bi->sql_oc_by_id, backsql_free_oc );
 	Debug( LDAP_DEBUG_TRACE, "<==destroy_schema_map()\n", 0, 0, 0 );
 	return 0;
 }
 
-#endif /* SLAPD_SQL */
-
