@@ -1,5 +1,5 @@
 /* bind.c - decode an ldap bind operation and pass it to a backend db */
-/* $OpenLDAP: pkg/ldap/servers/slapd/bind.c,v 1.157.2.16 2005/11/15 23:05:34 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/bind.c,v 1.189.2.2 2005/01/24 21:28:51 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -31,7 +31,6 @@
 #include <ac/string.h>
 #include <ac/socket.h>
 
-#include "ldap_pvt.h"
 #include "slap.h"
 #ifdef LDAP_SLAPI
 #include "slapi/slapi.h"
@@ -51,11 +50,7 @@ do_bind(
 	ber_tag_t tag;
 	Backend *be = NULL;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "do_bind: conn %d\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "do_bind\n", 0, 0, 0 );
-#endif
 
 	/*
 	 * Force to connection to "anonymous" until bind succeeds.
@@ -64,25 +59,26 @@ do_bind(
 	if ( op->o_conn->c_sasl_bind_in_progress ) {
 		be = op->o_conn->c_authz_backend;
 	}
-	if ( op->o_conn->c_dn.bv_len ) {
+	if ( !BER_BVISEMPTY( &op->o_conn->c_dn ) ) {
 		/* log authorization identity demotion */
 		Statslog( LDAP_DEBUG_STATS,
-			"conn=%lu op=%lu BIND anonymous mech=implicit ssf=0\n",
-			op->o_connid, op->o_opid, 0, 0, 0 );
+			"%s BIND anonymous mech=implicit ssf=0\n",
+			op->o_log_prefix, 0, 0, 0, 0 );
 	}
 	connection2anonymous( op->o_conn );
 	if ( op->o_conn->c_sasl_bind_in_progress ) {
 		op->o_conn->c_authz_backend = be;
 	}
 	ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
-	if ( op->o_dn.bv_val != NULL ) {
-		free( op->o_dn.bv_val );
-		op->o_dn.bv_val = ch_strdup( "" );
+	if ( !BER_BVISNULL( &op->o_dn ) ) {
+		/* NOTE: temporarily wasting few bytes
+		 * (until bind is completed), but saving
+		 * a couple of ch_free() and ch_strdup("") */ 
+		op->o_dn.bv_val[0] = '\0';
 		op->o_dn.bv_len = 0;
 	}
-	if ( op->o_ndn.bv_val != NULL ) {
-		free( op->o_ndn.bv_val );
-		op->o_ndn.bv_val = ch_strdup( "" );
+	if ( !BER_BVISNULL( &op->o_ndn ) ) {
+		op->o_ndn.bv_val[0] = '\0';
 		op->o_ndn.bv_len = 0;
 	}
 
@@ -109,12 +105,7 @@ do_bind(
 	tag = ber_scanf( ber, "{imt" /*}*/, &version, &dn, &method );
 
 	if ( tag == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_bind: conn %d  ber_scanf failed\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "bind: ber_scanf failed\n", 0, 0, 0 );
-#endif
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
 		rs->sr_err = SLAPD_DISCONNECT;
 		goto cleanup;
@@ -137,8 +128,7 @@ do_bind(
 				tag = ber_scanf( ber, "m", &op->orb_cred );
 			} else {
 				tag = LDAP_TAG_LDAPCRED;
-				op->orb_cred.bv_val = NULL;
-				op->orb_cred.bv_len = 0;
+				BER_BVZERO( &op->orb_cred );
 			}
 
 			if ( tag != LBER_ERROR ) {
@@ -154,12 +144,7 @@ do_bind(
 	}
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_bind: conn %d  get_ctrls failed\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_bind: get_ctrls failed\n", 0, 0, 0 );
-#endif
 		goto cleanup;
 	} 
 
@@ -170,55 +155,30 @@ do_bind(
 	rs->sr_err = dnPrettyNormal( NULL, &dn, &op->o_req_dn, &op->o_req_ndn,
 		op->o_tmpmemctx );
 	if ( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_bind: conn %d  invalid dn (%s)\n", 
-			op->o_connid, dn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "bind: invalid dn (%s)\n",
 			dn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid DN" );
 		goto cleanup;
 	}
 
 	if( op->orb_method == LDAP_AUTH_SASL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION,	 DETAIL1, 
-			"do_sasl_bind: conn %d  dn (%s) mech %s\n", 
-			op->o_connid, op->o_req_dn.bv_val, mech.bv_val );
-#else
 		Debug( LDAP_DEBUG_TRACE, "do_sasl_bind: dn (%s) mech %s\n",
 			op->o_req_dn.bv_val, mech.bv_val, NULL );
-#endif
 
 	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, DETAIL1, 
-			"do_bind: version=%ld dn=\"%s\" method=%ld\n",
-			(unsigned long) version, op->o_req_dn.bv_val,
-			(unsigned long) op->orb_method );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"do_bind: version=%ld dn=\"%s\" method=%ld\n",
 			(unsigned long) version, op->o_req_dn.bv_val,
 			(unsigned long) op->orb_method );
-#endif
 	}
 
-	Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu BIND dn=\"%s\" method=%ld\n",
-	    op->o_connid, op->o_opid, op->o_req_dn.bv_val,
-		(unsigned long) op->orb_method, 0 );
+	Statslog( LDAP_DEBUG_STATS, "%s BIND dn=\"%s\" method=%ld\n",
+	    op->o_log_prefix, op->o_req_dn.bv_val,
+		(unsigned long) op->orb_method, 0, 0 );
 
 	if ( version < LDAP_VERSION_MIN || version > LDAP_VERSION_MAX ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_bind: conn %d  unknown version = %ld\n",
-			op->o_connid, (unsigned long)version, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_bind: unknown version=%ld\n",
 			(unsigned long) version, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 			"requested protocol version not supported" );
 		goto cleanup;
@@ -238,8 +198,40 @@ do_bind(
 	op->o_conn->c_protocol = version;
 	ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 
+	op->orb_tmp_mech = mech;
+
+	op->o_bd = frontendDB;
+	rs->sr_err = frontendDB->be_bind( op, rs );
+
+cleanup:
+	if ( rs->sr_err == LDAP_SUCCESS ) {
+		if ( op->orb_method != LDAP_AUTH_SASL ) {
+			ber_dupbv( &op->o_conn->c_authmech, &mech );
+		}
+		op->o_conn->c_authtype = op->orb_method;
+	}
+
+	op->o_conn->c_sasl_bindop = NULL;
+
+	if( !BER_BVISNULL( &op->o_req_dn ) ) {
+		slap_sl_free( op->o_req_dn.bv_val, op->o_tmpmemctx );
+		BER_BVZERO( &op->o_req_dn );
+	}
+	if( !BER_BVISNULL( &op->o_req_ndn ) ) {
+		slap_sl_free( op->o_req_ndn.bv_val, op->o_tmpmemctx );
+		BER_BVZERO( &op->o_req_ndn );
+	}
+
+	return rs->sr_err;
+}
+
+int
+fe_op_bind( Operation *op, SlapReply *rs )
+{
+	struct berval	mech = op->orb_tmp_mech;
+
 	/* check for inappropriate controls */
-	if( get_manageDSAit( op ) == SLAP_CRITICAL_CONTROL ) {
+	if( get_manageDSAit( op ) == SLAP_CONTROL_CRITICAL ) {
 		send_ldap_error( op, rs,
 			LDAP_UNAVAILABLE_CRITICAL_EXTENSION,
 			"manageDSAit control inappropriate" );
@@ -250,31 +242,19 @@ do_bind(
 	op->o_conn->c_sasl_bindop = op;
 
 	if ( op->orb_method == LDAP_AUTH_SASL ) {
-		if ( version < LDAP_VERSION3 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, 
-				"do_bind: conn %d  sasl with LDAPv%ld\n",
-				op->o_connid, (unsigned long)version , 0 );
-#else
+		if ( op->o_protocol < LDAP_VERSION3 ) {
 			Debug( LDAP_DEBUG_ANY, "do_bind: sasl with LDAPv%ld\n",
-				(unsigned long) version, 0, 0 );
-#endif
+				(unsigned long)op->o_protocol, 0, 0 );
 			send_ldap_discon( op, rs,
 				LDAP_PROTOCOL_ERROR, "SASL bind requires LDAPv3" );
 			rs->sr_err = SLAPD_DISCONNECT;
 			goto cleanup;
 		}
 
-		if( mech.bv_len == 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, 
-				   "do_bind: conn %d  no SASL mechanism provided\n",
-				   op->o_connid, 0, 0 );
-#else
+		if( BER_BVISNULL( &mech ) || BER_BVISEMPTY( &mech ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"do_bind: no sasl mechanism provided\n",
 				0, 0, 0 );
-#endif
 			send_ldap_error( op, rs, LDAP_AUTH_METHOD_NOT_SUPPORTED,
 				"no SASL mechanism provided" );
 			goto cleanup;
@@ -299,6 +279,54 @@ do_bind(
 
 		rs->sr_err = slap_sasl_bind( op, rs );
 
+		ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
+		if( rs->sr_err == LDAP_SUCCESS ) {
+			ber_dupbv(&op->o_conn->c_dn, &op->orb_edn);
+			if( !BER_BVISEMPTY( &op->orb_edn ) ) {
+				/* edn is always normalized already */
+				ber_dupbv( &op->o_conn->c_ndn, &op->o_conn->c_dn );
+			}
+			op->o_tmpfree( op->orb_edn.bv_val, op->o_tmpmemctx );
+			BER_BVZERO( &op->orb_edn );
+			op->o_conn->c_authmech = op->o_conn->c_sasl_bind_mech;
+			BER_BVZERO( &op->o_conn->c_sasl_bind_mech );
+			op->o_conn->c_sasl_bind_in_progress = 0;
+
+			op->o_conn->c_sasl_ssf = op->orb_ssf;
+			if( op->orb_ssf > op->o_conn->c_ssf ) {
+				op->o_conn->c_ssf = op->orb_ssf;
+			}
+
+			if( !BER_BVISEMPTY( &op->o_conn->c_dn ) ) {
+				ber_len_t max = sockbuf_max_incoming_auth;
+				ber_sockbuf_ctrl( op->o_conn->c_sb,
+					LBER_SB_OPT_SET_MAX_INCOMING, &max );
+			}
+
+			/* log authorization identity */
+			Statslog( LDAP_DEBUG_STATS,
+				"%s BIND dn=\"%s\" mech=%s ssf=%d\n",
+				op->o_log_prefix,
+				BER_BVISNULL( &op->o_conn->c_dn ) ? "<empty>" : op->o_conn->c_dn.bv_val,
+				op->o_conn->c_authmech.bv_val, op->orb_ssf, 0 );
+
+			Debug( LDAP_DEBUG_TRACE,
+				"do_bind: SASL/%s bind: dn=\"%s\" ssf=%d\n",
+				op->o_conn->c_authmech.bv_val,
+				BER_BVISNULL( &op->o_conn->c_dn ) ? "<empty>" : op->o_conn->c_dn.bv_val,
+				op->orb_ssf );
+
+		} else if ( rs->sr_err == LDAP_SASL_BIND_IN_PROGRESS ) {
+			op->o_conn->c_sasl_bind_in_progress = 1;
+
+		} else {
+			if ( !BER_BVISNULL( &op->o_conn->c_sasl_bind_mech ) ) {
+				free( op->o_conn->c_sasl_bind_mech.bv_val );
+				BER_BVZERO( &op->o_conn->c_sasl_bind_mech );
+			}
+			op->o_conn->c_sasl_bind_in_progress = 0;
+		}
+
 #ifdef LDAP_SLAPI
 #define	pb	op->o_pb
 		/*
@@ -310,24 +338,27 @@ do_bind(
 		 */
 		if ( pb ) {
 			slapi_int_pblock_set_operation( pb, op );
-			slapi_pblock_set( pb, SLAPI_BIND_TARGET, (void *)dn.bv_val );
+			slapi_pblock_set( pb, SLAPI_BIND_TARGET, (void *)op->o_req_dn.bv_val );
 			slapi_pblock_set( pb, SLAPI_BIND_METHOD, (void *)op->orb_method );
-			slapi_pblock_set( pb, SLAPI_BIND_CREDENTIALS, (void *)&op->orb_cred );
+			slapi_pblock_set( pb,
+				SLAPI_BIND_CREDENTIALS, (void *)&op->orb_cred );
 			slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)(0) );
-			(void) slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_POST_BIND_FN, pb );
+			(void) slapi_int_call_plugins( op->o_bd,
+				SLAPI_PLUGIN_POST_BIND_FN, pb );
 		}
 #endif /* LDAP_SLAPI */
 
+		ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
+
 		goto cleanup;
 
 	} else {
 		/* Not SASL, cancel any in-progress bind */
 		ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
 
-		if ( op->o_conn->c_sasl_bind_mech.bv_val != NULL ) {
-			free(op->o_conn->c_sasl_bind_mech.bv_val);
-			op->o_conn->c_sasl_bind_mech.bv_val = NULL;
-			op->o_conn->c_sasl_bind_mech.bv_len = 0;
+		if ( !BER_BVISNULL( &op->o_conn->c_sasl_bind_mech ) ) {
+			free( op->o_conn->c_sasl_bind_mech.bv_val );
+			BER_BVZERO( &op->o_conn->c_sasl_bind_mech );
 		}
 		op->o_conn->c_sasl_bind_in_progress = 0;
 
@@ -336,18 +367,18 @@ do_bind(
 	}
 
 	if ( op->orb_method == LDAP_AUTH_SIMPLE ) {
-		ber_str2bv( "SIMPLE", sizeof("SIMPLE")-1, 0, &mech );
+		BER_BVSTR( &mech, "SIMPLE" );
 		/* accept "anonymous" binds */
-		if ( op->orb_cred.bv_len == 0 || op->o_req_ndn.bv_len == 0 ) {
+		if ( BER_BVISEMPTY( &op->orb_cred ) || BER_BVISEMPTY( &op->o_req_ndn ) ) {
 			rs->sr_err = LDAP_SUCCESS;
 
-			if( op->orb_cred.bv_len &&
+			if( !BER_BVISEMPTY( &op->orb_cred ) &&
 				!( global_allows & SLAP_ALLOW_BIND_ANON_CRED ))
 			{
 				/* cred is not empty, disallow */
 				rs->sr_err = LDAP_INVALID_CREDENTIALS;
 
-			} else if ( op->o_req_ndn.bv_len &&
+			} else if ( !BER_BVISEMPTY( &op->o_req_ndn ) &&
 				!( global_allows & SLAP_ALLOW_BIND_ANON_DN ))
 			{
 				/* DN is not empty, disallow */
@@ -369,14 +400,8 @@ do_bind(
 			 * just need to send success
 			 */
 			send_ldap_result( op, rs );
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, DETAIL1, 
-				"do_bind: conn %d  v%d anonymous bind\n",
-				op->o_connid, version , 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "do_bind: v%d anonymous bind\n",
-				version, 0, 0 );
-#endif
+				op->o_protocol, 0, 0 );
 			goto cleanup;
 
 		} else if ( global_disallows & SLAP_DISALLOW_BIND_SIMPLE ) {
@@ -385,15 +410,9 @@ do_bind(
 			rs->sr_text = "unwilling to perform simple authentication";
 
 			send_ldap_result( op, rs );
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, 
-				"do_bind: conn %d  v%d simple bind(%s) disallowed\n",
-				op->o_connid, version, op->o_req_ndn.bv_val );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"do_bind: v%d simple bind(%s) disallowed\n",
-				version, op->o_req_ndn.bv_val, 0 );
-#endif
+				op->o_protocol, op->o_req_ndn.bv_val, 0 );
 			goto cleanup;
 		}
 
@@ -406,33 +425,21 @@ do_bind(
 
 			send_ldap_result( op, rs );
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, DETAIL1, 
-				"do_bind: conn %d  v%d Kerberos V4 (step 1) bind refused\n",
-				op->o_connid, version , 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"do_bind: v%d Kerberos V4 (step 1) bind refused\n",
-				version, 0, 0 );
-#endif
+				op->o_protocol, 0, 0 );
 			goto cleanup;
 		}
-		ber_str2bv( "KRBV4", sizeof("KRBV4")-1, 0, &mech );
+		BER_BVSTR( &mech, "KRBV4" );
 
 	} else if ( op->orb_method == LDAP_AUTH_KRBV42 ) {
 		rs->sr_err = LDAP_AUTH_METHOD_NOT_SUPPORTED;
 		rs->sr_text = "Kerberos V4 (step 2) bind not supported";
 		send_ldap_result( op, rs );
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, DETAIL1, 
-			"do_bind: conn %d  v%d Kerberos V4 (step 2) bind refused\n",
-			op->o_connid, version , 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"do_bind: v%d Kerberos V4 (step 2) bind refused\n",
-			version, 0, 0 );
-#endif
+			op->o_protocol, 0, 0 );
 		goto cleanup;
 #endif
 
@@ -441,15 +448,9 @@ do_bind(
 		rs->sr_text = "unknown authentication method";
 
 		send_ldap_result( op, rs );
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_bind: conn %ld  v%d unknown authentication method (%ld)\n",
-			op->o_connid, version, op->orb_method );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"do_bind: v%d unknown authentication method (%ld)\n",
-			version, op->orb_method, 0 );
-#endif
+			op->o_protocol, op->orb_method, 0 );
 		goto cleanup;
 	}
 
@@ -463,7 +464,9 @@ do_bind(
 		/* don't return referral for bind requests */
 		/* noSuchObject is not allowed to be returned by bind */
 		rs->sr_err = LDAP_INVALID_CREDENTIALS;
+		op->o_bd = frontendDB;
 		send_ldap_result( op, rs );
+		op->o_bd = NULL;
 		goto cleanup;
 	}
 
@@ -477,7 +480,7 @@ do_bind(
 	if ( pb ) {
 		int rc;
 		slapi_int_pblock_set_operation( pb, op );
-		slapi_pblock_set( pb, SLAPI_BIND_TARGET, (void *)dn.bv_val );
+		slapi_pblock_set( pb, SLAPI_BIND_TARGET, (void *)op->o_req_dn.bv_val );
 		slapi_pblock_set( pb, SLAPI_BIND_METHOD, (void *)op->orb_method );
 		slapi_pblock_set( pb, SLAPI_BIND_CREDENTIALS, (void *)&op->orb_cred );
 		slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)(0) );
@@ -485,15 +488,9 @@ do_bind(
 
 		rc = slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_PRE_BIND_FN, pb );
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO,
-			"do_bind: Bind preoperation plugin returned %d\n",
-			rs->sr_err, 0, 0);
-#else
 		Debug(LDAP_DEBUG_TRACE,
 			"do_bind: Bind preoperation plugin returned %d.\n",
 			rs->sr_err, 0, 0);
-#endif
 
 		switch ( rc ) {
 		case SLAPI_BIND_SUCCESS:
@@ -515,13 +512,12 @@ do_bind(
 				rs->sr_err = LDAP_OTHER;
 			}
 
-			op->orb_edn.bv_val = NULL;
-			op->orb_edn.bv_len = 0;
+			BER_BVZERO( &op->orb_edn );
 
 			if ( rs->sr_err == LDAP_SUCCESS ) {
 				slapi_pblock_get( pb, SLAPI_CONN_DN,
 					(void *)&op->orb_edn.bv_val );
-				if ( op->orb_edn.bv_val == NULL ) {
+				if ( BER_BVISNULL( &op->orb_edn ) ) {
 					if ( rc == 1 ) {
 						/* No plugins were called; continue. */
 						break;
@@ -535,22 +531,21 @@ do_bind(
 				ber_dupbv(&op->o_conn->c_dn, &op->o_req_dn);
 				ber_dupbv(&op->o_conn->c_ndn, &op->o_req_ndn);
 				op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
-				op->o_req_dn.bv_val = NULL;
-				op->o_req_dn.bv_len = 0;
+				BER_BVZERO( &op->o_req_dn );
 				op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-				op->o_req_ndn.bv_val = NULL;
-				op->o_req_ndn.bv_len = 0;
-				if ( op->o_conn->c_dn.bv_len != 0 ) {
+				BER_BVZERO( &op->o_req_ndn );
+				if ( !BER_BVISEMPTY( &op->o_conn->c_dn ) ) {
 					ber_len_t max = sockbuf_max_incoming_auth;
 					ber_sockbuf_ctrl( op->o_conn->c_sb,
 						LBER_SB_OPT_SET_MAX_INCOMING, &max );
 				}
 				/* log authorization identity */
 				Statslog( LDAP_DEBUG_STATS,
-					"conn=%lu op=%lu BIND dn=\"%s\" mech=%s (SLAPI) ssf=0\n",
-					op->o_connid, op->o_opid,
-					op->o_conn->c_dn.bv_val ? op->o_conn->c_dn.bv_val : "<empty>",
-					mech.bv_val, 0 );
+					"%s BIND dn=\"%s\" mech=%s (SLAPI) ssf=0\n",
+					op->o_log_prefix,
+					BER_BVISNULL( &op->o_conn->c_dn )
+						? "<empty>" : op->o_conn->c_dn.bv_val,
+					mech.bv_val, 0, 0 );
 				ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 			}
 			goto cleanup;
@@ -570,7 +565,7 @@ do_bind(
 			}
 
 			/* be_bind returns regular/global edn */
-			if( op->orb_edn.bv_len ) {
+			if( !BER_BVISEMPTY( &op->orb_edn ) ) {
 				op->o_conn->c_dn = op->orb_edn;
 			} else {
 				ber_dupbv(&op->o_conn->c_dn, &op->o_req_dn);
@@ -578,7 +573,7 @@ do_bind(
 
 			ber_dupbv( &op->o_conn->c_ndn, &op->o_req_ndn );
 
-			if( op->o_conn->c_dn.bv_len != 0 ) {
+			if( !BER_BVISEMPTY( &op->o_conn->c_dn ) ) {
 				ber_len_t max = sockbuf_max_incoming_auth;
 				ber_sockbuf_ctrl( op->o_conn->c_sb,
 					LBER_SB_OPT_SET_MAX_INCOMING, &max );
@@ -586,27 +581,22 @@ do_bind(
 
 			/* log authorization identity */
 			Statslog( LDAP_DEBUG_STATS,
-				"conn=%lu op=%lu BIND dn=\"%s\" mech=%s ssf=0\n",
-				op->o_connid, op->o_opid,
-				op->o_conn->c_dn.bv_val, mech.bv_val, 0 );
+				"%s BIND dn=\"%s\" mech=%s ssf=0\n",
+				op->o_log_prefix,
+				op->o_conn->c_dn.bv_val, mech.bv_val, 0, 0 );
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, DETAIL1, 
-				"do_bind: v%d bind: \"%s\" to \"%s\" \n",
-				version, op->o_conn->c_dn.bv_val, op->o_conn->c_dn.bv_val );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"do_bind: v%d bind: \"%s\" to \"%s\"\n",
-				version, dn.bv_val, op->o_conn->c_dn.bv_val );
-#endif
+				op->o_protocol, op->o_req_dn.bv_val, op->o_conn->c_dn.bv_val );
 
 			ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 
 			/* send this here to avoid a race condition */
 			send_ldap_result( op, rs );
 
-		} else if (op->orb_edn.bv_val != NULL) {
+		} else if ( !BER_BVISNULL( &op->orb_edn ) ) {
 			free( op->orb_edn.bv_val );
+			BER_BVZERO( &op->orb_edn );
 		}
 
 	} else {
@@ -618,34 +608,13 @@ do_bind(
 	if ( pb != NULL &&
 		slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_POST_BIND_FN, pb ) < 0 )
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO,
-			"do_bind: Bind postoperation plugins failed\n",
-			0, 0, 0);
-#else
 		Debug(LDAP_DEBUG_TRACE,
 			"do_bind: Bind postoperation plugins failed.\n",
 			0, 0, 0);
-#endif
 	}
 #endif /* LDAP_SLAPI */
 
-cleanup:
-	if ( rs->sr_err == LDAP_SUCCESS ) {
-		if ( op->orb_method != LDAP_AUTH_SASL ) {
-			ber_dupbv( &op->o_conn->c_authmech, &mech );
-		}
-		op->o_conn->c_authtype = op->orb_method;
-	}
-
-	if( op->o_req_dn.bv_val != NULL ) {
-		sl_free( op->o_req_dn.bv_val, op->o_tmpmemctx );
-		op->o_req_dn.bv_val = NULL;
-	}
-	if( op->o_req_ndn.bv_val != NULL ) {
-		sl_free( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-		op->o_req_ndn.bv_val = NULL;
-	}
-
+cleanup:;
 	return rs->sr_err;
 }
+
