@@ -1,8 +1,8 @@
 /* limits.c - routines to handle regex-based size and time limits */
-/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.30.2.11 2004/06/16 18:19:56 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.62.2.3 2005/05/10 16:07:02 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -22,6 +22,10 @@
 #include <ac/string.h>
 
 #include "slap.h"
+#include "lutil.h"
+
+/* define to get an error if requesting limit higher than hard */
+#undef ABOVE_HARD_LIMIT_IS_ERROR
 
 static char *
 limits2str( unsigned i )
@@ -71,16 +75,9 @@ limits_get(
 	assert( op );
 	assert( limit );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, DETAIL1, "==> limits_get: conn=%lu op=%lu dn=\"%s\"\n",
-			op->o_connid, op->o_opid,
-			BER_BVISNULL( ndn ) ? "[anonymous]" : ndn->bv_val );
- 
-#else
-	Debug( LDAP_DEBUG_TRACE, "==> limits_get: conn=%lu op=%lu dn=\"%s\"\n",
-			op->o_connid, op->o_opid,
-			BER_BVISNULL( ndn ) ? "[anonymous]" : ndn->bv_val );
-#endif
+	Debug( LDAP_DEBUG_TRACE, "==> limits_get: %s dn=\"%s\"\n",
+			op->o_log_prefix,
+			BER_BVISNULL( ndn ) ? "[anonymous]" : ndn->bv_val, 0 );
 	/*
 	 * default values
 	 */
@@ -109,33 +106,20 @@ limits_get(
 						lm[0]->lm_group_ad );
 				if ( rc == 0 ) {
 					*limit = &lm[0]->lm_limits;
-#ifdef NEW_LOGGING
-					LDAP_LOG( SLAPD, DETAIL1, "<== limits_get: type=GROUP match=EXACT "
-							"dn=\"%s\" oc=\"%s\" ad=\"%s\"\n",
-							lm[0]->lm_pat.bv_val,
-							lm[0]->lm_group_oc->soc_cname.bv_val,
-							lm[0]->lm_group_ad->ad_cname.bv_val );
-#else
 					Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=GROUP match=EXACT "
 							"dn=\"%s\" oc=\"%s\" ad=\"%s\"\n",
 							lm[0]->lm_pat.bv_val,
 							lm[0]->lm_group_oc->soc_cname.bv_val,
 							lm[0]->lm_group_ad->ad_cname.bv_val );
 
-#endif
 					return( 0 );
 				}
 			} else {
 			
 				if ( dn_match( &lm[0]->lm_pat, ndn ) ) {
 					*limit = &lm[0]->lm_limits;
-#ifdef NEW_LOGGING
-					LDAP_LOG( SLAPD, DETAIL1, "<== limits_get: type=DN match=EXACT dn=\"%s\"\n",
-							lm[0]->lm_pat.bv_val, 0, 0 );
-#else
 					Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=EXACT dn=\"%s\"\n",
 							lm[0]->lm_pat.bv_val, 0, 0 );
-#endif
 					return( 0 );
 				}
 			}
@@ -187,13 +171,8 @@ limits_get(
 				}
 
 				*limit = &lm[0]->lm_limits;
-#ifdef NEW_LOGGING
-				LDAP_LOG( SLAPD, DETAIL1, "<== limits_get: type=DN match=%s dn=\"%s\"\n",
-						limits2str( style ), lm[0]->lm_pat.bv_val, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=%s dn=\"%s\"\n",
 						limits2str( style ), lm[0]->lm_pat.bv_val, 0 );
-#endif
 				return( 0 );
 			}
 
@@ -208,26 +187,16 @@ limits_get(
 						0, NULL, 0 ) == 0 )
 			{
 				*limit = &lm[0]->lm_limits;
-#ifdef NEW_LOGGING
-				LDAP_LOG( SLAPD, DETAIL1, "<== limits_get: type=DN match=%s dn=\"%s\"\n",
-						limits2str( style ), lm[0]->lm_pat.bv_val, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=%s dn=\"%s\"\n",
 						limits2str( style ), lm[0]->lm_pat.bv_val, 0 );
-#endif
 				return( 0 );
 			}
 			break;
 
 		case SLAP_LIMITS_ANONYMOUS:
 			if ( ndn->bv_len == 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( SLAPD, DETAIL1, "<== limits_get: type=DN match=%s\n",
-						limits2str( style ), 0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=%s\n",
 						limits2str( style ), 0, 0 );
-#endif
 				*limit = &lm[0]->lm_limits;
 				return( 0 );
 			}
@@ -236,13 +205,8 @@ limits_get(
 		case SLAP_LIMITS_USERS:
 			if ( ndn->bv_len != 0 ) {
 				*limit = &lm[0]->lm_limits;
-#ifdef NEW_LOGGING
-				LDAP_LOG( SLAPD, DETAIL1, "<== limits_get: type=DN match=%s\n",
-						limits2str( style ), 0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=%s\n",
 						limits2str( style ), 0, 0 );
-#endif
 				return( 0 );
 			}
 			break;
@@ -381,16 +345,10 @@ limits_parse(
 	assert( be );
 
 	if ( argc < 3 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( CONFIG, CRIT, 
-			"%s : line %d: missing arg(s) in "
-			"\"limits <pattern> <limits>\" line.\n", fname, lineno, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"%s : line %d: missing arg(s) in "
 			"\"limits <pattern> <limits>\" line.\n%s",
 			fname, lineno, "" );
-#endif
 		return( -1 );
 	}
 
@@ -456,17 +414,10 @@ limits_parse(
 					pattern += STRLENOF( "level" );
 
 				} else {
-#ifdef NEW_LOGGING
-					LDAP_LOG( CONFIG, WARNING , 
-						"%s : line %d: deprecated \"one\" style "
-						"\"limits <pattern> <limits>\" line; "
-						"use \"onelevel\" instead.\n", fname, lineno, 0 );
-#else
 					Debug( LDAP_DEBUG_ANY,
 						"%s : line %d: deprecated \"one\" style "
 						"\"limits <pattern> <limits>\" line; "
 						"use \"onelevel\" instead.\n", fname, lineno, 0 );
-#endif
 				}
 
 			} else if ( strncasecmp( pattern, "sub", STRLENOF( "sub" ) ) == 0 ) {
@@ -476,17 +427,10 @@ limits_parse(
 					pattern += STRLENOF( "tree" );
 
 				} else {
-#ifdef NEW_LOGGING
-					LDAP_LOG( CONFIG, WARNING , 
-						"%s : line %d: deprecated \"sub\" style "
-						"\"limits <pattern> <limits>\" line; "
-						"use \"subtree\" instead.\n", fname, lineno, 0 );
-#else
 					Debug( LDAP_DEBUG_ANY,
 						"%s : line %d: deprecated \"sub\" style "
 						"\"limits <pattern> <limits>\" line; "
 						"use \"subtree\" instead.\n", fname, lineno, 0 );
-#endif
 				}
 
 			} else if ( strncasecmp( pattern, "children", STRLENOF( "children" ) ) == 0 ) {
@@ -518,13 +462,6 @@ limits_parse(
 
 		default:
 			if ( pattern[0] != '=' ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( CONFIG, CRIT, 
-					"%s : line %d: missing '=' in "
-					"\"dn[.{exact|base|onelevel|subtree"
-					"|children|regex|anonymous}]" "=<pattern>\" in "
-					"\"limits <pattern> <limits>\" line.\n", fname, lineno, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"%s : line %d: missing '=' in "
 					"\"dn[.{exact|base|onelevel|subtree"
@@ -533,7 +470,6 @@ limits_parse(
 					"\"limits <pattern> <limits>\" "
 					"line.\n%s",
 					fname, lineno, "" );
-#endif
 				return( -1 );
 			}
 
@@ -559,8 +495,12 @@ limits_parse(
 			struct berval	oc, ad;
 
 			oc.bv_val = pattern + 1;
+			pattern = strchr( pattern, '=' );
+			if ( pattern == NULL ) {
+				return -1;
+			}
 
-			ad.bv_val = strchr(pattern, '/');
+			ad.bv_val = strchr( oc.bv_val, '/' );
 			if ( ad.bv_val != NULL ) {
 				const char	*text = NULL;
 				int		rc;
@@ -568,18 +508,14 @@ limits_parse(
 				oc.bv_len = ad.bv_val - oc.bv_val;
 
 				ad.bv_val++;
-				ad.bv_len = strlen( ad.bv_val );
+				ad.bv_len = pattern - ad.bv_val;
 				rc = slap_bv2ad( &ad, &group_ad, &text );
 				if ( rc != LDAP_SUCCESS ) {
 					goto no_ad;
 				}
 
-				pattern = ad.bv_val + ad.bv_len;
-
 			} else {
-				oc.bv_len = strlen( oc.bv_val );
-
-				pattern = oc.bv_val + oc.bv_len;
+				oc.bv_len = pattern - oc.bv_val;
 			}
 
 			group_oc = oc_bvfind( &oc );
@@ -611,21 +547,12 @@ no_ad:;
 		flags = SLAP_LIMITS_TYPE_GROUP | SLAP_LIMITS_EXACT;
 
 		if ( pattern[0] != '=' ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( CONFIG, CRIT, 
-				"%s : line %d: missing '=' in "
-				"\"group[/objectClass[/attributeType]]"
-				"=<pattern>\" in "
-				"\"limits <pattern> <limits>\" line.\n",
-				fname, lineno, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"%s : line %d: missing '=' in "
 				"\"group[/objectClass[/attributeType]]"
 				"=<pattern>\" in "
 				"\"limits <pattern> <limits>\" line.\n",
 				fname, lineno, 0 );
-#endif
 			return( -1 );
 		}
 
@@ -637,17 +564,10 @@ no_ad:;
 	for ( i = 2; i < argc; i++ ) {
 		if ( limits_parse_one( argv[i], &limit ) ) {
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( CONFIG, CRIT, 
-				"%s : line %d: unknown limit values \"%s\" in "
-				"\"limits <pattern> <limits>\" line.\n",
-				fname, lineno, argv[i] );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"%s : line %d: unknown limit values \"%s\" in "
 				"\"limits <pattern> <limits>\" line.\n",
 			fname, lineno, argv[i] );
-#endif
 
 			return( 1 );
 		}
@@ -702,17 +622,10 @@ no_ad:;
 	rc = limits_add( be, flags, pattern, group_oc, group_ad, &limit );
 	if ( rc ) {
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( CONFIG, CRIT, 
-			"%s : line %d: unable to add limit in "
-			"\"limits <pattern> <limits>\" line.\n",
-			fname, lineno, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"%s : line %d: unable to add limit in "
 			"\"limits <pattern> <limits>\" line.\n",
 		fname, lineno, 0 );
-#endif
 	}
 
 	return( rc );
@@ -734,7 +647,7 @@ limits_parse_one(
 			arg++;
 			if ( strncasecmp( arg, "soft=", STRLENOF( "soft=" ) ) == 0 ) {
 				arg += STRLENOF( "soft=" );
-				if ( strcasecmp( arg, "none" ) == 0 ) {
+				if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_t_soft = -1;
 
 				} else {
@@ -750,7 +663,7 @@ limits_parse_one(
 					}
 
 					if ( soft == -1 ) {
-						/* FIXME: use "none" instead */
+						/* FIXME: use "unlimited" instead; issue warning? */
 					}
 
 					limit->lms_t_soft = soft;
@@ -761,7 +674,7 @@ limits_parse_one(
 				if ( strcasecmp( arg, "soft" ) == 0 ) {
 					limit->lms_t_hard = 0;
 
-				} else if ( strcasecmp( arg, "none" ) == 0 ) {
+				} else if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_t_hard = -1;
 
 				} else {
@@ -777,7 +690,7 @@ limits_parse_one(
 					}
 
 					if ( hard == -1 ) {
-						/* FIXME: use "none" instead */
+						/* FIXME: use "unlimited" instead */
 					}
 
 					if ( hard == 0 ) {
@@ -793,7 +706,7 @@ limits_parse_one(
 			
 		} else if ( arg[0] == '=' ) {
 			arg++;
-			if ( strcasecmp( arg, "none" ) == 0 ) {
+			if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 				limit->lms_t_soft = -1;
 
 			} else {
@@ -817,7 +730,7 @@ limits_parse_one(
 			arg++;
 			if ( strncasecmp( arg, "soft=", STRLENOF( "soft=" ) ) == 0 ) {
 				arg += STRLENOF( "soft=" );
-				if ( strcasecmp( arg, "none" ) == 0 ) {
+				if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_s_soft = -1;
 
 				} else {
@@ -833,7 +746,7 @@ limits_parse_one(
 					}
 
 					if ( soft == -1 ) {
-						/* FIXME: use "none" instead */
+						/* FIXME: use "unlimited" instead */
 					}
 
 					limit->lms_s_soft = soft;
@@ -844,7 +757,7 @@ limits_parse_one(
 				if ( strcasecmp( arg, "soft" ) == 0 ) {
 					limit->lms_s_hard = 0;
 
-				} else if ( strcasecmp( arg, "none" ) == 0 ) {
+				} else if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_s_hard = -1;
 
 				} else {
@@ -860,7 +773,7 @@ limits_parse_one(
 					}
 
 					if ( hard == -1 ) {
-						/* FIXME: use "none" instead */
+						/* FIXME: use "unlimited" instead */
 					}
 
 					if ( hard == 0 ) {
@@ -872,7 +785,7 @@ limits_parse_one(
 				
 			} else if ( strncasecmp( arg, "unchecked=", STRLENOF( "unchecked=" ) ) == 0 ) {
 				arg += STRLENOF( "unchecked=" );
-				if ( strcasecmp( arg, "none" ) == 0 ) {
+				if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_s_unchecked = -1;
 
 				} else if ( strcasecmp( arg, "disabled" ) == 0 ) {
@@ -891,7 +804,7 @@ limits_parse_one(
 					}
 
 					if ( unchecked == -1 ) {
-						/*  FIXME: use "none" instead */
+						/*  FIXME: use "unlimited" instead */
 					}
 
 					limit->lms_s_unchecked = unchecked;
@@ -902,7 +815,7 @@ limits_parse_one(
 				if ( strcasecmp( arg, "noEstimate" ) == 0 ) {
 					limit->lms_s_pr_hide = 1;
 
-				} else if ( strcasecmp( arg, "none" ) == 0 ) {
+				} else if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_s_pr = -1;
 
 				} else {
@@ -918,7 +831,7 @@ limits_parse_one(
 					}
 
 					if ( pr == -1 ) {
-						/* FIXME: use "none" instead */
+						/* FIXME: use "unlimited" instead */
 					}
 
 					limit->lms_s_pr = pr;
@@ -927,7 +840,7 @@ limits_parse_one(
 			} else if ( strncasecmp( arg, "prtotal=", STRLENOF( "prtotal=" ) ) == 0 ) {
 				arg += STRLENOF( "prtotal=" );
 
-				if ( strcasecmp( arg, "none" ) == 0 ) {
+				if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 					limit->lms_s_pr_total = -1;
 
 				} else if ( strcasecmp( arg, "disabled" ) == 0 ) {
@@ -950,7 +863,7 @@ limits_parse_one(
 					}
 
 					if ( total == -1 ) {
-						/* FIXME: use "none" instead */
+						/* FIXME: use "unlimited" instead */
 					}
 
 					if ( total == 0 ) {
@@ -966,7 +879,7 @@ limits_parse_one(
 			
 		} else if ( arg[0] == '=' ) {
 			arg++;
-			if ( strcasecmp( arg, "none" ) == 0 ) {
+			if ( strcasecmp( arg, "unlimited" ) == 0 || strcasecmp( arg, "none" ) == 0 ) {
 				limit->lms_s_soft = -1;
 
 			} else {
@@ -987,6 +900,177 @@ limits_parse_one(
 	return 0;
 }
 
+static const char *lmpats[] = {
+	"base",
+	"base",
+	"onelevel",
+	"subtree",
+	"children",
+	"regex",
+	"anonymous",
+	"users",
+	"*"
+};
+
+/* Caller must provide an adequately sized buffer in bv */
+void
+limits_unparse( struct slap_limits *lim, struct berval *bv )
+{
+	struct berval btmp;
+	char *ptr;
+	int lm;
+
+	if ( !bv || !bv->bv_val ) return;
+
+	ptr = bv->bv_val;
+
+	if (( lim->lm_flags & SLAP_LIMITS_TYPE_MASK ) == SLAP_LIMITS_TYPE_GROUP ) {
+		ptr = lutil_strcopy( ptr, "group/" );
+		ptr = lutil_strcopy( ptr, lim->lm_group_oc->soc_cname.bv_val );
+		*ptr++ = '/';
+		ptr = lutil_strcopy( ptr, lim->lm_group_ad->ad_cname.bv_val );
+		ptr = lutil_strcopy( ptr, "=\"" );
+		ptr = lutil_strcopy( ptr, lim->lm_pat.bv_val );
+		*ptr++ = '"';
+	} else {
+		lm = lim->lm_flags & SLAP_LIMITS_MASK;
+		switch( lm ) {
+		case SLAP_LIMITS_ANONYMOUS:
+		case SLAP_LIMITS_USERS:
+		case SLAP_LIMITS_ANY:
+			ptr = lutil_strcopy( ptr, lmpats[lm] );
+			break;
+		case SLAP_LIMITS_UNDEFINED:
+		case SLAP_LIMITS_EXACT:
+		case SLAP_LIMITS_ONE:
+		case SLAP_LIMITS_SUBTREE:
+		case SLAP_LIMITS_CHILDREN:
+		case SLAP_LIMITS_REGEX:
+			ptr = lutil_strcopy( ptr, "dn." );
+			ptr = lutil_strcopy( ptr, lmpats[lm] );
+			*ptr++ = '=';
+			*ptr++ = '"';
+			ptr = lutil_strcopy( ptr, lim->lm_pat.bv_val );
+			*ptr++ = '"';
+			break;
+		}
+	}
+	*ptr++ = ' ';
+	bv->bv_len = ptr - bv->bv_val;
+	btmp.bv_val = ptr;
+	btmp.bv_len = 0;
+	limits_unparse_one( &lim->lm_limits, SLAP_LIMIT_SIZE|SLAP_LIMIT_TIME, &btmp );
+	bv->bv_len += btmp.bv_len;
+}
+
+/* Caller must provide an adequately sized buffer in bv */
+void
+limits_unparse_one( struct slap_limits_set *lim, int which, struct berval *bv )
+{
+	char *ptr;
+
+	if ( !bv || !bv->bv_val ) return;
+
+	ptr = bv->bv_val;
+
+	if ( which & SLAP_LIMIT_SIZE ) {
+		if ( lim->lms_s_soft != SLAPD_DEFAULT_SIZELIMIT ) {
+
+			/* If same as global limit, drop it */
+			if ( lim != &frontendDB->be_def_limit &&
+				lim->lms_s_soft == frontendDB->be_def_limit.lms_s_soft )
+				goto s_hard;
+			/* If there's also a hard limit, fully qualify this one */
+			else if ( lim->lms_s_hard )
+				ptr = lutil_strcopy( ptr, " size.soft=" );
+
+			/* If doing both size & time, qualify this */
+			else if ( which & SLAP_LIMIT_TIME )
+				ptr = lutil_strcopy( ptr, " size=" );
+
+			if ( lim->lms_s_soft == -1 )
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			else
+				ptr += sprintf( ptr, "%d", lim->lms_s_soft );
+			*ptr++ = ' ';
+		}
+s_hard:
+		if ( lim->lms_s_hard ) {
+			ptr = lutil_strcopy( ptr, " size.hard=" );
+			if ( lim->lms_s_hard == -1 )
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			else
+				ptr += sprintf( ptr, "%d", lim->lms_s_hard );
+			*ptr++ = ' ';
+		}
+		if ( lim->lms_s_unchecked != -1 ) {
+			ptr = lutil_strcopy( ptr, " size.unchecked=" );
+			if ( lim->lms_s_unchecked == 0 )
+				ptr = lutil_strcopy( ptr, "disabled" );
+			else
+				ptr += sprintf( ptr, "%d", lim->lms_s_unchecked );
+			*ptr++ = ' ';
+		}
+		if ( lim->lms_s_pr_hide ) {
+			ptr = lutil_strcopy( ptr, " size.pr=noEstimate " );
+		}
+		if ( lim->lms_s_pr ) {
+			ptr = lutil_strcopy( ptr, " size.pr=" );
+			if ( lim->lms_s_pr == -1 )
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			else
+				ptr += sprintf( ptr, "%d", lim->lms_s_pr );
+			*ptr++ = ' ';
+		}
+		if ( lim->lms_s_pr_total ) {
+			ptr = lutil_strcopy( ptr, " size.prtotal=" );
+			if ( lim->lms_s_pr_total == -1 )
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			else if ( lim->lms_s_pr_total == -2 )
+				ptr = lutil_strcopy( ptr, "disabled" );
+			else 
+				ptr += sprintf( ptr, "%d", lim->lms_s_pr_total );
+			*ptr++ = ' ';
+		}
+	}
+	if ( which & SLAP_LIMIT_TIME ) {
+		if ( lim->lms_t_soft != SLAPD_DEFAULT_TIMELIMIT ) {
+
+			/* If same as global limit, drop it */
+			if ( lim != &frontendDB->be_def_limit &&
+				lim->lms_t_soft == frontendDB->be_def_limit.lms_t_soft )
+				goto t_hard;
+
+			/* If there's also a hard limit, fully qualify this one */
+			else if ( lim->lms_t_hard ) 
+				ptr = lutil_strcopy( ptr, " time.soft=" );
+
+			/* If doing both size & time, qualify this */
+			else if ( which & SLAP_LIMIT_SIZE )
+				ptr = lutil_strcopy( ptr, " time=" );
+
+			if ( lim->lms_t_soft == -1 )
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			else
+				ptr += sprintf( ptr, "%d", lim->lms_t_soft );
+			*ptr++ = ' ';
+		}
+t_hard:
+		if ( lim->lms_t_hard ) {
+			ptr = lutil_strcopy( ptr, " time.hard=" );
+			if ( lim->lms_t_hard == -1 )
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			else
+				ptr += sprintf( ptr, "%d", lim->lms_t_hard );
+			*ptr++ = ' ';
+		}
+	}
+	if ( ptr != bv->bv_val ) {
+		ptr--;
+		*ptr = '\0';
+		bv->bv_len = ptr - bv->bv_val;
+	}
+}
 
 int
 limits_check( Operation *op, SlapReply *rs )
@@ -995,7 +1079,20 @@ limits_check( Operation *op, SlapReply *rs )
 	assert( rs );
 	/* FIXME: should this be always true? */
 	assert( op->o_tag == LDAP_REQ_SEARCH);
-	
+
+	/* protocol only allows 0..maxInt;
+	 *
+	 * internal searches:
+	 * - may use SLAP_NO_LIMIT ( = -1 ) to indicate no limits;
+	 * - should use slimit = N and tlimit = SLAP_NO_LIMIT to
+	 *   indicate searches that should return exactly N matches,
+	 *   and handle errors thru a callback (see for instance
+	 *   slap_sasl_match() and slap_sasl2dn())
+	 */
+	if ( op->ors_tlimit == SLAP_NO_LIMIT && op->ors_slimit == SLAP_NO_LIMIT ) {
+		return 0;
+	}
+
 	/* allow root to set no limit */
 	if ( be_isroot( op ) ) {
 		op->ors_limit = NULL;
@@ -1022,18 +1119,27 @@ limits_check( Operation *op, SlapReply *rs )
 		} else {
 			if ( op->ors_limit->lms_t_hard == 0 ) {
 				if ( op->ors_limit->lms_t_soft > 0
-						&& ( op->ors_tlimit < 0 || op->ors_tlimit > op->ors_limit->lms_t_soft ) ) {
+						&& ( op->ors_tlimit > op->ors_limit->lms_t_soft ) ) {
 					op->ors_tlimit = op->ors_limit->lms_t_soft;
 				}
 
 			} else if ( op->ors_limit->lms_t_hard > 0 ) {
-				if ( op->ors_tlimit < 0 || op->ors_tlimit > op->ors_limit->lms_t_hard ) {
+#ifdef ABOVE_HARD_LIMIT_IS_ERROR
+				if ( op->ors_tlimit == SLAP_MAX_LIMIT ) {
+					op->ors_tlimit = op->ors_limit->lms_t_hard;
+
+				} else if ( op->ors_tlimit > op->ors_limit->lms_t_hard ) {
 					/* error if exceeding hard limit */
 					rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 					send_ldap_result( op, rs );
 					rs->sr_err = LDAP_SUCCESS;
 					return -1;
 				}
+#else /* ! ABOVE_HARD_LIMIT_IS_ERROR */
+				if ( op->ors_tlimit > op->ors_limit->lms_t_hard ) {
+					op->ors_tlimit = op->ors_limit->lms_t_hard;
+				}
+#endif /* ! ABOVE_HARD_LIMIT_IS_ERROR */
 			}
 		}
 
@@ -1048,9 +1154,10 @@ limits_check( Operation *op, SlapReply *rs )
 		}
 
 		/* if paged results is requested */	
-		if ( get_pagedresults( op ) > SLAP_NO_CONTROL ) {
+		if ( get_pagedresults( op ) > SLAP_CONTROL_IGNORED ) {
 			int	slimit = -2;
 			int	pr_total;
+			PagedResultsState *ps = op->o_pagedresults_state;
 
 			/* paged results is not allowed */
 			if ( op->ors_limit->lms_s_pr_total == -2 ) {
@@ -1062,7 +1169,7 @@ limits_check( Operation *op, SlapReply *rs )
 				return -1;
 			}
 			
-			if ( op->ors_limit->lms_s_pr > 0 && op->o_pagedresults_size > op->ors_limit->lms_s_pr ) {
+			if ( op->ors_limit->lms_s_pr > 0 && ps->ps_size > op->ors_limit->lms_s_pr ) {
 				rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 				rs->sr_text = "illegal pagedResults page size";
 				send_ldap_result( op, rs );
@@ -1082,13 +1189,22 @@ limits_check( Operation *op, SlapReply *rs )
 			}
 
 			if ( pr_total == -1 ) {
-				slimit = -1;
+				if ( op->ors_slimit == 0 || op->ors_slimit == SLAP_MAX_LIMIT ) {
+					slimit = -1;
 
-			} else if ( pr_total > 0 && ( op->ors_slimit == SLAP_NO_LIMIT || op->ors_slimit > pr_total ) ) {
+				} else {
+					slimit = op->ors_slimit - ps->ps_count;
+				}
+
+#ifdef ABOVE_HARD_LIMIT_IS_ERROR
+			} else if ( pr_total > 0 && op->ors_slimit != SLAP_MAX_LIMIT
+					&& ( op->ors_slimit == SLAP_NO_LIMIT || op->ors_slimit > pr_total ) )
+			{
 				rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 				send_ldap_result( op, rs );
 				rs->sr_err = LDAP_SUCCESS;
 				return -1;
+#endif /* ! ABOVE_HARD_LIMIT_IS_ERROR */
 	
 			} else {
 				/* if no limit is required, use soft limit */
@@ -1097,18 +1213,23 @@ limits_check( Operation *op, SlapReply *rs )
 
 				/* first round of pagedResults: set count to any appropriate limit */
 
-				/* if the limit is set, check that it does not violate any limit */
-				if ( op->ors_slimit > 0 ) {
-					slimit2 = op->ors_slimit;
+				/* if the limit is set, check that it does not violate any server-side limit */
+#ifdef ABOVE_HARD_LIMIT_IS_ERROR
+				if ( op->ors_slimit == SLAP_MAX_LIMIT ) {
+					slimit2 = op->ors_slimit = pr_total;
+#else /* ! ABOVE_HARD_LIMIT_IS_ERROR */
+				if ( op->ors_slimit == SLAP_MAX_LIMIT || op->ors_slimit > pr_total ) {
+					slimit2 = op->ors_slimit = pr_total;
+#endif /* ! ABOVE_HARD_LIMIT_IS_ERROR */
 
 				} else if ( op->ors_slimit == 0 ) {
 					slimit2 = pr_total;
 
 				} else {
-					slimit2 = -1;
+					slimit2 = op->ors_slimit;
 				}
 
-				total = slimit2 - op->o_pagedresults_state.ps_count;
+				total = slimit2 - ps->ps_count;
 
 				if ( total >= 0 ) {
 					if ( op->ors_limit->lms_s_pr > 0 ) {
@@ -1140,11 +1261,11 @@ limits_check( Operation *op, SlapReply *rs )
 		
 			/* if got any limit, use it */
 			if ( slimit != -2 ) {
-				if ( op->ors_slimit <= 0 ) {
+				if ( op->ors_slimit == 0 ) {
 					op->ors_slimit = slimit;
 
 				} else if ( slimit > 0 ) {
-					if ( op->ors_slimit - op->o_pagedresults_state.ps_count > slimit ) {
+					if ( op->ors_slimit - ps->ps_count > slimit ) {
 						rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 						send_ldap_result( op, rs );
 						rs->sr_err = LDAP_SUCCESS;
@@ -1173,13 +1294,22 @@ limits_check( Operation *op, SlapReply *rs )
 
 			/* explicit hard limit: error if violated */
 			} else if ( op->ors_limit->lms_s_hard > 0 ) {
-				if ( op->ors_slimit > op->ors_limit->lms_s_hard ) {
+#ifdef ABOVE_HARD_LIMIT_IS_ERROR
+				if ( op->ors_slimit == SLAP_MAX_LIMIT ) {
+					op->ors_slimit = op->ors_limit->lms_s_hard;
+
+				} else if ( op->ors_slimit > op->ors_limit->lms_s_hard ) {
 					/* if limit exceeds hard, error */
 					rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 					send_ldap_result( op, rs );
 					rs->sr_err = LDAP_SUCCESS;
 					return -1;
 				}
+#else /* ! ABOVE_HARD_LIMIT_IS_ERROR */
+				if ( op->ors_slimit > op->ors_limit->lms_s_hard ) {
+					op->ors_slimit = op->ors_limit->lms_s_hard;
+				}
+#endif /* ! ABOVE_HARD_LIMIT_IS_ERROR */
 			}
 		}
 
