@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.157.2.13 2004/10/09 04:26:50 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.198.2.5 2005/05/06 16:42:50 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,8 +34,6 @@
 #include <ac/wait.h>
 #include <ac/errno.h>
 
-#include "ldap_pvt.h"
-
 #include "slap.h"
 #include "lutil.h"
 #include "ldif.h"
@@ -65,7 +63,8 @@ static struct sockaddr_in	bind_addr;
 #endif
 
 typedef int (MainFunc) LDAP_P(( int argc, char *argv[] ));
-extern MainFunc slapadd, slapcat, slapdn, slapindex, slappasswd, slaptest;
+extern MainFunc slapadd, slapcat, slapdn, slapindex, slappasswd,
+	slaptest, slapauth, slapacl;
 
 static struct {
 	char *name;
@@ -77,6 +76,14 @@ static struct {
 	{"slapindex", slapindex},
 	{"slappasswd", slappasswd},
 	{"slaptest", slaptest},
+	{"slapauth", slapauth},
+	{"slapacl", slapacl},
+	/* NOTE: new tools must be added in chronological order,
+	 * not in alphabetical order, because for backwards
+	 * compatibility name[4] is used to identify the
+	 * tools; so name[4]=='a' must refer to "slapadd" and
+	 * not to "slapauth".  Alphabetical order can be used
+	 * for tools whose name[4] is not used yet */
 	{NULL, NULL}
 };
 
@@ -128,6 +135,55 @@ static int version = 0;
 
 void *slap_tls_ctx;
 
+static int
+slapd_opt_slp( const char *val, void *arg )
+{
+#ifdef HAVE_SLP
+	/* NULL is default */
+	if ( val == NULL || strcasecmp( val, "on" ) == 0 ) {
+		slapd_register_slp = 1;
+
+	} else if ( strcasecmp( val, "off" ) == 0 ) {
+		slapd_register_slp = 0;
+
+	/* NOTE: add support for URL specification? */
+
+	} else {
+		fprintf(stderr, "unrecognized value \"%s\" for SLP option\n", val );
+		return -1;
+	}
+
+	return 0;
+		
+#else
+	fputs( "slapd: SLP support is not available\n", stderr );
+	return 0;
+#endif
+}
+
+/*
+ * Option helper structure:
+ * 
+ * oh_nam	is left-hand part of <option>[=<value>]
+ * oh_fnc	is handler function
+ * oh_arg	is an optional arg to oh_fnc
+ * oh_usage	is the one-line usage string related to the option,
+ *		which is assumed to start with <option>[=<value>]
+ *
+ * please leave valid options in the structure, and optionally #ifdef
+ * their processing inside the helper, so that reasonable and helpful
+ * error messages can be generated if a disabled option is requested.
+ */
+struct option_helper {
+	struct berval	oh_name;
+	int		(*oh_fnc)(const char *val, void *arg);
+	void		*oh_arg;
+	const char	*oh_usage;
+} option_helpers[] = {
+	{ BER_BVC("slp"),	slapd_opt_slp,	NULL, "slp[={on|off}] enable/disable SLP" },
+	{ BER_BVNULL }
+};
+
 static void
 usage( char *name )
 {
@@ -136,10 +192,12 @@ usage( char *name )
 	fprintf( stderr,
 		"\t-4\t\tIPv4 only\n"
 		"\t-6\t\tIPv6 only\n"
-		"\t-T {acdipt}\tRun in Tool mode\n"
+		"\t-T {acl|add|auth|cat|dn|index|passwd|test}\n"
+		"\t\t\tRun in Tool mode\n"
 		"\t-c cookie\tSync cookie of consumer\n"
 		"\t-d level\tDebug level" "\n"
 		"\t-f filename\tConfiguration file\n"
+		"\t-F dir\tConfiguration directory\n"
 #if defined(HAVE_SETUID) && defined(HAVE_SETGID)
 		"\t-g group\tGroup (id or name) to run as\n"
 #endif
@@ -148,14 +206,26 @@ usage( char *name )
 		"\t-l facility\tSyslog facility (default: LOCAL4)\n"
 #endif
 		"\t-n serverName\tService name\n"
+		"\t-o <opt>[=val] generic means to specify options" );
+	if ( !BER_BVISNULL( &option_helpers[0].oh_name ) ) {
+		int	i;
+
+		fprintf( stderr, "; supported options:\n" );
+		for ( i = 0; !BER_BVISNULL( &option_helpers[i].oh_name ); i++) {
+			fprintf( stderr, "\t\t%s\n", option_helpers[i].oh_usage );
+		}
+	} else {
+		fprintf( stderr, "\n" );
+	}
+	fprintf( stderr,	
 #ifdef HAVE_CHROOT
 		"\t-r directory\tSandbox directory to chroot to\n"
 #endif
 		"\t-s level\tSyslog level\n"
 #if defined(HAVE_SETUID) && defined(HAVE_SETGID)
 		"\t-u user\t\tUser (id or name) to run as\n"
-		"\t-V\t\tprint version info (-VV only)\n"
 #endif
+		"\t-V\t\tprint version info (-VV only)\n"
     );
 }
 
@@ -182,15 +252,11 @@ int main( int argc, char **argv )
 	int g_argc = argc;
 	char **g_argv = argv;
 
-#ifdef HAVE_NT_SERVICE_MANAGER
-	char		*configfile = ".\\slapd.conf";
-#else
-	char		*configfile = SLAPD_DEFAULT_CONFIGFILE;
-#endif
+	char		*configfile = NULL;
+	char		*configdir = NULL;
 	char	    *serverName;
 	int	    serverMode = SLAP_SERVER_MODE;
 
-	struct berval cookie = BER_BVNULL;
 	struct sync_cookie *scp = NULL;
 	struct sync_cookie *scp_entry = NULL;
 
@@ -200,8 +266,10 @@ int main( int argc, char **argv )
 		leakfile = stderr;
 	}
 #endif
+	char	*serverNamePrefix = "";
+	size_t	l;
 
-	sl_mem_init();
+	slap_sl_mem_init();
 
 	serverName = lutil_progname( "slapd", argc, argv );
 
@@ -218,6 +286,7 @@ int main( int argc, char **argv )
 	{
 		int *i;
 		char *newConfigFile;
+		char *newConfigDir;
 		char *newUrls;
 		char *regService = NULL;
 
@@ -228,53 +297,38 @@ int main( int argc, char **argv )
 		}
 
 		i = (int*)lutil_getRegParam( regService, "DebugLevel" );
-		if ( i != NULL ) 
-		{
+		if ( i != NULL ) {
 			slap_debug = *i;
-#ifdef NEW_LOGGING
-			lutil_log_initialize( argc, argv );
-			LDAP_LOG( SLAPD, INFO, 
-				"main: new debug level from registry is: %d\n", 
-				slap_debug, 0, 0 );
-#else
-			Debug( LDAP_DEBUG_ANY, "new debug level from registry is: %d\n", slap_debug, 0, 0 );
-#endif
+			Debug( LDAP_DEBUG_ANY,
+				"new debug level from registry is: %d\n", slap_debug, 0, 0 );
 		}
 
 		newUrls = (char *) lutil_getRegParam(regService, "Urls");
-		if (newUrls)
-		{
+		if (newUrls) {
 		    if (urls)
 			ch_free(urls);
 
 		    urls = ch_strdup(newUrls);
-#ifdef NEW_LOGGING
-		    LDAP_LOG( SLAPD, INFO, 
-				"main: new urls from registry: %s\n", urls, 0, 0 );
-#else
 		    Debug(LDAP_DEBUG_ANY, "new urls from registry: %s\n",
-			  urls, 0, 0);
-#endif
-
+				urls, 0, 0);
 		}
 
 		newConfigFile = (char*)lutil_getRegParam( regService, "ConfigFile" );
-		if ( newConfigFile != NULL ) 
-		{
+		if ( newConfigFile != NULL ) {
 			configfile = newConfigFile;
-#ifdef NEW_LOGGING
-			LDAP_LOG( SLAPD, INFO, 
-				"main: new config file from registry is: %s\n", configfile, 0, 0 );
-#else
 			Debug ( LDAP_DEBUG_ANY, "new config file from registry is: %s\n", configfile, 0, 0 );
-#endif
+		}
 
+		newConfigDir = (char*)lutil_getRegParam( regService, "ConfigDir" );
+		if ( newConfigDir != NULL ) {
+			configdir = newConfigDir;
+			Debug ( LDAP_DEBUG_ANY, "new config dir from registry is: %s\n", configdir, 0, 0 );
 		}
 	}
 #endif
 
 	while ( (i = getopt( argc, argv,
-			     "c:d:f:h:s:n:tT:V"
+			     "c:d:f:F:h:n:o:s:StT:V"
 #if LDAP_PF_INET6
 				"46"
 #endif
@@ -306,21 +360,14 @@ int main( int argc, char **argv )
 		case 'c':	/* provide sync cookie, override if exist in replica */
 			scp = (struct sync_cookie *) ch_calloc( 1,
 										sizeof( struct sync_cookie ));
-			ber_str2bv( optarg, strlen( optarg ), 1, &cookie );
-			ber_bvarray_add( &scp->octet_str, &cookie );
+			ber_str2bv( optarg, 0, 1, &scp->octet_str );
 			slap_parse_sync_cookie( scp );
 
 			LDAP_STAILQ_FOREACH( scp_entry, &slap_sync_cookie, sc_next ) {
 				if ( scp->rid == scp_entry->rid ) {
-#ifdef NEW_LOGGING
-					LDAP_LOG( OPERATION, CRIT,
-							"main: duplicated replica id in cookies\n",
-							0, 0, 0 );
-#else
 					Debug( LDAP_DEBUG_ANY,
 						    "main: duplicated replica id in cookies\n",
 							0, 0, 0 );
-#endif
 					slap_sync_cookie_free( scp, 1 );
 					goto destroy;
 				}
@@ -343,6 +390,44 @@ int main( int argc, char **argv )
 			configfile = ch_strdup( optarg );
 			break;
 
+		case 'F':	/* use config dir */
+			configdir = ch_strdup( optarg );
+			break;
+
+		case 'o': {
+			char		*val = strchr( optarg, '=' );
+			struct berval	opt;
+			int		i;
+
+			opt.bv_val = optarg;
+			
+			if ( val ) {
+				opt.bv_len = ( val - optarg );
+				val++;
+			
+			} else {
+				opt.bv_len = strlen( optarg );
+			}
+
+			for ( i = 0; !BER_BVISNULL( &option_helpers[i].oh_name ); i++ ) {
+				if ( ber_bvstrcasecmp( &option_helpers[i].oh_name, &opt ) == 0 ) {
+					assert( option_helpers[i].oh_fnc != NULL );
+					if ( (*option_helpers[i].oh_fnc)( val, option_helpers[i].oh_arg ) == -1 ) {
+						/* we assume the option parsing helper
+						 * issues appropriate and self-explanatory
+						 * error messages... */
+						goto stop;
+					}
+					break;
+				}
+			}
+
+			if ( BER_BVISNULL( &option_helpers[i].oh_name ) ) {
+				goto unhandled_option;
+			}
+			break;
+		}
+
 		case 's':	/* set syslog level */
 			ldap_syslog = atoi( optarg );
 			break;
@@ -389,14 +474,31 @@ int main( int argc, char **argv )
 			break;
 
 		case 'T':
-			for (i=0; tools[i].name; i++) {
-				if ( optarg[0] == tools[i].name[4] ) {
-					rc = tools[i].func(argc, argv);
-					MAIN_RETURN(rc);
+			/* try full option string first */
+			for ( i = 0; tools[i].name; i++ ) {
+				if ( strcmp( optarg, &tools[i].name[4] ) == 0 ) {
+					rc = tools[i].func( argc, argv );
+					MAIN_RETURN( rc );
 				}
 			}
+
+			/* try bits of option string (backward compatibility for single char) */
+			l = strlen( optarg );
+			for ( i = 0; tools[i].name; i++ ) {
+				if ( strncmp( optarg, &tools[i].name[4], l ) == 0 ) {
+					rc = tools[i].func( argc, argv );
+					MAIN_RETURN( rc );
+				}
+			}
+			
+			/* issue error */
+			serverName = optarg;
+			serverNamePrefix = "slap";
+			fprintf( stderr, "program name \"%s%s\" unrecognized; "
+					"aborting...\n", serverNamePrefix, serverName );
 			/* FALLTHRU */
 		default:
+unhandled_option:;
 			usage( argv[0] );
 			rc = 1;
 			SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 15 );
@@ -404,14 +506,11 @@ int main( int argc, char **argv )
 		}
 	}
 
-#ifdef NEW_LOGGING
-	lutil_log_initialize( argc, argv );
-#else
-	lutil_set_debug_level( "slapd", slap_debug );
+	(void) ldap_pvt_thread_initialize();
+
 	ber_set_option(NULL, LBER_OPT_DEBUG_LEVEL, &slap_debug);
 	ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &slap_debug);
 	ldif_debug = slap_debug;
-#endif
 
 	if ( version ) {
 		fprintf( stderr, "%s\n", Versionstr );
@@ -437,11 +536,7 @@ int main( int argc, char **argv )
 #endif
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, INFO, "%s", Versionstr, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ANY, "%s", Versionstr, 0, 0 );
-#endif
 
 	if( check == CHECK_NONE && slapd_daemon_init( urls ) != 0 ) {
 		rc = 1;
@@ -483,13 +578,9 @@ int main( int argc, char **argv )
 #endif
 
 	if ( slap_schema_init( ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, CRIT, "main: schema initialization error\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "schema initialization error\n",
 		    0, 0, 0 );
-#endif
 
 		goto destroy;
 	}
@@ -501,13 +592,9 @@ int main( int argc, char **argv )
 	}
 
 	if ( slap_controls_init( ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, CRIT, "main: controls initialization error\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "controls initialization error\n",
 		    0, 0, 0 );
-#endif
 
 		goto destroy;
 	}
@@ -524,23 +611,29 @@ int main( int argc, char **argv )
 
 #ifdef LDAP_SLAPI
 	if ( slapi_int_initialize() != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, CRIT, "main: slapi initialization error\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "slapi initialization error\n",
 		    0, 0, 0 );
-#endif
 
 		goto destroy;
 	}
 #endif /* LDAP_SLAPI */
 
+	if ( frontend_init() ) {
+		goto destroy;
+	}
+
 	if ( overlay_init() ) {
 		goto destroy;
 	}
 
-	if ( read_config( configfile, 0 ) != 0 ) {
+#ifdef SLAP_DYNACL
+	if ( acl_init() ) {
+		goto destroy;
+	}
+#endif /* SLAP_DYNACL */
+
+	if ( read_config( configfile, configdir ) != 0 ) {
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 19 );
 
@@ -561,25 +654,10 @@ int main( int argc, char **argv )
 		}
 	}
 
-	if ( glue_sub_init( ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, "main: subordinate config error\n", 0, 0, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY,
-		    "subordinate config error\n",
-		    0, 0, 0 );
-#endif
-		goto destroy;
-	}
-
 	if ( slap_schema_check( ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, "main: schema prep error\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "schema prep error\n",
 		    0, 0, 0 );
-#endif
 
 		goto destroy;
 	}
@@ -587,13 +665,9 @@ int main( int argc, char **argv )
 #ifdef HAVE_TLS
 	rc = ldap_pvt_tls_init();
 	if( rc != 0) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, "main: tls init failed: %d\n", rc, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "main: TLS init failed: %d\n",
 		    0, 0, 0 );
-#endif
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 20 );
 		goto destroy;
@@ -610,13 +684,9 @@ int main( int argc, char **argv )
 
 		rc = ldap_pvt_tls_init_def_ctx();
 		if( rc != 0) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( SLAPD, CRIT, "main: tls init def ctx failed: %d\n", rc, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			    "main: TLS init def ctx failed: %d\n",
 			    rc, 0, 0 );
-#endif
 			rc = 1;
 			SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 20 );
 			goto destroy;
@@ -670,11 +740,7 @@ int main( int argc, char **argv )
 		goto shutdown;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, INFO, "main: slapd starting.\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ANY, "slapd starting\n", 0, 0, 0 );
-#endif
 
 
 	if ( slapd_pid_file != NULL ) {
@@ -707,7 +773,8 @@ int main( int argc, char **argv )
 
 #ifdef HAVE_NT_EVENT_LOG
 	if (is_NT_Service)
-	lutil_LogStartedEvent( serverName, slap_debug, configfile, urls );
+	lutil_LogStartedEvent( serverName, slap_debug, configfile ?
+		configfile : SLAPD_DEFAULT_CONFIGFILE , urls );
 #endif
 
 	rc = slapd_daemon();
@@ -746,11 +813,7 @@ stop:
 	lutil_LogStoppedEvent( serverName );
 #endif
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, CRIT, "main: slapd stopped.\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ANY, "slapd stopped.\n", 0, 0, 0 );
-#endif
 
 
 #ifdef HAVE_NT_SERVICE_MANAGER
