@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.182.2.14 2005/01/20 17:01:08 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.227.2.9 2005/06/09 17:25:55 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -31,7 +31,6 @@
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include "ldap_pvt.h"
 #include "slap.h"
 #ifdef LDAP_SLAPI
 #include "slapi/slapi.h"
@@ -50,20 +49,11 @@ do_modify(
 	ber_len_t	len;
 	Modifications	*modlist = NULL;
 	Modifications	**modtail = &modlist;
-#ifdef LDAP_DEBUG
-	Modifications *tmp;
-#endif
-#ifdef LDAP_SLAPI
-	LDAPMod		**modv = NULL;
-#endif
-	int manageDSAit;
-	int increment = 0;
+	int		increment = 0;
+	char		textbuf[ SLAP_TEXT_BUFLEN ];
+	size_t		textlen = sizeof( textbuf );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "do_modify: enter\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "do_modify\n", 0, 0, 0 );
-#endif
 
 	/*
 	 * Parse the modify request.  It looks like this:
@@ -85,21 +75,13 @@ do_modify(
 	 */
 
 	if ( ber_scanf( op->o_ber, "{m" /*}*/, &dn ) == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modify: ber_scanf failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modify: ber_scanf failed\n", 0, 0, 0 );
-#endif
 
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
 		return SLAPD_DISCONNECT;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ARGS, "do_modify: dn (%s)\n", dn.bv_val, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ARGS, "do_modify: dn (%s)\n", dn.bv_val, 0, 0 );
-#endif
 
 	/* collect modifications & save for later */
 	for ( tag = ber_first_element( op->o_ber, &len, &last );
@@ -122,6 +104,7 @@ do_modify(
 
 		mod = (Modifications *) ch_malloc( sizeof(Modifications) );
 		mod->sml_op = mop;
+		mod->sml_flags = 0;
 		mod->sml_type = tmp.sml_type;
 		mod->sml_values = tmp.sml_values;
 		mod->sml_nvalues = NULL;
@@ -132,15 +115,9 @@ do_modify(
 		switch( mop ) {
 		case LDAP_MOD_ADD:
 			if ( mod->sml_values == NULL ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, ERR, 
-					"do_modify: modify/add operation (%ld) requires values\n",
-					(long)mop, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"do_modify: modify/add operation (%ld) requires values\n",
 					(long) mop, 0, 0 );
-#endif
 
 				send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 					"modify/add operation requires values" );
@@ -157,15 +134,9 @@ do_modify(
 			if( op->o_protocol >= LDAP_VERSION3 ) {
 				increment++;
 				if ( mod->sml_values == NULL ) {
-#ifdef NEW_LOGGING
-					LDAP_LOG( OPERATION, ERR, "do_modify: "
-						"modify/increment operation (%ld) requires value\n",
-						(long)mop, 0, 0 );
-#else
 					Debug( LDAP_DEBUG_ANY, "do_modify: "
 						"modify/increment operation (%ld) requires value\n",
 						(long) mop, 0, 0 );
-#endif
 
 					send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 						"modify/increment operation requires value" );
@@ -173,15 +144,9 @@ do_modify(
 				}
 
 				if( mod->sml_values[1].bv_val ) {
-#ifdef NEW_LOGGING
-					LDAP_LOG( OPERATION, ERR, "do_modify: modify/increment "
-						"operation (%ld) requires single value\n",
-						(long)mop, 0, 0 );
-#else
 					Debug( LDAP_DEBUG_ANY, "do_modify: modify/increment "
 						"operation (%ld) requires single value\n",
 						(long) mop, 0, 0 );
-#endif
 
 					send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 						"modify/increment operation requires single value" );
@@ -193,15 +158,9 @@ do_modify(
 			/* fall thru */
 
 		default: {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, ERR, 
-					"do_modify: unrecognized modify operation (%ld)\n",
-					(long)mop, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"do_modify: unrecognized modify operation (%ld)\n",
 					(long) mop, 0, 0 );
-#endif
 
 				send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 					"unrecognized modify operation" );
@@ -214,11 +173,7 @@ do_modify(
 	*modtail = NULL;
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modify: get_ctrls failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modify: get_ctrls failed\n", 0, 0, 0 );
-#endif
 
 		goto cleanup;
 	}
@@ -226,36 +181,64 @@ do_modify(
 	rs->sr_err = dnPrettyNormal( NULL, &dn, &op->o_req_dn, &op->o_req_ndn,
 		op->o_tmpmemctx );
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modify: conn %d  invalid dn (%s)\n",
-			op->o_connid, dn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_modify: invalid dn (%s)\n", dn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid DN" );
 		goto cleanup;
 	}
 
-	if( op->o_req_ndn.bv_len == 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_modify: attempt to modify root DSE.\n",0, 0, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_modify: root dse!\n", 0, 0, 0 );
+	rs->sr_err = slap_mods_check( modlist, &rs->sr_text,
+		textbuf, textlen, NULL );
+
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		send_ldap_result( op, rs );
+		goto cleanup;
+	}
+
+	/* FIXME: needs review */
+	op->orm_modlist = modlist;
+	op->orm_increment = increment;
+
+	op->o_bd = frontendDB;
+	rs->sr_err = frontendDB->be_modify( op, rs );
+
+cleanup:
+	slap_graduate_commit_csn( op );
+
+	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
+	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
+	if ( op->orm_modlist != NULL ) slap_mods_free( op->orm_modlist );
+
+	return rs->sr_err;
+}
+
+int
+fe_op_modify( Operation *op, SlapReply *rs )
+{
+#ifdef LDAP_DEBUG
+	Modifications	*tmp;
 #endif
+	int		manageDSAit;
+	Modifications	*modlist = op->orm_modlist;
+	Modifications	**modtail = &modlist;
+#ifdef LDAP_SLAPI
+	LDAPMod		**modv = NULL;
+#endif
+	int		increment = op->orm_increment;
+	int		rc = 0;
+	BackendDB *op_be;
+	char		textbuf[ SLAP_TEXT_BUFLEN ];
+	size_t		textlen = sizeof( textbuf );
+	
+	if( op->o_req_ndn.bv_len == 0 ) {
+		Debug( LDAP_DEBUG_ANY, "do_modify: root dse!\n", 0, 0, 0 );
 
 		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 			"modify upon the root DSE not supported" );
 		goto cleanup;
 
-	} else if ( bvmatch( &op->o_req_ndn, &global_schemandn ) ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR,
-			"do_modify: attempt to modify subschema subentry.\n" , 0, 0, 0  );
-#else
+	} else if ( bvmatch( &op->o_req_ndn, &frontendDB->be_schemandn ) ) {
 		Debug( LDAP_DEBUG_ANY, "do_modify: subschema subentry!\n", 0, 0, 0 );
-#endif
 
 		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 			"modification of subschema subentry not supported" );
@@ -263,31 +246,9 @@ do_modify(
 	}
 
 #ifdef LDAP_DEBUG
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, "do_modify: modifications:\n", 0, 0, 0  );
-#else
 	Debug( LDAP_DEBUG_ARGS, "modifications:\n", 0, 0, 0 );
-#endif
 
 	for ( tmp = modlist; tmp != NULL; tmp = tmp->sml_next ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, DETAIL1, "\t%s:  %s\n", 
-			tmp->sml_op == LDAP_MOD_ADD ? "add" :
-				(tmp->sml_op == LDAP_MOD_INCREMENT ? "increment" :
-					(tmp->sml_op == LDAP_MOD_DELETE ? "delete" :
-						"replace")), tmp->sml_type.bv_val, 0 );
-
-		if ( tmp->sml_values == NULL ) {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tno values", 0, 0, 0 );
-		} else if ( tmp->sml_values[0].bv_val == NULL ) {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tzero values", 0, 0, 0 );
-		} else if ( tmp->sml_values[1].bv_val == NULL ) {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tone value", 0, 0, 0 );
-		} else {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tmultiple values", 0, 0, 0 );
-		}
-
-#else
 		Debug( LDAP_DEBUG_ARGS, "\t%s: %s\n",
 			tmp->sml_op == LDAP_MOD_ADD ? "add" :
 				(tmp->sml_op == LDAP_MOD_INCREMENT ? "increment" :
@@ -307,27 +268,26 @@ do_modify(
 			Debug( LDAP_DEBUG_ARGS, "%s\n",
 			   "\t\tmultiple values", NULL, NULL );
 		}
-#endif
 	}
 
 	if ( StatslogTest( LDAP_DEBUG_STATS ) ) {
 		char abuf[BUFSIZ/2], *ptr = abuf;
 		int len = 0;
 
-		Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD dn=\"%s\"\n",
-			op->o_connid, op->o_opid, dn.bv_val, 0, 0 );
+		Statslog( LDAP_DEBUG_STATS, "%s MOD dn=\"%s\"\n",
+			op->o_log_prefix, op->o_req_dn.bv_val, 0, 0, 0 );
 
 		for ( tmp = modlist; tmp != NULL; tmp = tmp->sml_next ) {
 			if (len + 1 + tmp->sml_type.bv_len > sizeof(abuf)) {
-				Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD attr=%s\n",
-				    op->o_connid, op->o_opid, abuf, 0, 0 );
+				Statslog( LDAP_DEBUG_STATS, "%s MOD attr=%s\n",
+				    op->o_log_prefix, abuf, 0, 0, 0 );
 
-	    		len = 0;
+	    			len = 0;
 				ptr = abuf;
 
 				if( 1 + tmp->sml_type.bv_len > sizeof(abuf)) {
-					Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD attr=%s\n",
-						op->o_connid, op->o_opid, tmp->sml_type.bv_val, 0, 0 );
+					Statslog( LDAP_DEBUG_STATS, "%s MOD attr=%s\n",
+						op->o_log_prefix, tmp->sml_type.bv_val, 0, 0, 0 );
 					continue;
 				}
 			}
@@ -339,8 +299,8 @@ do_modify(
 			len += tmp->sml_type.bv_len;
 		}
 		if (len) {
-			Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD attr=%s\n",
-	    			op->o_connid, op->o_opid, abuf, 0, 0 );
+			Statslog( LDAP_DEBUG_STATS, "%s MOD attr=%s\n",
+	    			op->o_log_prefix, abuf, 0, 0, 0 );
 		}
 	}
 #endif	/* LDAP_DEBUG */
@@ -352,7 +312,7 @@ do_modify(
 	 * appropriate one, or send a referral to our "referral server"
 	 * if we don't hold it.
 	 */
-	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 0 );
+	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 1 );
 	if ( op->o_bd == NULL ) {
 		rs->sr_ref = referral_rewrite( default_referral,
 			NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
@@ -360,16 +320,26 @@ do_modify(
 
 		if (rs->sr_ref != NULL ) {
 			rs->sr_err = LDAP_REFERRAL;
+			op->o_bd = frontendDB;
 			send_ldap_result( op, rs );
+			op->o_bd = NULL;
 
 			if (rs->sr_ref != default_referral) ber_bvarray_free( rs->sr_ref );
 		} else {
+			op->o_bd = frontendDB;
 			send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 				"no global superior knowledge" );
+			op->o_bd = NULL;
 		}
 		goto cleanup;
 	}
 
+	/* If we've got a glued backend, check the real backend */
+	op_be = op->o_bd;
+	if ( SLAP_GLUE_INSTANCE( op->o_bd )) {
+		op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 0 );
+	}
+
 	/* check restrictions */
 	if( backend_check_restrictions( op, rs, NULL ) != LDAP_SUCCESS ) {
 		send_ldap_result( op, rs );
@@ -381,6 +351,15 @@ do_modify(
 		goto cleanup;
 	}
 
+	{
+		rs->sr_err = slap_mods_obsolete_check( op, modlist,
+			&rs->sr_text, textbuf, textlen );
+		if ( rs->sr_err != LDAP_SUCCESS ) {
+			send_ldap_result( op, rs );
+			goto cleanup;
+		}
+	}
+
 	/* check for modify/increment support */
 	if( increment && !SLAP_INCREMENT( op->o_bd ) ) {
 		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
@@ -391,7 +370,7 @@ do_modify(
 #define pb	op->o_pb
 	if ( pb ) {
 		slapi_int_pblock_set_operation( pb, op );
-		slapi_pblock_set( pb, SLAPI_MODIFY_TARGET, (void *)dn.bv_val );
+		slapi_pblock_set( pb, SLAPI_MODIFY_TARGET, (void *)op->o_req_dn.bv_val );
 		slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
 		modv = slapi_int_modifications2ldapmods( &modlist );
 		slapi_pblock_set( pb, SLAPI_MODIFY_MODS, (void *)modv );
@@ -415,15 +394,9 @@ do_modify(
 			 * A preoperation plugin failure will abort the
 			 * entire operation.
 			 */
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO,
-				"do_modify: modify preoperation plugin failed\n",
-				0, 0, 0 );
-#else
 			Debug(LDAP_DEBUG_TRACE,
 				"do_modify: modify preoperation plugin failed.\n",
 				0, 0, 0);
-#endif
 			if ( ( slapi_pblock_get( op->o_pb, SLAPI_RESULT_CODE,
 				(void *)&rs->sr_err ) != 0 ) || rs->sr_err == LDAP_SUCCESS )
 			{
@@ -465,17 +438,18 @@ do_modify(
 		if ( !SLAP_SHADOW(op->o_bd) || repl_user )
 #endif
 		{
-			int update = op->o_bd->be_update_ndn.bv_len;
-			char textbuf[SLAP_TEXT_BUFLEN];
-			size_t textlen = sizeof textbuf;
-			slap_callback cb = { NULL, slap_replog_cb, NULL, NULL };
+			int		update = !BER_BVISEMPTY( &op->o_bd->be_update_ndn );
+			slap_callback	cb = { NULL, slap_replog_cb, NULL, NULL };
 
-			rs->sr_err = slap_mods_check( modlist, update, &rs->sr_text,
-				textbuf, textlen, NULL );
+			op->o_bd = op_be;
 
-			if( rs->sr_err != LDAP_SUCCESS ) {
-				send_ldap_result( op, rs );
-				goto cleanup;
+			if ( !update ) {
+				rs->sr_err = slap_mods_no_user_mod_check( op, modlist,
+					&rs->sr_text, textbuf, textlen );
+				if ( rs->sr_err != LDAP_SUCCESS ) {
+					send_ldap_result( op, rs );
+					goto cleanup;
+				}
 			}
 
 			if ( !repl_user ) {
@@ -499,7 +473,8 @@ do_modify(
 			if ( !repl_user )
 #endif
 			{
-				/* but we log only the ones not from a replicator user */
+				/* but multimaster slapd logs only the ones 
+				 * not from a replicator user */
 				cb.sc_next = op->o_callback;
 				op->o_callback = &cb;
 			}
@@ -514,12 +489,17 @@ do_modify(
 				rs->sr_ref = referral_rewrite( defref,
 					NULL, &op->o_req_dn,
 					LDAP_SCOPE_DEFAULT );
-				if (!rs->sr_ref) rs->sr_ref = defref;
+				if ( rs->sr_ref == NULL ) {
+					/* FIXME: must duplicate, because
+					 * overlays may muck with it */
+					rs->sr_ref = defref;
+				}
 				rs->sr_err = LDAP_REFERRAL;
 				send_ldap_result( op, rs );
-				if (rs->sr_ref != defref) {
+				if ( rs->sr_ref != defref ) {
 					ber_bvarray_free( rs->sr_ref );
 				}
+
 			} else {
 				send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 					"shadow context; no update referral" );
@@ -535,34 +515,96 @@ do_modify(
 	if ( pb != NULL && slapi_int_call_plugins( op->o_bd,
 		SLAPI_PLUGIN_POST_MODIFY_FN, pb ) < 0 )
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO,
-			"do_modify: modify postoperation plugins failed\n", 0, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_TRACE,
 			"do_modify: modify postoperation plugins failed.\n", 0, 0, 0);
-#endif
 	}
 #endif /* defined( LDAP_SLAPI ) */
 
-cleanup:
-	slap_graduate_commit_csn( op );
-
-	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
-	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-	if ( modlist != NULL ) slap_mods_free( modlist );
+cleanup:;
 #if defined( LDAP_SLAPI )
 	if ( modv != NULL ) slapi_int_free_ldapmods( modv );
 #endif
+
 	return rs->sr_err;
 }
 
+/*
+ * Obsolete constraint checking.
+ */
+int
+slap_mods_obsolete_check(
+	Operation *op,
+	Modifications *ml,
+	const char **text,
+	char *textbuf,
+	size_t textlen )
+{
+	if( get_manageDIT( op ) ) return LDAP_SUCCESS;
+
+	for ( ; ml != NULL; ml = ml->sml_next ) {
+		if ( is_at_obsolete( ml->sml_desc->ad_type ) &&
+			(( ml->sml_op != LDAP_MOD_REPLACE &&
+				ml->sml_op != LDAP_MOD_DELETE ) ||
+					ml->sml_values != NULL ))
+		{
+			/*
+			 * attribute is obsolete,
+			 * only allow replace/delete with no values
+			 */
+			snprintf( textbuf, textlen,
+				"%s: attribute is obsolete",
+				ml->sml_type.bv_val );
+			*text = textbuf;
+			return LDAP_CONSTRAINT_VIOLATION;
+		}
+	}
+
+	return LDAP_SUCCESS;
+}
+
+/*
+ * No-user-modification constraint checking.
+ */
+int
+slap_mods_no_user_mod_check(
+	Operation *op,
+	Modifications *ml,
+	const char **text,
+	char *textbuf,
+	size_t textlen )
+{
+	for ( ; ml != NULL; ml = ml->sml_next ) {
+		if ( !is_at_no_user_mod( ml->sml_desc->ad_type ) ) continue;
+
+		if( get_manageDIT( op )) {
+			if ( ml->sml_desc->ad_type->sat_flags & SLAP_AT_MANAGEABLE ) {
+				continue;
+			}
+
+			/* attribute not manageable */
+			snprintf( textbuf, textlen,
+				"%s: no-user-modification attribute not manageable",
+				ml->sml_type.bv_val );
+
+		} else {
+			/* user modification disallowed */
+			snprintf( textbuf, textlen,
+				"%s: no user modification allowed",
+				ml->sml_type.bv_val );
+		}
+
+		*text = textbuf;
+		return LDAP_CONSTRAINT_VIOLATION;
+	}
+
+	return LDAP_SUCCESS;
+}
+
 /*
  * Do basic attribute type checking and syntax validation.
  */
 int slap_mods_check(
 	Modifications *ml,
-	int update,
 	const char **text,
 	char *textbuf,
 	size_t textlen,
@@ -616,15 +658,7 @@ int slap_mods_check(
 			return LDAP_UNDEFINED_TYPE;
 		}
 
-		if (!update && is_at_no_user_mod( ad->ad_type )) {
-			/* user modification disallowed */
-			snprintf( textbuf, textlen,
-				"%s: no user modification allowed",
-				ml->sml_type.bv_val );
-			*text = textbuf;
-			return LDAP_CONSTRAINT_VIOLATION;
-		}
-
+#if 0
 		if ( is_at_obsolete( ad->ad_type ) &&
 			(( ml->sml_op != LDAP_MOD_REPLACE &&
 				ml->sml_op != LDAP_MOD_DELETE ) ||
@@ -640,6 +674,7 @@ int slap_mods_check(
 			*text = textbuf;
 			return LDAP_CONSTRAINT_VIOLATION;
 		}
+#endif
 
 		if ( ml->sml_op == LDAP_MOD_INCREMENT &&
 #ifdef SLAPD_REAL_SYNTAX
@@ -740,15 +775,9 @@ int slap_mods_check(
 						ad->ad_type->sat_equality,
 						&ml->sml_values[nvals], &ml->sml_nvalues[nvals], ctx );
 					if( rc ) {
-#ifdef NEW_LOGGING
-						LDAP_LOG( OPERATION, DETAIL1,
-							"str2entry:  NULL (ssyn_normalize %d)\n",
-							rc, 0, 0 );
-#else
 						Debug( LDAP_DEBUG_ANY,
 							"<= str2entry NULL (ssyn_normalize %d)\n",
 							rc, 0, 0 );
-#endif
 						snprintf( textbuf, textlen,
 							"%s: value #%ld normalization failed",
 							ml->sml_type.bv_val, (long) nvals );
@@ -761,59 +790,37 @@ int slap_mods_check(
 				ml->sml_nvalues[nvals].bv_len = 0;
 			}
 
-			if( nvals ) {
-				/* check for duplicates */
-				int		i, j;
+			/* check for duplicates, but ignore Deletes.
+			 */
+			if( nvals > 1 && ml->sml_op != LDAP_MOD_DELETE ) {
+				int		i, j, rc, match;
 				MatchingRule *mr = ad->ad_type->sat_equality;
 
-				/* check if the values we're adding already exist */
-				if( mr == NULL || !mr->smr_match ) {
-					for ( i = 1; ml->sml_values[i].bv_val != NULL; i++ ) {
-						/* test asserted values against themselves */
-						for( j = 0; j < i; j++ ) {
-							if ( bvmatch( &ml->sml_values[i],
-								&ml->sml_values[j] ) )
-							{
-								/* value exists already */
-								snprintf( textbuf, textlen,
-									"%s: value #%d provided more than once",
-									ml->sml_desc->ad_cname.bv_val, j );
-								*text = textbuf;
-								return LDAP_TYPE_OR_VALUE_EXISTS;
-							}
-						}
-					}
+				for ( i = 1; i < nvals ; i++ ) {
+					/* test asserted values against themselves */
+					for( j = 0; j < i; j++ ) {
+						rc = ordered_value_match( &match, ml->sml_desc, mr,
+							SLAP_MR_EQUALITY
+								| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX
+								| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
+								| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
+							ml->sml_nvalues
+								? &ml->sml_nvalues[i]
+								: &ml->sml_values[i],
+							ml->sml_nvalues
+								? &ml->sml_nvalues[j]
+								: &ml->sml_values[j],
+							text );
+						if ( rc == LDAP_SUCCESS && match == 0 ) {
+							/* value exists already */
+							snprintf( textbuf, textlen,
+								"%s: value #%d provided more than once",
+								ml->sml_desc->ad_cname.bv_val, j );
+							*text = textbuf;
+							return LDAP_TYPE_OR_VALUE_EXISTS;
 
-				} else {
-					int rc;
-					int match;
-
-					for ( i = 1; ml->sml_values[i].bv_val != NULL; i++ ) {
-						/* test asserted values against themselves */
-						for( j = 0; j < i; j++ ) {
-							rc = value_match( &match, ml->sml_desc, mr,
-								SLAP_MR_EQUALITY
-									| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX
-									| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
-									| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
-								ml->sml_nvalues
-									? &ml->sml_nvalues[i]
-									: &ml->sml_values[i],
-								ml->sml_nvalues
-									? &ml->sml_nvalues[j]
-									: &ml->sml_values[j],
-								text );
-							if ( rc == LDAP_SUCCESS && match == 0 ) {
-								/* value exists already */
-								snprintf( textbuf, textlen,
-									"%s: value #%d provided more than once",
-									ml->sml_desc->ad_cname.bv_val, j );
-								*text = textbuf;
-								return LDAP_TYPE_OR_VALUE_EXISTS;
-
-							} else if ( rc != LDAP_SUCCESS ) {
-								return rc;
-							}
+						} else if ( rc != LDAP_SUCCESS ) {
+							return rc;
 						}
 					}
 				}
@@ -825,6 +832,28 @@ int slap_mods_check(
 	return LDAP_SUCCESS;
 }
 
+/* Enter with bv->bv_len = sizeof buffer, returns with
+ * actual length of string
+ */
+void slap_timestamp( time_t *tm, struct berval *bv )
+{
+	struct tm *ltm;
+#ifdef HAVE_GMTIME_R
+	struct tm ltm_buf;
+
+	ltm = gmtime_r( tm, &ltm_buf );
+#else
+	ldap_pvt_thread_mutex_lock( &gmtime_mutex );
+	ltm = gmtime( tm );
+#endif
+
+	bv->bv_len = lutil_gentime( bv->bv_val, bv->bv_len, ltm );
+
+#ifndef HAVE_GMTIME_R
+	ldap_pvt_thread_mutex_unlock( &gmtime_mutex );
+#endif
+}
+
 int slap_mods_opattrs(
 	Operation *op,
 	Modifications *mods,
@@ -846,32 +875,17 @@ int slap_mods_opattrs(
 	assert( *modtail == NULL );
 
 	if ( SLAP_LASTMOD( op->o_bd )) {
-		struct tm *ltm;
-#ifdef HAVE_GMTIME_R
-		struct tm ltm_buf;
-#endif
 		time_t now = slap_get_time();
 
-#ifdef HAVE_GMTIME_R
-		ltm = gmtime_r( &now, &ltm_buf );
-#else
-		ldap_pvt_thread_mutex_lock( &gmtime_mutex );
-		ltm = gmtime( &now );
-#endif /* HAVE_GMTIME_R */
-		lutil_gentime( timebuf, sizeof(timebuf), ltm );
-
 		slap_get_csn( op, csnbuf, sizeof(csnbuf), &csn, manage_ctxcsn );
 
-#ifndef HAVE_GMTIME_R
-		ldap_pvt_thread_mutex_unlock( &gmtime_mutex );
-#endif
-
 		timestamp.bv_val = timebuf;
-		timestamp.bv_len = strlen(timebuf);
+		timestamp.bv_len = sizeof(timebuf);
+
+		slap_timestamp( &now, &timestamp );
 
 		if( op->o_dn.bv_len == 0 ) {
-			name.bv_val = SLAPD_ANONYMOUS;
-			name.bv_len = sizeof(SLAPD_ANONYMOUS)-1;
+			BER_BVSTR( &name, SLAPD_ANONYMOUS );
 			nname = name;
 		} else {
 			name = op->o_dn;
@@ -882,13 +896,14 @@ int slap_mods_opattrs(
 	if( op->o_tag == LDAP_REQ_ADD ) {
 		struct berval tmpval;
 
-		if( global_schemacheck ) {
+		{
 			int rc = mods_structural_class( mods, &tmpval,
 				text, textbuf, textlen );
 			if( rc != LDAP_SUCCESS ) return rc;
 
 			mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 			mod->sml_op = mop;
+			mod->sml_flags = SLAP_MOD_INTERNAL;
 			mod->sml_type.bv_val = NULL;
 			mod->sml_desc = slap_schema.si_ad_structuralObjectClass;
 			mod->sml_values =
@@ -915,6 +930,7 @@ int slap_mods_opattrs(
 		
 			mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 			mod->sml_op = mop;
+			mod->sml_flags = SLAP_MOD_INTERNAL;
 			mod->sml_type.bv_val = NULL;
 			mod->sml_desc = slap_schema.si_ad_entryUUID;
 			mod->sml_values =
@@ -937,6 +953,7 @@ int slap_mods_opattrs(
 
 			mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 			mod->sml_op = mop;
+			mod->sml_flags = SLAP_MOD_INTERNAL;
 			mod->sml_type.bv_val = NULL;
 			mod->sml_desc = slap_schema.si_ad_creatorsName;
 			mod->sml_values =
@@ -956,6 +973,7 @@ int slap_mods_opattrs(
 
 			mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 			mod->sml_op = mop;
+			mod->sml_flags = SLAP_MOD_INTERNAL;
 			mod->sml_type.bv_val = NULL;
 			mod->sml_desc = slap_schema.si_ad_createTimestamp;
 			mod->sml_values =
@@ -973,6 +991,7 @@ int slap_mods_opattrs(
 	if ( SLAP_LASTMOD( op->o_bd )) {
 		mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 		mod->sml_op = mop;
+		mod->sml_flags = SLAP_MOD_INTERNAL;
 		mod->sml_type.bv_val = NULL;
 		mod->sml_desc = slap_schema.si_ad_entryCSN;
 		mod->sml_values = (BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
@@ -986,6 +1005,7 @@ int slap_mods_opattrs(
 
 		mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 		mod->sml_op = mop;
+		mod->sml_flags = SLAP_MOD_INTERNAL;
 		mod->sml_type.bv_val = NULL;
 		mod->sml_desc = slap_schema.si_ad_modifiersName;
 		mod->sml_values = (BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
@@ -1004,6 +1024,7 @@ int slap_mods_opattrs(
 
 		mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 		mod->sml_op = mop;
+		mod->sml_flags = SLAP_MOD_INTERNAL;
 		mod->sml_type.bv_val = NULL;
 		mod->sml_desc = slap_schema.si_ad_modifyTimestamp;
 		mod->sml_values = (BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
