@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modrdn.c,v 1.114.2.13 2005/01/20 17:01:08 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/modrdn.c,v 1.146.2.6 2005/06/08 21:50:16 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -38,7 +38,6 @@
 #include <ac/socket.h>
 #include <ac/string.h>
 
-#include "ldap_pvt.h"
 #include "slap.h"
 #ifdef LDAP_SLAPI
 #include "slapi/slapi.h"
@@ -50,31 +49,18 @@ do_modrdn(
     SlapReply	*rs
 )
 {
-	struct berval dn = BER_BVNULL;
-	struct berval newrdn = BER_BVNULL;
-	struct berval newSuperior = BER_BVNULL;
+	struct berval	dn = BER_BVNULL;
+	struct berval	newrdn = BER_BVNULL;
+	struct berval	newSuperior = BER_BVNULL;
 	ber_int_t	deloldrdn;
 
 	struct berval pnewSuperior = BER_BVNULL;
 
 	struct berval nnewSuperior = BER_BVNULL;
 
-	Backend	*newSuperior_be = NULL;
 	ber_len_t	length;
-	int manageDSAit;
 
-	struct berval pdn = BER_BVNULL;
-	struct berval org_req_dn = BER_BVNULL;
-	struct berval org_req_ndn = BER_BVNULL;
-	struct berval org_dn = BER_BVNULL;
-	struct berval org_ndn = BER_BVNULL;
-	int	org_managedsait;
-
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "do_modrdn: begin\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "do_modrdn\n", 0, 0, 0 );
-#endif
 
 
 	/*
@@ -91,11 +77,7 @@ do_modrdn(
 	if ( ber_scanf( op->o_ber, "{mmb", &dn, &newrdn, &deloldrdn )
 	    == LBER_ERROR )
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modrdn: ber_scanf failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "ber_scanf failed\n", 0, 0, 0 );
-#endif
 
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
 		return SLAPD_DISCONNECT;
@@ -108,14 +90,9 @@ do_modrdn(
 			/* Conection record indicates v2 but field 
 			 * newSuperior is present: report error.
 			 */
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-				"do_modrdn: (v2) invalid field newSuperior.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			    "modrdn(v2): invalid field newSuperior!\n",
 			    0, 0, 0 );
-#endif
 
 			send_ldap_discon( op, rs,
 				LDAP_PROTOCOL_ERROR, "newSuperior requires LDAPv3" );
@@ -126,13 +103,8 @@ do_modrdn(
 		if ( ber_scanf( op->o_ber, "m", &newSuperior ) 
 		     == LBER_ERROR ) {
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-				"do_modrdn: ber_scanf(\"m\") failed\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "ber_scanf(\"m\") failed\n",
 				0, 0, 0 );
-#endif
 
 			send_ldap_discon( op, rs,
 				LDAP_PROTOCOL_ERROR, "decoding error" );
@@ -143,24 +115,13 @@ do_modrdn(
 		op->orr_nnewSup = &nnewSuperior;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ARGS, 
-		"do_modrdn: dn (%s) newrdn (%s) newsuperior(%s)\n",
-		dn.bv_val, newrdn.bv_val,
-		newSuperior.bv_len ? newSuperior.bv_val : "" );
-#else
 	Debug( LDAP_DEBUG_ARGS,
 	    "do_modrdn: dn (%s) newrdn (%s) newsuperior (%s)\n",
 		dn.bv_val, newrdn.bv_val,
 		newSuperior.bv_len ? newSuperior.bv_val : "" );
-#endif
 
 	if ( ber_scanf( op->o_ber, /*{*/ "}") == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modrdn: ber_scanf failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modrdn: ber_scanf failed\n", 0, 0, 0 );
-#endif
 
 		send_ldap_discon( op, rs,
 			LDAP_PROTOCOL_ERROR, "decoding error" );
@@ -169,11 +130,7 @@ do_modrdn(
 	}
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modrdn: get_ctrls failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modrdn: get_ctrls failed\n", 0, 0, 0 );
-#endif
 
 		/* get_ctrls has sent results.	Now clean up. */
 		goto cleanup;
@@ -181,69 +138,25 @@ do_modrdn(
 
 	rs->sr_err = dnPrettyNormal( NULL, &dn, &op->o_req_dn, &op->o_req_ndn, op->o_tmpmemctx );
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_modrdn: conn %d  invalid dn (%s)\n",
-			op->o_connid, dn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_modrdn: invalid dn (%s)\n", dn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid DN" );
 		goto cleanup;
 	}
 
-	if( op->o_req_ndn.bv_len == 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR,
-			"do_modrdn:  attempt to modify root DSE.\n", 0, 0, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_modrdn: root dse!\n", 0, 0, 0 );
-#endif
-
-		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
-			"cannot rename the root DSE" );
-		goto cleanup;
-
-	} else if ( bvmatch( &op->o_req_ndn, &global_schemandn ) ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR,
-			"do_modrdn: attempt to modify subschema subentry: %s (%ld)\n",
-			global_schemandn.bv_val, (long) global_schemandn.bv_len, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_modrdn: subschema subentry: %s (%ld)\n",
-			global_schemandn.bv_val, (long) global_schemandn.bv_len, 0 );
-#endif
-
-		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
-			"cannot rename subschema subentry" );
-		goto cleanup;
-	}
-
 	/* FIXME: should have/use rdnPretty / rdnNormalize routines */
 
 	rs->sr_err = dnPrettyNormal( NULL, &newrdn, &op->orr_newrdn, &op->orr_nnewrdn, op->o_tmpmemctx );
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_modrdn: conn %d  invalid newrdn (%s)\n",
-			op->o_connid, newrdn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_modrdn: invalid newrdn (%s)\n", newrdn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid new RDN" );
 		goto cleanup;
 	}
 
-	if( rdnValidate( &op->orr_newrdn ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_modrdn: invalid rdn (%s).\n", op->orr_newrdn.bv_val, 0, 0 );
-#else
+	if( rdn_validate( &op->orr_newrdn ) != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "do_modrdn: invalid rdn (%s)\n",
 			op->orr_newrdn.bv_val, 0, 0 );
-#endif
 
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid new RDN" );
 		goto cleanup;
@@ -253,22 +166,63 @@ do_modrdn(
 		rs->sr_err = dnPrettyNormal( NULL, &newSuperior, &pnewSuperior,
 			&nnewSuperior, op->o_tmpmemctx );
 		if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, 
-				"do_modrdn: conn %d  invalid newSuperior (%s)\n",
-				op->o_connid, newSuperior.bv_val, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"do_modrdn: invalid newSuperior (%s)\n",
 				newSuperior.bv_val, 0, 0 );
-#endif
 			send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid newSuperior" );
 			goto cleanup;
 		}
 	}
 
-	Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MODRDN dn=\"%s\"\n",
-	    op->o_connid, op->o_opid, op->o_req_dn.bv_val, 0, 0 );
+	/* FIXME: temporary? */
+	op->orr_deleteoldrdn = deloldrdn;
+
+	op->o_bd = frontendDB;
+	rs->sr_err = frontendDB->be_modrdn( op, rs );
+
+cleanup:
+	slap_graduate_commit_csn( op );
+
+	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
+	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
+
+	op->o_tmpfree( op->orr_newrdn.bv_val, op->o_tmpmemctx );	
+	op->o_tmpfree( op->orr_nnewrdn.bv_val, op->o_tmpmemctx );	
+
+	if ( !BER_BVISNULL( &pnewSuperior ) ) 
+		op->o_tmpfree( pnewSuperior.bv_val, op->o_tmpmemctx );
+	if ( !BER_BVISNULL( &nnewSuperior ) )
+		op->o_tmpfree( nnewSuperior.bv_val, op->o_tmpmemctx );
+
+	return rs->sr_err;
+}
+
+int
+fe_op_modrdn( Operation *op, SlapReply *rs )
+{
+	Backend		*newSuperior_be = NULL;
+	int		manageDSAit;
+	struct berval	pdn = BER_BVNULL;
+	BackendDB *op_be;
+	
+	if( op->o_req_ndn.bv_len == 0 ) {
+		Debug( LDAP_DEBUG_ANY, "do_modrdn: root dse!\n", 0, 0, 0 );
+
+		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+			"cannot rename the root DSE" );
+		goto cleanup;
+
+	} else if ( bvmatch( &op->o_req_ndn, &frontendDB->be_schemandn ) ) {
+		Debug( LDAP_DEBUG_ANY, "do_modrdn: subschema subentry: %s (%ld)\n",
+			frontendDB->be_schemandn.bv_val, (long)frontendDB->be_schemandn.bv_len, 0 );
+
+		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+			"cannot rename subschema subentry" );
+		goto cleanup;
+	}
+
+	Statslog( LDAP_DEBUG_STATS, "%s MODRDN dn=\"%s\"\n",
+	    op->o_log_prefix, op->o_req_dn.bv_val, 0, 0, 0 );
 
 	manageDSAit = get_manageDSAit( op );
 
@@ -277,7 +231,7 @@ do_modrdn(
 	 * appropriate one, or send a referral to our "referral server"
 	 * if we don't hold it.
 	 */
-	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 0 );
+	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 1 );
 	if ( op->o_bd == NULL ) {
 		rs->sr_ref = referral_rewrite( default_referral,
 			NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
@@ -285,16 +239,26 @@ do_modrdn(
 
 		if ( rs->sr_ref != NULL ) {
 			rs->sr_err = LDAP_REFERRAL;
+			op->o_bd = frontendDB;
 			send_ldap_result( op, rs );
+			op->o_bd = NULL;
 
 			if (rs->sr_ref != default_referral) ber_bvarray_free( rs->sr_ref );
 		} else {
+			op->o_bd = frontendDB;
 			send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 				"no global superior knowledge" );
+			op->o_bd = NULL;
 		}
 		goto cleanup;
 	}
 
+	/* If we've got a glued backend, check the real backend */
+	op_be = op->o_bd;
+	if ( SLAP_GLUE_INSTANCE( op->o_bd )) {
+		op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 0 );
+	}
+
 	/* check restrictions */
 	if( backend_check_restrictions( op, rs, NULL ) != LDAP_SUCCESS ) {
 		send_ldap_result( op, rs );
@@ -310,7 +274,7 @@ do_modrdn(
 	 * the same backend, otherwise we return an error.
 	 */
 	if( op->orr_newSup ) {
-		newSuperior_be = select_backend( &nnewSuperior, 0, 0 );
+		newSuperior_be = select_backend( op->orr_nnewSup, 0, 0 );
 
 		if ( newSuperior_be != op->o_bd ) {
 			/* newSuperior is in different backend */
@@ -325,11 +289,11 @@ do_modrdn(
 #define	pb	op->o_pb
 	if ( pb ) {
 		slapi_int_pblock_set_operation( pb, op );
-		slapi_pblock_set( pb, SLAPI_MODRDN_TARGET, (void *)dn.bv_val );
-		slapi_pblock_set( pb, SLAPI_MODRDN_NEWRDN, (void *)newrdn.bv_val );
+		slapi_pblock_set( pb, SLAPI_MODRDN_TARGET, (void *)op->o_req_dn.bv_val );
+		slapi_pblock_set( pb, SLAPI_MODRDN_NEWRDN, (void *)op->orr_newrdn.bv_val );
 		slapi_pblock_set( pb, SLAPI_MODRDN_NEWSUPERIOR,
-				(void *)newSuperior.bv_val );
-		slapi_pblock_set( pb, SLAPI_MODRDN_DELOLDRDN, (void *)deloldrdn );
+				(void *)op->orr_newSup->bv_val );
+		slapi_pblock_set( pb, SLAPI_MODRDN_DELOLDRDN, (void *)op->orr_deleteoldrdn);
 		slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
 
 		rs->sr_err = slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_PRE_MODRDN_FN, pb );
@@ -338,13 +302,8 @@ do_modrdn(
 			 * A preoperation plugin failure will abort the
 			 * entire operation.
 			 */
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, "do_modrdn: modrdn preoperation plugin "
-					"failed\n", 0, 0, 0 );
-#else
 			Debug(LDAP_DEBUG_TRACE, "do_modrdn: modrdn preoperation plugin "
 					"failed.\n", 0, 0, 0);
-#endif
 			if ( ( slapi_pblock_get( pb, SLAPI_RESULT_CODE, (void *)&rs->sr_err ) != 0 ) ||
 				 rs->sr_err == LDAP_SUCCESS ) {
 				rs->sr_err = LDAP_OTHER;
@@ -368,7 +327,9 @@ do_modrdn(
 #endif
 		{
 			slap_callback cb = { NULL, slap_replog_cb, NULL, NULL };
-			op->orr_deleteoldrdn = deloldrdn;
+
+			op->o_bd = op_be;
+
 #ifdef SLAPD_MULTIMASTER
 			if ( !op->o_bd->be_update_ndn.bv_len || !repl_user )
 #endif
@@ -379,6 +340,12 @@ do_modrdn(
 			op->o_bd->be_modrdn( op, rs );
 
 			if ( op->o_bd->be_delete ) {
+				struct berval	org_req_dn = BER_BVNULL;
+				struct berval	org_req_ndn = BER_BVNULL;
+				struct berval	org_dn = BER_BVNULL;
+				struct berval	org_ndn = BER_BVNULL;
+				int		org_managedsait;
+
 				org_req_dn = op->o_req_dn;
 				org_req_ndn = op->o_req_ndn;
 				org_dn = op->o_dn;
@@ -386,11 +353,11 @@ do_modrdn(
 				org_managedsait = get_manageDSAit( op );
 				op->o_dn = op->o_bd->be_rootdn;
 				op->o_ndn = op->o_bd->be_rootndn;
-				op->o_managedsait = 1;
+				op->o_managedsait = SLAP_CONTROL_NONCRITICAL;
 
 				while ( rs->sr_err == LDAP_SUCCESS &&
 						op->o_delete_glue_parent ) {
-	                op->o_delete_glue_parent = 0;
+					op->o_delete_glue_parent = 0;
 					if ( !be_issuffix( op->o_bd, &op->o_req_ndn )) {
 						slap_callback cb = { NULL };
 						cb.sc_response = slap_null_cb;
@@ -404,7 +371,7 @@ do_modrdn(
 					}
 				}
 				op->o_managedsait = org_managedsait;
-	            op->o_dn = org_dn;
+	            		op->o_dn = org_dn;
 				op->o_ndn = org_ndn;
 				op->o_req_dn = org_req_dn;
 				op->o_req_ndn = org_req_ndn;
@@ -438,29 +405,12 @@ do_modrdn(
 
 #if defined( LDAP_SLAPI )
 	if ( pb != NULL && slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_POST_MODRDN_FN, pb ) < 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modrdn: modrdn postoperation plugins "
-				"failed\n", 0, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_TRACE, "do_modrdn: modrdn postoperation plugins "
 				"failed.\n", 0, 0, 0);
-#endif
 	}
 #endif /* defined( LDAP_SLAPI ) */
 
-cleanup:
-
-	slap_graduate_commit_csn( op );
-
-	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
-	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-
-	op->o_tmpfree( op->orr_newrdn.bv_val, op->o_tmpmemctx );	
-	op->o_tmpfree( op->orr_nnewrdn.bv_val, op->o_tmpmemctx );	
-
-	if ( pnewSuperior.bv_val ) op->o_tmpfree( pnewSuperior.bv_val, op->o_tmpmemctx );
-	if ( nnewSuperior.bv_val ) op->o_tmpfree( nnewSuperior.bv_val, op->o_tmpmemctx );
-
+cleanup:;
 	return rs->sr_err;
 }
 
@@ -491,34 +441,20 @@ slap_modrdn2mods(
 		rs->sr_err = slap_bv2ad( &new_rdn[a_cnt]->la_attr, &desc, &rs->sr_text );
 
 		if ( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, 
-				"slap_modrdn2modlist: %s: %s (new)\n", 
-				rs->sr_text, 
-				new_rdn[ a_cnt ]->la_attr.bv_val, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"slap_modrdn2modlist: %s: %s (new)\n",
 				rs->sr_text, 
 				new_rdn[ a_cnt ]->la_attr.bv_val, 0 );
-#endif
 			goto done;		
 		}
 
 		/* ACL check of newly added attrs */
 		if ( op->o_bd && !access_allowed( op, e, desc,
-			&new_rdn[a_cnt]->la_value, ACL_WRITE, NULL ) ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, 
-				"slap_modrdn2modlist: access to attr \"%s\" "
-				"(new) not allowed\n", 
-				new_rdn[a_cnt]->la_attr.bv_val, 0, 0 );
-#else
+			&new_rdn[a_cnt]->la_value, ACL_WADD, NULL ) ) {
 			Debug( LDAP_DEBUG_TRACE,
 				"slap_modrdn2modlist: access to attr \"%s\" "
 				"(new) not allowed\n", 
 				new_rdn[ a_cnt ]->la_attr.bv_val, 0, 0 );
-#endif
 			rs->sr_text = "access to naming attributes (new) not allowed";
 			rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
 			goto done;
@@ -544,6 +480,7 @@ slap_modrdn2mods(
 			mod_tmp->sml_nvalues = NULL;
 		}
 		mod_tmp->sml_op = SLAP_MOD_SOFTADD;
+		mod_tmp->sml_flags = SLAP_MOD_INTERNAL;
 		mod_tmp->sml_next = mod;
 		mod = mod_tmp;
 	}
@@ -556,39 +493,23 @@ slap_modrdn2mods(
 
 			rs->sr_err = slap_bv2ad( &old_rdn[d_cnt]->la_attr, &desc, &rs->sr_text );
 			if ( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"slap_modrdn2modlist: %s: %s (old)\n", 
-					rs->sr_text, 
-					old_rdn[d_cnt]->la_attr.bv_val, 
-					0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 					"slap_modrdn2modlist: %s: %s (old)\n",
 					rs->sr_text, 
 					old_rdn[d_cnt]->la_attr.bv_val, 
 					0 );
-#endif
 				goto done;		
 			}
 
-			/* ACL check of newly added attrs */
+			/* ACL check of old rdn attrs removal */
 			if ( op->o_bd && !access_allowed( op, e, desc,
-				&old_rdn[d_cnt]->la_value, ACL_WRITE, 
+				&old_rdn[d_cnt]->la_value, ACL_WDEL, 
 				NULL ) ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"slap_modrdn2modlist: access "
-					"to attr \"%s\" (old) not allowed\n", 
-					old_rdn[ d_cnt ]->la_attr.bv_val, 
-					0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 					"slap_modrdn2modlist: access "
 					"to attr \"%s\" (old) not allowed\n", 
 					old_rdn[ d_cnt ]->la_attr.bv_val,
 					0, 0 );
-#endif
 				rs->sr_text = "access to naming attributes (old) not allowed";
 				rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
 				goto done;
@@ -614,6 +535,7 @@ slap_modrdn2mods(
 				mod_tmp->sml_nvalues = NULL;
 			}
 			mod_tmp->sml_op = LDAP_MOD_DELETE;
+			mod_tmp->sml_flags = SLAP_MOD_INTERNAL;
 			mod_tmp->sml_next = mod;
 			mod = mod_tmp;
 		}
