@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/module.c,v 1.21.2.2 2003/12/01 00:58:35 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/module.c,v 1.26.2.1 2005/01/20 18:03:55 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2003 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -63,12 +63,7 @@ int module_init (void)
 		__etoa( ebuf );
 		error = ebuf;
 #endif
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, 
-			"module_init: lt_dlinit failed: %s\n", error, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_ANY, "lt_dlinit failed: %s\n", error, 0, 0);
-#endif
 
 		return -1;
 	}
@@ -89,12 +84,7 @@ int module_kill (void)
 		__etoa( ebuf );
 		error = ebuf;
 #endif
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, "module_kill: lt_dlexit failed: %s\n", 
-			error, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_ANY, "lt_dlexit failed: %s\n", error, 0, 0);
-#endif
 
 		return -1;
 	}
@@ -115,13 +105,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 
 	module = (module_loaded_t *)ch_calloc(1, sizeof(module_loaded_t));
 	if (module == NULL) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, 
-			"module_load:  (%s) out of memory.\n", file_name, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_ANY, "module_load failed: (%s) out of memory\n", file_name,
 			0, 0);
-#endif
 
 		return -1;
 	}
@@ -142,24 +127,14 @@ int module_load(const char* file_name, int argc, char *argv[])
 		__etoa( ebuf );
 		error = ebuf;
 #endif
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, 
-			"module_load: lt_dlopenext failed: (%s) %s.\n", 
-			file_name, error, 0 );
-#else
 		Debug(LDAP_DEBUG_ANY, "lt_dlopenext failed: (%s) %s\n", file_name,
 			error, 0);
-#endif
 
 		ch_free(module);
 		return -1;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, INFO, "module_load: loaded module %s\n", file_name, 0, 0 );
-#else
 	Debug(LDAP_DEBUG_CONFIG, "loaded module %s\n", file_name, 0, 0);
-#endif
 
    
 #ifdef HAVE_EBCDIC
@@ -169,14 +144,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 #ifdef HAVE_EBCDIC
 #pragma convlit(resume)
 #endif
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load: module %s : no init_module() function found\n",
-		    file_name, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: no init_module() function found\n",
 			file_name, 0, 0);
-#endif
 
 		lt_dlclose(module->lib);
 		ch_free(module);
@@ -200,13 +169,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 	 */
 	rc = initialize(argc, argv);
 	if (rc == -1) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load:  module %s init_module() failed\n", file_name, 0, 0);
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: init_module() failed\n",
 			file_name, 0, 0);
-#endif
 
 		lt_dlclose(module->lib);
 		ch_free(module);
@@ -216,14 +180,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 	if (rc >= (int)(sizeof(module_regtable) / sizeof(struct module_regtable_t))
 		|| module_regtable[rc].proc == NULL)
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load: module %s: unknown registration type (%d).\n", 
-			file_name, rc, 0);
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: unknown registration type (%d)\n",
 			file_name, rc, 0);
-#endif
 
 		module_unload(module);
 		return -1;
@@ -231,14 +189,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 
 	rc = (module_regtable[rc].proc)(module, file_name);
 	if (rc != 0) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load: module %s:%s could not be registered.\n",
-			file_name, module_regtable[rc].type, 0 );
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: %s module could not be registered\n",
 			file_name, module_regtable[rc].type, 0);
-#endif
 
 		module_unload(module);
 		return rc;
@@ -247,14 +199,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 	module->next = module_list;
 	module_list = module;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, INFO, 
-		"module_load: module %s:%s registered\n", file_name,
-		module_regtable[rc].type, 0 );
-#else
 	Debug(LDAP_DEBUG_CONFIG, "module %s: %s module registered\n",
 		file_name, module_regtable[rc].type, 0);
-#endif
 
 	return 0;
 }
