@@ -1,5 +1,5 @@
 /* passwd.c - password extended operation routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/passwd.c,v 1.53.2.16 2005/03/14 19:34:31 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/passwd.c,v 1.95.2.7 2005/06/08 21:50:16 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -23,6 +23,10 @@
 #include <ac/string.h>
 #include <ac/unistd.h>
 
+#ifdef SLAPD_CRYPT
+#include <ac/crypt.h>
+#endif
+
 #include "slap.h"
 
 #include <lber_pvt.h>
@@ -50,6 +54,8 @@ int passwd_extop(
 	slap_callback cb2 = { NULL, slap_replog_cb, NULL, NULL };
 	int i, nhash;
 	char **hashes;
+	int	rc;
+	BackendDB *op_be;
 
 	cb2.sc_next = &cb;
 
@@ -72,18 +78,19 @@ int passwd_extop(
 		return rs->sr_err;
 	}
 
-	if ( id.bv_len ) {
-		op->o_req_dn = id;
-		/* ndn is in tmpmem, so we don't need to free it */
-		rs->sr_err = dnNormalize( 0, NULL, NULL, &id, &op->o_req_ndn, op->o_tmpmemctx );
+	if ( !BER_BVISEMPTY( &id ) ) {
+		rs->sr_err = dnPrettyNormal( NULL, &id, &op->o_req_dn,
+				&op->o_req_ndn, op->o_tmpmemctx );
 		if ( rs->sr_err != LDAP_SUCCESS ) {
 			rs->sr_text = "Invalid DN";
-			return rs->sr_err;
+			rc = rs->sr_err;
+			goto error_return;
 		}
-		op->o_bd = select_backend( &op->o_req_ndn, 0, 0 );
+		op->o_bd = select_backend( &op->o_req_ndn, 0, 1 );
+
 	} else {
-		op->o_req_dn = op->o_dn;
-		op->o_req_ndn = op->o_ndn;
+		ber_dupbv_x( &op->o_req_dn, &op->o_dn, op->o_tmpmemctx );
+		ber_dupbv_x( &op->o_req_ndn, &op->o_ndn, op->o_tmpmemctx );
 		ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
 		op->o_bd = op->o_conn->c_authz_backend;
 		ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
@@ -91,23 +98,37 @@ int passwd_extop(
 
 	if( op->o_bd == NULL ) {
 #ifdef HAVE_CYRUS_SASL
-		return slap_sasl_setpass( op, rs );
+		rc = slap_sasl_setpass( op, rs );
 #else
 		rs->sr_text = "no authz backend";
-		return LDAP_OTHER;
+		rc = LDAP_OTHER;
 #endif
+		goto error_return;
 	}
 
 	if ( op->o_req_ndn.bv_len == 0 ) {
 		rs->sr_text = "no password is associated with the Root DSE";
-		return LDAP_UNWILLING_TO_PERFORM;
+		rc = LDAP_UNWILLING_TO_PERFORM;
+		goto error_return;
+	}
+
+	/* If we've got a glued backend, check the real backend */
+	op_be = op->o_bd;
+	if ( SLAP_GLUE_INSTANCE( op->o_bd )) {
+		op->o_bd = select_backend( &op->o_req_ndn, 0, 0 );
 	}
 
 	if (backend_check_restrictions( op, rs,
 			(struct berval *)&slap_EXOP_MODIFY_PASSWD ) != LDAP_SUCCESS) {
-		return rs->sr_err;
+		rc = rs->sr_err;
+		goto error_return;
 	}
 
+	/* check for referrals */
+	if ( backend_check_referrals( op, rs ) != LDAP_SUCCESS ) {
+		rc = rs->sr_err;
+		goto error_return;
+	}
 
 #ifndef SLAPD_MULTIMASTER
 	/* This does not apply to multi-master case */
@@ -124,12 +145,14 @@ int passwd_extop(
 			} else {
 				rs->sr_ref = defref;
 			}
-			return LDAP_REFERRAL;
+			rc = LDAP_REFERRAL;
+			goto error_return;
 
 		}
 
 		rs->sr_text = "shadow context; no update referral";
-		return LDAP_UNWILLING_TO_PERFORM;
+		rc = LDAP_UNWILLING_TO_PERFORM;
+		goto error_return;
 	}
 #endif /* !SLAPD_MULTIMASTER */
 
@@ -142,22 +165,27 @@ int passwd_extop(
 	}
 	if ( qpw->rs_new.bv_len == 0 ) {
 		rs->sr_text = "password generation failed";
-		return LDAP_OTHER;
+		rc = LDAP_OTHER;
+		goto error_return;
 	}
 
+	op->o_bd = op_be;
+
 	/* Give the backend a chance to handle this itself */
 	if ( op->o_bd->be_extended ) {
 		rs->sr_err = op->o_bd->be_extended( op, rs );
 		if ( rs->sr_err != LDAP_UNWILLING_TO_PERFORM &&
 			rs->sr_err != SLAP_CB_CONTINUE ) {
-			return rs->sr_err;
+			rc = rs->sr_err;
+			goto error_return;
 		}
 	}
 
 	/* The backend didn't handle it, so try it here */
 	if( op->o_bd && !op->o_bd->be_modify ) {
 		rs->sr_text = "operation not supported for current user";
-		return LDAP_UNWILLING_TO_PERFORM;
+		rc = LDAP_UNWILLING_TO_PERFORM;
+		goto error_return;
 	}
 
 	ml = ch_malloc( sizeof(Modifications) );
@@ -185,11 +213,13 @@ int passwd_extop(
 	ml->sml_nvalues = NULL;
 	ml->sml_desc = slap_schema.si_ad_userPassword;
 	ml->sml_op = LDAP_MOD_REPLACE;
+	ml->sml_flags = 0;
 	ml->sml_next = qpw->rs_mods;
 	qpw->rs_mods = ml;
 
 	if ( hashes[i] ) {
 		rs->sr_err = LDAP_OTHER;
+
 	} else {
 		slap_callback *sc = op->o_callback;
 
@@ -218,8 +248,18 @@ int passwd_extop(
 		free( qpw->rs_new.bv_val );
 	}
 
+	rc = rs->sr_err;
 	op->oq_extended = qext;
-	return rs->sr_err;
+
+error_return:;
+	if ( !BER_BVISNULL( &op->o_req_dn ) ) {
+		op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
+	}
+	if ( !BER_BVISNULL( &op->o_req_ndn ) ) {
+		op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
+	}
+
+	return rc;
 }
 
 int slap_passwd_parse( struct berval *reqdata,
@@ -249,13 +289,8 @@ int slap_passwd_parse( struct berval *reqdata,
 	tag = ber_scanf( ber, "{" /*}*/ );
 
 	if( tag == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"slap_passwd_parse: decoding error\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"slap_passwd_parse: decoding error\n", 0, 0, 0 );
-#endif
 		rc = LDAP_PROTOCOL_ERROR;
 		goto done;
 	}
@@ -263,13 +298,8 @@ int slap_passwd_parse( struct berval *reqdata,
 	tag = ber_peek_tag( ber, &len );
 	if( tag == LDAP_TAG_EXOP_MODIFY_PASSWD_ID ) {
 		if( id == NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-			   "slap_passwd_parse: ID not allowed.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: ID not allowed.\n",
 				0, 0, 0 );
-#endif
 
 			*text = "user must change own password";
 			rc = LDAP_UNWILLING_TO_PERFORM;
@@ -279,29 +309,19 @@ int slap_passwd_parse( struct berval *reqdata,
 		tag = ber_scanf( ber, "m", id );
 
 		if( tag == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-			   "slap_passwd_parse:  ID parse failed.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: ID parse failed.\n",
 				0, 0, 0 );
-#endif
 
 			goto decoding_error;
 		}
 
-		tag = ber_peek_tag( ber, &len);
+		tag = ber_peek_tag( ber, &len );
 	}
 
 	if( tag == LDAP_TAG_EXOP_MODIFY_PASSWD_OLD ) {
 		if( oldpass == NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-			   "slap_passwd_parse: OLD not allowed.\n" , 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: OLD not allowed.\n",
 				0, 0, 0 );
-#endif
 
 			*text = "use bind to verify old password";
 			rc = LDAP_UNWILLING_TO_PERFORM;
@@ -311,13 +331,8 @@ int slap_passwd_parse( struct berval *reqdata,
 		tag = ber_scanf( ber, "m", oldpass );
 
 		if( tag == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-			   "slap_passwd_parse:  ID parse failed.\n" , 0, 0, 0 );
-#else
-			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: ID parse failed.\n",
+			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: OLD parse failed.\n",
 				0, 0, 0 );
-#endif
 
 			goto decoding_error;
 		}
@@ -327,13 +342,8 @@ int slap_passwd_parse( struct berval *reqdata,
 
 	if( tag == LDAP_TAG_EXOP_MODIFY_PASSWD_NEW ) {
 		if( newpass == NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-			   "slap_passwd_parse:  NEW not allowed.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: NEW not allowed.\n",
 				0, 0, 0 );
-#endif
 
 			*text = "user specified passwords disallowed";
 			rc = LDAP_UNWILLING_TO_PERFORM;
@@ -343,13 +353,8 @@ int slap_passwd_parse( struct berval *reqdata,
 		tag = ber_scanf( ber, "m", newpass );
 
 		if( tag == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-			   "slap_passwd_parse:  OLD parse failed.\n", 0, 0, 0 );
-#else
-			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: OLD parse failed.\n",
+			Debug( LDAP_DEBUG_TRACE, "slap_passwd_parse: NEW parse failed.\n",
 				0, 0, 0 );
-#endif
 
 			goto decoding_error;
 		}
@@ -359,14 +364,9 @@ int slap_passwd_parse( struct berval *reqdata,
 
 	if( len != 0 ) {
 decoding_error:
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"slap_passwd_parse: decoding error, len=%ld\n", (long)len, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"slap_passwd_parse: decoding error, len=%ld\n",
 			(long) len, 0, 0 );
-#endif
 
 		*text = "data decoding error";
 		rc = LDAP_PROTOCOL_ERROR;
@@ -387,13 +387,8 @@ struct berval * slap_passwd_return(
 
 	assert( cred != NULL );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, 
-		"slap_passwd_return: %ld\n",(long)cred->bv_len, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "slap_passwd_return: %ld\n",
 		(long) cred->bv_len, 0, 0 );
-#endif
 	
 	ber_init_w_nullc( ber, LBER_USE_DER );
 
@@ -409,35 +404,43 @@ struct berval * slap_passwd_return(
 	return bv;
 }
 
+/*
+ * if "e" is provided, access to each value of the password is checked first
+ */
 int
 slap_passwd_check(
-	Connection *conn,
-	Attribute *a,
-	struct berval *cred,
-	const char **text )
+	Operation	*op,
+	Entry		*e,
+	Attribute	*a,
+	struct berval	*cred,
+	const char	**text )
 {
-	int result = 1;
-	struct berval *bv;
+	int			result = 1;
+	struct berval		*bv;
+	AccessControlState	acl_state = ACL_STATE_INIT;
 
-#if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
-	ldap_pvt_thread_mutex_lock( &passwd_mutex );
 #ifdef SLAPD_SPASSWD
-	lutil_passwd_sasl_conn = conn->c_sasl_authctx;
-#endif
+	ldap_pvt_thread_pool_setkey( op->o_threadctx, slap_sasl_bind,
+		op->o_conn->c_sasl_authctx, NULL );
 #endif
 
 	for ( bv = a->a_vals; bv->bv_val != NULL; bv++ ) {
-		if( !lutil_passwd( bv, cred, NULL, text ) ) {
+		/* if e is provided, check access */
+		if ( e && access_allowed( op, e, a->a_desc, bv,
+					ACL_AUTH, &acl_state ) == 0 )
+		{
+			continue;
+		}
+		
+		if ( !lutil_passwd( bv, cred, NULL, text ) ) {
 			result = 0;
 			break;
 		}
 	}
 
-#if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
 #ifdef SLAPD_SPASSWD
-	lutil_passwd_sasl_conn = NULL;
-#endif
-	ldap_pvt_thread_mutex_unlock( &passwd_mutex );
+	ldap_pvt_thread_pool_setkey( op->o_threadctx, slap_sasl_bind,
+		NULL, NULL );
 #endif
 
 	return result;
@@ -446,11 +449,7 @@ slap_passwd_check(
 void
 slap_passwd_generate( struct berval *pass )
 {
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "slap_passwd_generate: begin\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "slap_passwd_generate\n", 0, 0, 0 );
-#endif
 	pass->bv_val = NULL;
 	pass->bv_len = 0;
 
@@ -473,16 +472,7 @@ slap_passwd_hash_type(
 
 	assert( hash );
 
-#if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
-	ldap_pvt_thread_mutex_lock( &passwd_mutex );
-#endif
-
 	lutil_passwd_hash( cred , hash, new, text );
-	
-#if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
-	ldap_pvt_thread_mutex_unlock( &passwd_mutex );
-#endif
-
 }
 void
 slap_passwd_hash(
@@ -500,3 +490,40 @@ slap_passwd_hash(
 
 	slap_passwd_hash_type( cred, new, hash, text );
 }
+
+#ifdef SLAPD_CRYPT
+static ldap_pvt_thread_mutex_t passwd_mutex;
+static lutil_cryptfunc slapd_crypt;
+
+static int slapd_crypt( const char *key, const char *salt, char **hash )
+{
+	char *cr;
+	int rc;
+
+	ldap_pvt_thread_mutex_lock( &passwd_mutex );
+
+	cr = crypt( key, salt );
+	if ( cr == NULL || cr[0] == '\0' ) {
+		/* salt must have been invalid */
+		rc = LUTIL_PASSWD_ERR;
+	} else {
+		if ( hash ) {
+			*hash = ber_strdup( cr );
+			rc = LUTIL_PASSWD_OK;
+		}
+		rc = strcmp( salt, cr ) ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
+	}
+
+	ldap_pvt_thread_mutex_unlock( &passwd_mutex );
+	return rc;
+}
+#endif /* SLAPD_CRYPT */
+
+void slap_passwd_init()
+{
+#ifdef SLAPD_CRYPT
+	ldap_pvt_thread_mutex_init( &passwd_mutex );
+	lutil_cryptptr = slapd_crypt;
+#endif
+}
+
