@@ -1,5 +1,5 @@
 /* schemaparse.c - routines to parse config file objectclass definitions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/schemaparse.c,v 1.64.2.5 2005/01/20 17:01:09 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/schemaparse.c,v 1.71.2.3 2005/05/18 16:29:22 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2005 The OpenLDAP Foundation.
@@ -25,8 +25,6 @@
 #include "slap.h"
 #include "ldap_schema.h"
 
-int	global_schemacheck = 1; /* schemacheck ON is default */
-
 static void		oc_usage(void); 
 static void		at_usage(void);
 
@@ -37,12 +35,14 @@ static char *const err2text[] = {
 	"user-defined ObjectClass includes operational attributes",
 	"user-defined ObjectClass has inappropriate SUPerior",
 	"Duplicate objectClass",
+	"Inconsistent duplicate objectClass",
 	"AttributeType not found",
 	"AttributeType inappropriate matching rule",
 	"AttributeType inappropriate USAGE",
 	"AttributeType inappropriate SUPerior",
 	"AttributeType SYNTAX or SUPerior required",
 	"Duplicate attributeType",
+	"Inconsistent duplicate attributeType",
 	"MatchingRule not found",
 	"MatchingRule incomplete",
 	"Duplicate matchingRule",
@@ -55,7 +55,8 @@ static char *const err2text[] = {
 	"Duplicate Content Rule",
 	"Content Rule not for STRUCTURAL object class",
 	"Content Rule AUX contains inappropriate object class",
-	"Content Rule attribute type list contains duplicate"
+	"Content Rule attribute type list contains duplicate",
+	NULL
 };
 
 char *
@@ -126,8 +127,8 @@ parse_cr(
     const char	*fname,
     int		lineno,
     char	*line,
-    char	**argv
-)
+    char	**argv,
+	ContentRule **scr )
 {
 	LDAPContentRule *cr;
 	int		code;
@@ -149,7 +150,7 @@ parse_cr(
 		return 1;
 	}
 
-	code = cr_add(cr,1,&err);
+	code = cr_add(cr,1,scr,&err);
 	if ( code ) {
 		fprintf( stderr, "%s: line %d: %s: \"%s\"\n",
 			 fname, lineno, scherr2str(code), err);
@@ -165,8 +166,8 @@ parse_oc(
     const char	*fname,
     int		lineno,
     char	*line,
-    char	**argv
-)
+    char	**argv,
+	ObjectClass **soc )
 {
 	LDAPObjectClass *oc;
 	int		code;
@@ -188,7 +189,7 @@ parse_oc(
 		return 1;
 	}
 
-	code = oc_add(oc,1,&err);
+	code = oc_add(oc,1,soc,&err);
 	if ( code ) {
 		fprintf( stderr, "%s: line %d: %s: \"%s\"\n",
 			 fname, lineno, scherr2str(code), err);
@@ -247,8 +248,8 @@ parse_at(
     const char	*fname,
     int		lineno,
     char	*line,
-    char	**argv
-)
+    char	**argv,
+	AttributeType **sat )
 {
 	LDAPAttributeType *at;
 	int		code;
@@ -277,7 +278,7 @@ parse_at(
 		return 1;
 	}
 
-	code = at_add(at,&err);
+	code = at_add(at,1,sat,&err);
 	if ( code ) {
 		fprintf( stderr, "%s: line %d: %s: \"%s\"\n",
 			 fname, lineno, scherr2str(code), err);
