@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/starttls.c,v 1.28.2.2 2004/01/01 18:16:35 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/starttls.c,v 1.36.2.1 2005/01/20 18:03:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -18,8 +18,6 @@
 #include <stdio.h>
 #include <ac/socket.h>
 
-#include <ldap_pvt.h>
-
 #include "slap.h"
 
 #ifdef HAVE_TLS
@@ -61,8 +59,8 @@ starttls_extop ( Operation *op, SlapReply *rs )
 		( op->o_conn->c_dn.bv_len != 0 ) )
 	{
 		Statslog( LDAP_DEBUG_STATS,
-			"conn=%lu op=%lu AUTHZ anonymous mech=starttls ssf=0\n",
-			op->o_connid, op->o_opid, 0, 0, 0 );
+			"%s AUTHZ anonymous mech=starttls ssf=0\n",
+			op->o_log_prefix, 0, 0, 0, 0 );
 
 		/* force to anonymous */
 		connection2anonymous( op->o_conn );
@@ -77,9 +75,7 @@ starttls_extop ( Operation *op, SlapReply *rs )
 	}
 
 	/* fail if TLS could not be initialized */
-	if (ldap_pvt_tls_get_option( NULL, LDAP_OPT_X_TLS_CTX, &ctx ) != 0
-		|| ctx == NULL)
-	{
+	if ( slap_tls_ctx == NULL ) {
 		if (default_referral != NULL) {
 			/* caller will put the referral in the result */
 			rc = LDAP_REFERRAL;
@@ -100,8 +96,7 @@ done:
 	/* give up connection lock */
 	ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 
-	/*
-	 * RACE CONDITION: we give up lock before sending result
+	/* FIXME: RACE CONDITION! we give up lock before sending result
 	 * Should be resolved by reworking connection state, not
 	 * by moving send here (so as to ensure proper TLS sequencing)
 	 */
