@@ -1,8 +1,17 @@
 /* syntax.c - routines to manage syntax definitions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/syntax.c,v 1.21.2.6 2002/11/11 02:42:37 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/syntax.c,v 1.40.2.1 2005/01/20 18:03:56 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2005 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -14,7 +23,6 @@
 #include <ac/socket.h>
 
 #include "slap.h"
-#include "ldap_pvt.h"
 
 struct sindexrec {
 	char		*sir_name;
@@ -22,24 +30,26 @@ struct sindexrec {
 };
 
 static Avlnode	*syn_index = NULL;
-static Syntax *syn_list = NULL;
+static LDAP_SLIST_HEAD(SyntaxList, slap_syntax) syn_list
+	= LDAP_SLIST_HEAD_INITIALIZER(&syn_list);
 
 static int
 syn_index_cmp(
-    struct sindexrec	*sir1,
-    struct sindexrec	*sir2
+	const void *v_sir1,
+	const void *v_sir2
 )
 {
+	const struct sindexrec *sir1 = v_sir1, *sir2 = v_sir2;
 	return (strcmp( sir1->sir_name, sir2->sir_name ));
 }
 
 static int
 syn_index_name_cmp(
-    const char		*name,
-    struct sindexrec	*sir
+	const void *name,
+	const void *sir
 )
 {
-	return (strcmp( name, sir->sir_name ));
+	return (strcmp( name, ((const struct sindexrec *)sir)->sir_name ));
 }
 
 Syntax *
@@ -47,8 +57,7 @@ syn_find( const char *synname )
 {
 	struct sindexrec	*sir = NULL;
 
-	if ( (sir = (struct sindexrec *) avl_find( syn_index, synname,
-	    (AVL_CMP) syn_index_name_cmp )) != NULL ) {
+	if ( (sir = avl_find( syn_index, synname, syn_index_name_cmp )) != NULL ) {
 		return( sir->sir_syn );
 	}
 	return( NULL );
@@ -59,20 +68,23 @@ syn_find_desc( const char *syndesc, int *len )
 {
 	Syntax		*synp;
 
-	for (synp = syn_list; synp; synp = synp->ssyn_next)
-		if ((*len = dscompare( synp->ssyn_syn.syn_desc, syndesc, '{')))
+	LDAP_SLIST_FOREACH(synp, &syn_list, ssyn_next) {
+		if ((*len = dscompare( synp->ssyn_syn.syn_desc, syndesc, '{' /*'}'*/ ))) {
 			return synp;
+		}
+	}
 	return( NULL );
 }
 
 void
 syn_destroy( void )
 {
-	Syntax *s, *n;
+	Syntax *s;
 
 	avl_free(syn_index, ldap_memfree);
-	for (s=syn_list; s; s=n) {
-		n = s->ssyn_next;
+	while( !LDAP_SLIST_EMPTY(&syn_list) ) {
+		s = LDAP_SLIST_FIRST(&syn_list);
+		LDAP_SLIST_REMOVE_HEAD(&syn_list, ssyn_next);
 		ldap_syntax_free((LDAPSyntax *)s);
 	}
 }
@@ -83,32 +95,22 @@ syn_insert(
     const char		**err
 )
 {
-	Syntax		**synp;
 	struct sindexrec	*sir;
 
-	synp = &syn_list;
-	while ( *synp != NULL ) {
-		synp = &(*synp)->ssyn_next;
-	}
-	*synp = ssyn;
-
+	LDAP_SLIST_NEXT( ssyn, ssyn_next ) = NULL;
+	LDAP_SLIST_INSERT_HEAD( &syn_list, ssyn, ssyn_next );
+ 
 	if ( ssyn->ssyn_oid ) {
 		sir = (struct sindexrec *)
 			SLAP_CALLOC( 1, sizeof(struct sindexrec) );
 		if( sir == NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR, 
-				"syn_insert: SLAP_CALLOC Error\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "SLAP_CALLOC Error\n", 0, 0, 0 );
-#endif
 			return LDAP_OTHER;
 		}
 		sir->sir_name = ssyn->ssyn_oid;
 		sir->sir_syn = ssyn;
 		if ( avl_insert( &syn_index, (caddr_t) sir,
-				 (AVL_CMP) syn_index_cmp,
-				 (AVL_DUP) avl_dup_error ) ) {
+		                 syn_index_cmp, avl_dup_error ) ) {
 			*err = ssyn->ssyn_oid;
 			ldap_memfree(sir);
 			return SLAP_SCHERR_SYN_DUP;
@@ -131,18 +133,13 @@ syn_add(
 
 	ssyn = (Syntax *) SLAP_CALLOC( 1, sizeof(Syntax) );
 	if( ssyn == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"syn_add: SLAP_CALLOC Error\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "SLAP_CALLOC Error\n", 0, 0, 0 );
-#endif
 		return LDAP_OTHER;
 	}
 
 	AC_MEMCPY( &ssyn->ssyn_syn, syn, sizeof(LDAPSyntax) );
 
-	ssyn->ssyn_next = NULL;
+	LDAP_SLIST_NEXT(ssyn,ssyn_next) = NULL;
 
 	/*
 	 * note: ssyn_bvoid uses the same memory of ssyn_syn.syn_oid;
@@ -152,7 +149,6 @@ syn_add(
 	ssyn->ssyn_oidlen = strlen(syn->syn_oid);
 	ssyn->ssyn_flags = def->sd_flags;
 	ssyn->ssyn_validate = def->sd_validate;
-	ssyn->ssyn_normalize = def->sd_normalize;
 	ssyn->ssyn_pretty = def->sd_pretty;
 
 #ifdef SLAPD_BINARY_CONVERSION
@@ -174,14 +170,8 @@ register_syntax(
 
 	syn = ldap_str2syntax( def->sd_desc, &code, &err, LDAP_SCHEMA_ALLOW_ALL);
 	if ( !syn ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( CONFIG, ERR, 
-			"register_syntax: Error - %s before %s in %s.\n",
-			ldap_scherr2str(code), err, def->sd_desc );
-#else
 		Debug( LDAP_DEBUG_ANY, "Error in register_syntax: %s before %s in %s\n",
 		    ldap_scherr2str(code), err, def->sd_desc );
-#endif
 
 		return( -1 );
 	}
@@ -191,14 +181,8 @@ register_syntax(
 	ldap_memfree( syn );
 
 	if ( code ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( CONFIG, ERR, 
-			"register_syntax: Error - %s %s in %s\n", 
-			scherr2str(code), err, def->sd_desc );
-#else
 		Debug( LDAP_DEBUG_ANY, "Error in register_syntax: %s %s in %s\n",
 		    scherr2str(code), err, def->sd_desc );
-#endif
 
 		return( -1 );
 	}
@@ -209,14 +193,12 @@ register_syntax(
 int
 syn_schema_info( Entry *e )
 {
-	struct berval	vals[2];
-	Syntax		*syn;
-
 	AttributeDescription *ad_ldapSyntaxes = slap_schema.si_ad_ldapSyntaxes;
+	Syntax		*syn;
+	struct berval	val;
+	struct berval	nval;
 
-	vals[1].bv_val = NULL;
-
-	for ( syn = syn_list; syn; syn = syn->ssyn_next ) {
+	LDAP_SLIST_FOREACH(syn, &syn_list, ssyn_next ) {
 		if ( ! syn->ssyn_validate ) {
 			/* skip syntaxes without validators */
 			continue;
@@ -226,23 +208,22 @@ syn_schema_info( Entry *e )
 			continue;
 		}
 
-		if ( ldap_syntax2bv( &syn->ssyn_syn, vals ) == NULL ) {
+		if ( ldap_syntax2bv( &syn->ssyn_syn, &val ) == NULL ) {
 			return -1;
 		}
 #if 0
-#ifdef NEW_LOGGING
-		LDAP_LOG( config, ENTRY,
-			   "syn_schema_info: Merging syn [%ld] %s\n",
-			   (long)vals[0].bv_len, vals[0].bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "Merging syn [%ld] %s\n",
-	       (long) vals[0].bv_len, vals[0].bv_val, 0 );
+	       (long) val.bv_len, val.bv_val, 0 );
 #endif
 
-#endif
-		if( attr_merge( e, ad_ldapSyntaxes, vals ) )
+		nval.bv_val = syn->ssyn_oid;
+		nval.bv_len = strlen(syn->ssyn_oid);
+
+		if( attr_merge_one( e, ad_ldapSyntaxes, &val, &nval ) )
+		{
 			return -1;
-		ldap_memfree( vals[0].bv_val );
+		}
+		ldap_memfree( val.bv_val );
 	}
 	return 0;
 }
